import { BlockchainOperator } from "./interfaces/BlockchainOperator";
export declare class BasicBlockchain {
    private op;
    constructor(op: BlockchainOperator);
    /**
     * Adds a block at the end of the blockchain.
     */
    pushBlock(b: any): Promise<any>;
    /**
     * Get the block identified by `number`
     * @param number block ID.
     * @returns {*} Promise<Block>
     */
    getBlock(number: number): Promise<any>;
    /**
     * Get the nth block from the top of the blockchain.
     * @param index Index from top. Defaults to `0`. E.g. `0` = HEAD, `1` = HEAD~1, etc.
     * @returns {*} Promise<Block>
     */
    head(index?: number): Promise<any>;
    /**
     * Blockchain size, in number of blocks.
     * @returns {*} Size.
     */
    height(): Promise<number>;
    /**
     * Get the (n+1)th blocks top blocks of the blockchain, ordered by number ascending.
     * @param n Quantity from top. E.g. `1` = [HEAD], `3` = [HEAD, HEAD~1, HEAD~2], etc.
     * @returns {*} Promise<Block>
     */
    headRange(n: number): Promise<any[]>;
    /**
     * Pops the blockchain HEAD.
     * @returns {*} Promise<Block> The reverted block.
     */
    revertHead(): Promise<any>;
}
