"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const BasicBlockchain_1 = require("./BasicBlockchain");
const _ = require('underscore');
class IndexedBlockchain extends BasicBlockchain_1.BasicBlockchain {
    constructor(bcOperations, indexOperations, numberField, pkFields) {
        super(bcOperations);
        this.indexOperations = indexOperations;
        this.numberField = numberField;
        this.pkFields = pkFields;
        this.initIndexer = indexOperations.initIndexer(pkFields);
    }
    recordIndex(index) {
        return __awaiter(this, void 0, void 0, function* () {
            // Wait indexer init
            yield this.initIndexer;
            return this.indexOperations.recordIndex(index);
        });
    }
    indexTrim(maxNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            // Wait indexer init
            yield this.initIndexer;
            const subIndexes = yield this.indexOperations.getSubIndexes();
            // Trim the subIndexes
            const records = {};
            for (const subIndex of subIndexes) {
                records[subIndex] = [];
                const pks = typeof this.pkFields[subIndex].pk !== 'string' && this.pkFields[subIndex].pk.length ? Array.from(this.pkFields[subIndex].pk) : [this.pkFields[subIndex].pk];
                const rm = this.pkFields[subIndex].remove;
                let potentialRecords = yield this.indexOperations.findTrimable(subIndex, this.numberField, maxNumber);
                potentialRecords = reduceBy(potentialRecords, pks);
                for (const potential of potentialRecords) {
                    const subCriteriasRowsToDelete = criteriasFromPks(pks, potential);
                    subCriteriasRowsToDelete[this.numberField] = { $lt: maxNumber };
                    const rowsToReduce = yield this.indexOperations.findWhere(subIndex, subCriteriasRowsToDelete);
                    // No reduction if 1 line to delete
                    if (rowsToReduce.length > 1) {
                        const reduced = reduce(rowsToReduce);
                        const subCriteriasRowsToKeep = criteriasFromPks(pks, potential);
                        subCriteriasRowsToKeep[this.numberField] = { $gte: maxNumber };
                        const toKeep = yield this.indexOperations.findWhere(subIndex, subCriteriasRowsToKeep);
                        const subCriteriasAllRowsOfObject = criteriasFromPks(pks, potential);
                        yield this.indexOperations.removeWhere(subIndex, subCriteriasAllRowsOfObject);
                        // Add the reduced row + rows to keep
                        if (!rm || !reduced[rm]) {
                            records[subIndex] = records[subIndex].concat([reduced]).concat(toKeep);
                        }
                    }
                }
            }
            yield this.recordIndex(records);
            return Promise.resolve();
        });
    }
    indexCount(indexName, criterias) {
        return __awaiter(this, void 0, void 0, function* () {
            // Wait indexer init
            yield this.initIndexer;
            const records = yield this.indexOperations.findWhere(indexName, criterias);
            return records.length;
        });
    }
    indexReduce(indexName, criterias) {
        return __awaiter(this, void 0, void 0, function* () {
            // Wait indexer init
            yield this.initIndexer;
            const records = yield this.indexOperations.findWhere(indexName, criterias);
            return reduce(records);
        });
    }
    indexReduceGroupBy(indexName, criterias, properties) {
        return __awaiter(this, void 0, void 0, function* () {
            // Wait indexer init
            yield this.initIndexer;
            const records = yield this.indexOperations.findWhere(indexName, criterias);
            return reduceBy(records, properties);
        });
    }
    indexRevert(blockNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            const subIndexes = yield this.indexOperations.getSubIndexes();
            for (const subIndex of subIndexes) {
                const removeCriterias = {};
                removeCriterias[this.numberField] = blockNumber;
                yield this.indexOperations.removeWhere(subIndex, removeCriterias);
            }
        });
    }
}
exports.IndexedBlockchain = IndexedBlockchain;
function reduce(records) {
    return records.reduce((obj, record) => {
        const keys = Object.keys(record);
        for (const k of keys) {
            if (record[k] !== undefined && record[k] !== null) {
                obj[k] = record[k];
            }
        }
        return obj;
    }, {});
}
function reduceBy(reducables, properties) {
    const reduced = reducables.reduce((map, entry) => {
        const id = properties.map((prop) => entry[prop]).join('-');
        map[id] = map[id] || [];
        map[id].push(entry);
        return map;
    }, {});
    return _.values(reduced).map((rows) => reduce(rows));
}
function criteriasFromPks(pks, values) {
    const criterias = {};
    for (const key of pks) {
        criterias[key] = values[key];
    }
    return criterias;
}
//# sourceMappingURL=IndexedBlockchain.js.map