"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const indexer = require('../../lib/indexer').Indexer;
class SQLBlockchain {
    constructor(dal) {
        this.dal = dal;
    }
    store(b) {
        return this.dal.bindexDAL.saveEntity(b);
    }
    read(i) {
        return this.dal.bindexDAL.sqlFindOne({ number: i });
    }
    head(n = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            /**
             * IMPORTANT NOTICE
             * ----------------
             *
             * There is a difference between the protocol's HEAD (P_HEAD) and the database's HEAD (DB_HEAD). The relation is:
             *
             *     DB_HEAD~<i> = P_HEAD~<i+1>
             *
             * In this class methods, we expose DB_HEAD logic. But the data is stored/retrieved by DAL objects using P_HEAD logic.
             *
             * So if we want DB_HEAD~0 for example, we must ask P_HEAD~(0+1). The DAL object will then retrieve P_HEAD~1, which
             * is the latest stored block in the blockchain.
             *
             * Note: the DAL object cannot retrieve P_HEAD~0, this value does not exist and refers to the potential incoming block.
             *
             * Why this behavior?
             * ------------------
             *
             * Because the DAL was wrongly coded. It will be easy to fix this problem once indexer.js will only use **this class'
             * methods**. Then, we will be able to replace (n + 1) by just (n), and replace also the complementary behavior in
             * the DAL (BIndexDAL).
             */
            return this.dal.bindexDAL.head(n + 1);
        });
    }
    height() {
        return __awaiter(this, void 0, void 0, function* () {
            const head = yield this.dal.bindexDAL.head(1); // We do not use head(0). See the above notice.
            if (head) {
                return head.number + 1;
            }
            else {
                return 0;
            }
        });
    }
    headRange(m) {
        return this.dal.bindexDAL.range(1, m); // We do not use range(0, m). See the above notice.
    }
    revertHead() {
        return __awaiter(this, void 0, void 0, function* () {
            const head = yield this.dal.bindexDAL.head(1); // We do not use head(0). See the above notice.
            yield this.dal.bindexDAL.removeBlock(head.number);
            return head;
        });
    }
}
exports.SQLBlockchain = SQLBlockchain;
//# sourceMappingURL=SqlBlockchain.js.map