"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractIndex_1 = require("../dal/sqliteDAL/AbstractIndex");
const _ = require('underscore');
class SQLIndex {
    constructor(db, definitions) {
        this.db = db;
        this.definitions = definitions;
        this.indexes = {};
    }
    initIndexer(pkFields) {
        return __awaiter(this, void 0, void 0, function* () {
            const keys = _.keys(pkFields);
            for (const k of keys) {
                if (this.definitions[k].handler) {
                    // External table: managed by another object
                    this.indexes[k] = this.definitions[k].handler;
                }
                else {
                    // Internal table: managed here
                    const indexTable = new AbstractIndex_1.AbstractIndex(this.db, k, [], this.definitions[k].fields, [], this.definitions[k].booleans);
                    this.indexes[k] = indexTable;
                    indexTable.init = () => {
                        return indexTable.exec('BEGIN;' +
                            'CREATE TABLE IF NOT EXISTS ' + indexTable.table + ' (' +
                            this.definitions[k].sqlFields.join(',') +
                            ');' +
                            'COMMIT;');
                    };
                    yield indexTable.init();
                }
            }
        });
    }
    getSubIndexes() {
        return Promise.resolve(_.keys(this.indexes));
    }
    findTrimable(subIndex, numberField, maxNumber) {
        if (this.definitions[subIndex].findTrimable) {
            return this.definitions[subIndex].findTrimable(maxNumber);
        }
        else {
            const criterias = {};
            criterias[numberField] = { $lt: maxNumber };
            return this.indexes[subIndex].sqlFind(criterias);
        }
    }
    removeWhere(subIndex, criterias) {
        if (!this.indexes[subIndex]) {
            return Promise.resolve();
        }
        return this.indexes[subIndex].sqlRemoveWhere(criterias);
    }
    recordIndex(index) {
        return __awaiter(this, void 0, void 0, function* () {
            const subIndexes = _.keys(index);
            // Feed the this.indexes
            for (const subIndex of subIndexes) {
                yield this.indexes[subIndex].insertBatch(index[subIndex]);
            }
            return Promise.resolve();
        });
    }
    findWhere(subIndex, criterias) {
        if (!this.indexes[subIndex]) {
            return Promise.resolve([]);
        }
        return this.indexes[subIndex].sqlFind(criterias);
    }
}
exports.SQLIndex = SQLIndex;
//# sourceMappingURL=SqlIndex.js.map