"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../../../lib/common-libs/constants");
const GenericParser_1 = require("./GenericParser");
const index_1 = require("../../../lib/common-libs/index");
class CertificationParser extends GenericParser_1.GenericParser {
    constructor() {
        super([
            { prop: "version", regexp: constants_1.CommonConstants.DOCUMENTS.DOC_VERSION },
            { prop: "type", regexp: constants_1.CommonConstants.CERTIFICATION.CERT_TYPE },
            { prop: "currency", regexp: constants_1.CommonConstants.DOCUMENTS.DOC_CURRENCY },
            { prop: "issuer", regexp: constants_1.CommonConstants.DOCUMENTS.DOC_ISSUER },
            { prop: "idty_issuer", regexp: constants_1.CommonConstants.CERTIFICATION.IDTY_ISSUER },
            { prop: "idty_sig", regexp: constants_1.CommonConstants.CERTIFICATION.IDTY_SIG },
            { prop: "idty_buid", regexp: constants_1.CommonConstants.CERTIFICATION.IDTY_TIMESTAMP },
            { prop: "idty_uid", regexp: constants_1.CommonConstants.CERTIFICATION.IDTY_UID },
            { prop: "buid", regexp: constants_1.CommonConstants.CERTIFICATION.CERT_TIMESTAMP }
        ], index_1.rawer.getOfficialCertification);
    }
    _clean(obj) {
        obj.sig = obj.signature;
        obj.block = obj.buid;
        if (obj.block) {
            obj.number = obj.block.split('-')[0];
            obj.fpr = obj.block.split('-')[1];
        }
        else {
            obj.number = '0';
            obj.fpr = '';
        }
    }
    _verify(obj) {
        return ["version", "type", "currency", "issuer", "idty_issuer", "idty_sig", "idty_buid", "idty_uid", "block"].reduce((p, field) => {
            return p || (!obj[field] && "Wrong format for certification") || "";
        }, "");
    }
}
exports.CertificationParser = CertificationParser;
//# sourceMappingURL=certification.js.map