"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const GenericParser_1 = require("./GenericParser");
const constants_1 = require("../../../lib/common-libs/constants");
const common_1 = require("../../../lib/common");
const index_1 = require("../../../lib/common-libs/index");
class IdentityParser extends GenericParser_1.GenericParser {
    constructor() {
        super([
            { prop: "version", regexp: constants_1.CommonConstants.DOCUMENTS.DOC_VERSION },
            { prop: "type", regexp: constants_1.CommonConstants.IDENTITY.IDTY_TYPE },
            { prop: "currency", regexp: constants_1.CommonConstants.DOCUMENTS.DOC_CURRENCY },
            { prop: "pubkey", regexp: constants_1.CommonConstants.DOCUMENTS.DOC_ISSUER },
            { prop: "uid", regexp: constants_1.CommonConstants.IDENTITY.IDTY_UID },
            { prop: "buid", regexp: constants_1.CommonConstants.DOCUMENTS.TIMESTAMP }
        ], index_1.rawer.getOfficialIdentity);
    }
    _clean(obj) {
        obj.sig = obj.signature;
        if (obj.uid && obj.buid && obj.pubkey) {
            obj.hash = common_1.hashf(obj.uid + obj.buid + obj.pubkey).toUpperCase();
        }
    }
    _verify(obj) {
        if (!obj.pubkey) {
            return "No pubkey found";
        }
        if (!obj.uid) {
            return "Wrong user id format";
        }
        if (!obj.buid) {
            return "Could not extract block uid";
        }
        if (!obj.sig) {
            return "No signature found for self-certification";
        }
        return "";
    }
}
exports.IdentityParser = IdentityParser;
//# sourceMappingURL=identity.js.map