"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../../../lib/common-libs/constants");
const GenericParser_1 = require("./GenericParser");
const index_1 = require("../../../lib/common-libs/index");
const buid_1 = require("../../../lib/common-libs/buid");
class MembershipParser extends GenericParser_1.GenericParser {
    constructor() {
        super([
            { prop: "version", regexp: constants_1.CommonConstants.MEMBERSHIP.VERSION },
            { prop: "currency", regexp: constants_1.CommonConstants.MEMBERSHIP.CURRENCY },
            { prop: "issuer", regexp: constants_1.CommonConstants.MEMBERSHIP.ISSUER },
            { prop: "membership", regexp: constants_1.CommonConstants.MEMBERSHIP.MEMBERSHIP },
            { prop: "userid", regexp: constants_1.CommonConstants.MEMBERSHIP.USERID },
            { prop: "block", regexp: constants_1.CommonConstants.MEMBERSHIP.BLOCK },
            { prop: "certts", regexp: constants_1.CommonConstants.MEMBERSHIP.CERTTS }
        ], index_1.rawer.getMembership);
    }
    _clean(obj) {
        if (obj.block) {
            obj.number = obj.block.split('-')[0];
            obj.fpr = obj.block.split('-')[1];
        }
        else {
            obj.number = '0';
            obj.fpr = '';
        }
    }
    _verify(obj) {
        let err = null;
        const codes = {
            'BAD_VERSION': 150,
            'BAD_CURRENCY': 151,
            'BAD_ISSUER': 152,
            'BAD_MEMBERSHIP': 153,
            'BAD_REGISTRY_TYPE': 154,
            'BAD_BLOCK': 155,
            'BAD_USERID': 156,
            'BAD_CERTTS': 157
        };
        if (!err) {
            if (!obj.version || !obj.version.match(constants_1.CommonConstants.DOCUMENTS_VERSION_REGEXP))
                err = { code: codes.BAD_VERSION, message: "Version unknown" };
        }
        if (!err) {
            if (obj.issuer && !obj.issuer.match(constants_1.CommonConstants.BASE58))
                err = { code: codes.BAD_ISSUER, message: "Incorrect issuer field" };
        }
        if (!err) {
            if (!(obj.membership || "").match(/^(IN|OUT)$/))
                err = { code: codes.BAD_MEMBERSHIP, message: "Incorrect Membership field: must be either IN or OUT" };
        }
        if (!err) {
            if (obj.block && !obj.block.match(constants_1.CommonConstants.BLOCK_UID))
                err = { code: codes.BAD_BLOCK, message: "Incorrect Block field: must be a positive or zero integer, a dash and an uppercased SHA1 hash" };
        }
        if (!err) {
            if (obj.userid && !obj.userid.match(constants_1.CommonConstants.USER_ID))
                err = { code: codes.BAD_USERID, message: "UserID must match udid2 format" };
        }
        if (!err) {
            if (!buid_1.Buid.format.isBuid(obj.certts))
                err = { code: codes.BAD_CERTTS, message: "CertTS must be a valid timestamp" };
        }
        return err && err.message;
    }
    ;
}
exports.MembershipParser = MembershipParser;
//# sourceMappingURL=membership.js.map