"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../../../lib/common-libs/constants");
const GenericParser_1 = require("./GenericParser");
const common_1 = require("../../../lib/common");
const index_1 = require("../../../lib/common-libs/index");
class RevocationParser extends GenericParser_1.GenericParser {
    constructor() {
        super([
            { prop: "version", regexp: constants_1.CommonConstants.DOCUMENTS.DOC_VERSION },
            { prop: "type", regexp: constants_1.CommonConstants.REVOCATION.REVOC_TYPE },
            { prop: "currency", regexp: constants_1.CommonConstants.DOCUMENTS.DOC_CURRENCY },
            { prop: "issuer", regexp: constants_1.CommonConstants.DOCUMENTS.DOC_ISSUER },
            { prop: "sig", regexp: constants_1.CommonConstants.REVOCATION.IDTY_SIG },
            { prop: "buid", regexp: constants_1.CommonConstants.REVOCATION.IDTY_TIMESTAMP },
            { prop: "uid", regexp: constants_1.CommonConstants.REVOCATION.IDTY_UID }
        ], index_1.rawer.getOfficialRevocation);
    }
    _clean(obj) {
        obj.pubkey = obj.issuer;
        obj.revocation = obj.signature;
        if (obj.uid && obj.buid && obj.pubkey) {
            obj.hash = common_1.hashf(obj.uid + obj.buid + obj.pubkey).toUpperCase();
        }
    }
    _verify(obj) {
        if (!obj.pubkey) {
            return "No pubkey found";
        }
        if (!obj.uid) {
            return "Wrong user id format";
        }
        if (!obj.buid) {
            return "Could not extract block uid";
        }
        if (!obj.sig) {
            return "No signature found for identity";
        }
        if (!obj.revocation) {
            return "No revocation signature found";
        }
        return "";
    }
}
exports.RevocationParser = RevocationParser;
//# sourceMappingURL=revocation.js.map