import { BlockDTO } from "../dto/BlockDTO";
import { DuniterBlockchain } from "../blockchain/DuniterBlockchain";
import { QuickSynchronizer } from "./QuickSync";
export declare class BlockchainContext {
    private conf;
    private dal;
    private logger;
    private blockchain;
    private quickSynchronizer;
    /**
     * The virtual next HEAD. Computed each time a new block is added, because a lot of HEAD variables are deterministic
     * and can be computed one, just after a block is added for later controls.
     */
    private vHEAD;
    /**
     * The currently written HEAD, aka. HEAD_1 relatively to incoming HEAD.
     */
    private vHEAD_1;
    private HEADrefreshed;
    /**
     * Refresh the virtual HEAD value for determined variables of the next coming block, avoiding to recompute them
     * each time a new block arrives to check if the values are correct. We can know and store them early on, in vHEAD.
     */
    private refreshHead();
    /**
     * Gets a copy of vHEAD, extended with some extra properties.
     * @param props The extra properties to add.
     */
    getvHeadCopy(props?: any): Promise<any>;
    /**
     * Get currently written HEAD.
     */
    getvHEAD_1(): Promise<any>;
    /**
     * Utility method: gives the personalized difficulty level of a given issuer for next block.
     * @param issuer The issuer we want to get the difficulty level.
     */
    getIssuerPersonalizedDifficulty(issuer: string): Promise<any>;
    setConfDAL(newConf: any, newDAL: any, theBlockchain: DuniterBlockchain, theQuickSynchronizer: QuickSynchronizer): void;
    checkBlock(block: BlockDTO, withPoWAndSignature: boolean): Promise<any>;
    private addBlock(obj, index?, HEAD?);
    addSideBlock(obj: BlockDTO): Promise<BlockDTO>;
    revertCurrentBlock(): Promise<BlockDTO>;
    applyNextAvailableFork(): Promise<any>;
    checkAndAddBlock(block: BlockDTO): Promise<any>;
    current(): Promise<any>;
    checkHaveEnoughLinks(target: string, newLinks: any): Promise<any>;
    quickApplyBlocks(blocks: BlockDTO[], to: number): Promise<any>;
}
