"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const DuniterBlockchain_1 = require("../blockchain/DuniterBlockchain");
const BlockDTO_1 = require("../dto/BlockDTO");
const DBTransaction_1 = require("../db/DBTransaction");
const indexer_1 = require("../indexer");
const _ = require('underscore');
const constants = require('../constants');
let sync_bindex = [];
let sync_iindex = [];
let sync_mindex = [];
let sync_cindex = [];
let sync_sindex = [];
let sync_bindexSize = 0;
let sync_allBlocks = [];
let sync_expires = [];
let sync_nextExpiring = 0;
let sync_currConf;
const sync_memoryWallets = {};
const sync_memoryDAL = {
    getWallet: (conditions) => Promise.resolve(sync_memoryWallets[conditions] || { conditions, balance: 0 }),
    saveWallet: (wallet) => __awaiter(this, void 0, void 0, function* () {
        // Make a copy
        sync_memoryWallets[wallet.conditions] = {
            conditions: wallet.conditions,
            balance: wallet.balance
        };
    }),
    sindexDAL: {
        getAvailableForConditions: (conditions) => null
    }
};
class QuickSynchronizer {
    constructor(blockchain, conf, dal, logger) {
        this.blockchain = blockchain;
        this.conf = conf;
        this.dal = dal;
        this.logger = logger;
    }
    saveBlocksInMainBranch(blocks) {
        return __awaiter(this, void 0, void 0, function* () {
            // VERY FIRST: parameters, otherwise we compute wrong variables such as UDTime
            if (blocks[0].number == 0) {
                yield this.blockchain.saveParametersForRoot(blocks[0], this.conf, this.dal);
            }
            // Helper to retrieve a block with local cache
            const getBlock = (number) => {
                const firstLocalNumber = blocks[0].number;
                if (number >= firstLocalNumber) {
                    let offset = number - firstLocalNumber;
                    return Promise.resolve(blocks[offset]);
                }
                return this.dal.getBlock(number);
            };
            const getBlockByNumberAndHash = (number, hash) => __awaiter(this, void 0, void 0, function* () {
                const block = yield getBlock(number);
                if (!block || block.hash != hash) {
                    throw 'Block #' + [number, hash].join('-') + ' not found neither in DB nor in applying blocks';
                }
                return block;
            });
            for (const block of blocks) {
                block.fork = false;
                const current = yield getBlock(block.number - 1);
                this.blockchain.updateBlocksComputedVars(current, block);
            }
            // Transactions recording
            yield this.updateTransactionsForBlocks(blocks, getBlockByNumberAndHash);
            yield this.dal.blockDAL.saveBunch(blocks);
            yield DuniterBlockchain_1.DuniterBlockchain.pushStatsForBlocks(blocks, this.dal);
        });
    }
    updateTransactionsForBlocks(blocks, getBlockByNumberAndHash) {
        return __awaiter(this, void 0, void 0, function* () {
            let txs = [];
            for (const block of blocks) {
                const newOnes = [];
                for (const tx of block.transactions) {
                    const [number, hash] = tx.blockstamp.split('-');
                    const refBlock = (yield getBlockByNumberAndHash(parseInt(number), hash));
                    // We force the usage of the reference block's currency
                    tx.currency = refBlock.currency;
                    tx.hash = tx.getHash();
                    const dbTx = DBTransaction_1.DBTransaction.fromTransactionDTO(tx, refBlock.medianTime, true, false, refBlock.number, refBlock.medianTime);
                    newOnes.push(dbTx);
                }
                txs = txs.concat(newOnes);
            }
            return this.dal.updateTransactions(txs);
        });
    }
    quickApplyBlocks(blocks, to) {
        return __awaiter(this, void 0, void 0, function* () {
            sync_memoryDAL.sindexDAL = { getAvailableForConditions: (conditions) => this.dal.sindexDAL.getAvailableForConditions(conditions) };
            let blocksToSave = [];
            for (const block of blocks) {
                sync_allBlocks.push(block);
                // The new kind of object stored
                const dto = BlockDTO_1.BlockDTO.fromJSONObject(block);
                if (block.number == 0) {
                    sync_currConf = BlockDTO_1.BlockDTO.getConf(block);
                }
                if (block.number <= to - this.conf.forksize) {
                    blocksToSave.push(dto);
                    const index = indexer_1.Indexer.localIndex(dto, sync_currConf);
                    const local_iindex = indexer_1.Indexer.iindex(index);
                    const local_cindex = indexer_1.Indexer.cindex(index);
                    const local_sindex = indexer_1.Indexer.sindex(index);
                    const local_mindex = indexer_1.Indexer.mindex(index);
                    sync_iindex = sync_iindex.concat(local_iindex);
                    sync_cindex = sync_cindex.concat(local_cindex);
                    sync_mindex = sync_mindex.concat(local_mindex);
                    const HEAD = yield indexer_1.Indexer.quickCompleteGlobalScope(block, sync_currConf, sync_bindex, sync_iindex, sync_mindex, sync_cindex, {
                        getBlock: (number) => {
                            return Promise.resolve(sync_allBlocks[number]);
                        },
                        getBlockByBlockstamp: (blockstamp) => {
                            return Promise.resolve(sync_allBlocks[parseInt(blockstamp)]);
                        }
                    });
                    sync_bindex.push(HEAD);
                    // Remember expiration dates
                    for (const entry of index) {
                        if (entry.expires_on || entry.revokes_on) {
                            sync_expires.push(entry.expires_on || entry.revokes_on);
                        }
                    }
                    sync_expires = _.uniq(sync_expires);
                    yield this.blockchain.createNewcomers(local_iindex, this.dal, this.logger);
                    if (block.dividend
                        || block.joiners.length
                        || block.actives.length
                        || block.revoked.length
                        || block.excluded.length
                        || block.certifications.length
                        || block.transactions.length
                        || block.medianTime >= sync_nextExpiring) {
                        // logger.warn('>> Block#%s', block.number)
                        for (let i = 0; i < sync_expires.length; i++) {
                            let expire = sync_expires[i];
                            if (block.medianTime > expire) {
                                sync_expires.splice(i, 1);
                                i--;
                            }
                        }
                        let currentNextExpiring = sync_nextExpiring;
                        sync_nextExpiring = sync_expires.reduce((max, value) => max ? Math.min(max, value) : value, sync_nextExpiring);
                        const nextExpiringChanged = currentNextExpiring !== sync_nextExpiring;
                        // Fills in correctly the SINDEX
                        yield Promise.all(_.where(sync_sindex.concat(local_sindex), { op: 'UPDATE' }).map((entry) => __awaiter(this, void 0, void 0, function* () {
                            if (!entry.conditions) {
                                const src = yield this.dal.sindexDAL.getSource(entry.identifier, entry.pos);
                                entry.conditions = src.conditions;
                            }
                        })));
                        // Flush the INDEX (not bindex, which is particular)
                        yield this.dal.mindexDAL.insertBatch(sync_mindex);
                        yield this.dal.iindexDAL.insertBatch(sync_iindex);
                        yield this.dal.sindexDAL.insertBatch(sync_sindex);
                        yield this.dal.cindexDAL.insertBatch(sync_cindex);
                        sync_mindex = [];
                        sync_iindex = [];
                        sync_cindex = [];
                        sync_sindex = local_sindex;
                        sync_sindex = sync_sindex.concat(yield indexer_1.Indexer.ruleIndexGenDividend(HEAD, this.dal));
                        sync_sindex = sync_sindex.concat(yield indexer_1.Indexer.ruleIndexGarbageSmallAccounts(HEAD, sync_sindex, sync_memoryDAL));
                        if (nextExpiringChanged) {
                            sync_cindex = sync_cindex.concat(yield indexer_1.Indexer.ruleIndexGenCertificationExpiry(HEAD, this.dal));
                            sync_mindex = sync_mindex.concat(yield indexer_1.Indexer.ruleIndexGenMembershipExpiry(HEAD, this.dal));
                            sync_iindex = sync_iindex.concat(yield indexer_1.Indexer.ruleIndexGenExclusionByMembership(HEAD, sync_mindex, this.dal));
                            sync_iindex = sync_iindex.concat(yield indexer_1.Indexer.ruleIndexGenExclusionByCertificatons(HEAD, sync_cindex, local_iindex, this.conf, this.dal));
                            sync_mindex = sync_mindex.concat(yield indexer_1.Indexer.ruleIndexGenImplicitRevocation(HEAD, this.dal));
                        }
                        // Update balances with UD + local garbagings
                        yield this.blockchain.updateWallets(sync_sindex, sync_memoryDAL);
                        // --> Update links
                        yield this.dal.updateWotbLinks(local_cindex.concat(sync_cindex));
                        // Flush the INDEX again
                        yield this.dal.mindexDAL.insertBatch(sync_mindex);
                        yield this.dal.iindexDAL.insertBatch(sync_iindex);
                        yield this.dal.sindexDAL.insertBatch(sync_sindex);
                        yield this.dal.cindexDAL.insertBatch(sync_cindex);
                        sync_mindex = [];
                        sync_iindex = [];
                        sync_cindex = [];
                        sync_sindex = [];
                        // Create/Update nodes in wotb
                        yield this.blockchain.updateMembers(block, this.dal);
                    }
                    // Trim the bindex
                    sync_bindexSize = this.conf.forksize + [
                        block.issuersCount,
                        block.issuersFrame,
                        this.conf.medianTimeBlocks,
                        this.conf.dtDiffEval,
                        blocks.length
                    ].reduce((max, value) => {
                        return Math.max(max, value);
                    }, 0);
                    if (sync_bindexSize && sync_bindex.length >= 2 * sync_bindexSize) {
                        // We trim it, not necessary to store it all (we already store the full blocks)
                        sync_bindex.splice(0, sync_bindexSize);
                        // Process triming continuously to avoid super long ending of sync
                        yield this.dal.trimIndexes(sync_bindex[0].number);
                    }
                }
                else {
                    if (blocksToSave.length) {
                        yield this.saveBlocksInMainBranch(blocksToSave);
                    }
                    blocksToSave = [];
                    // Save the INDEX
                    yield this.dal.bindexDAL.insertBatch(sync_bindex);
                    yield this.dal.mindexDAL.insertBatch(sync_mindex);
                    yield this.dal.iindexDAL.insertBatch(sync_iindex);
                    yield this.dal.sindexDAL.insertBatch(sync_sindex);
                    yield this.dal.cindexDAL.insertBatch(sync_cindex);
                    // Save the intermediary table of wallets
                    const conditions = _.keys(sync_memoryWallets);
                    const nonEmptyKeys = _.filter(conditions, (k) => sync_memoryWallets[k] && sync_memoryWallets[k].balance > 0);
                    const walletsToRecord = nonEmptyKeys.map((k) => sync_memoryWallets[k]);
                    yield this.dal.walletDAL.insertBatch(walletsToRecord);
                    for (const cond of conditions) {
                        delete sync_memoryWallets[cond];
                    }
                    if (block.number === 0) {
                        yield this.blockchain.saveParametersForRoot(block, this.conf, this.dal);
                    }
                    // Last block: cautious mode to trigger all the INDEX expiry mechanisms
                    const { index, HEAD } = yield DuniterBlockchain_1.DuniterBlockchain.checkBlock(dto, constants.WITH_SIGNATURES_AND_POW, this.conf, this.dal);
                    yield this.blockchain.pushTheBlock(dto, index, HEAD, this.conf, this.dal, this.logger);
                    // Clean temporary variables
                    sync_bindex = [];
                    sync_iindex = [];
                    sync_mindex = [];
                    sync_cindex = [];
                    sync_sindex = [];
                    sync_bindexSize = 0;
                    sync_allBlocks = [];
                    sync_expires = [];
                    sync_nextExpiring = 0;
                    // sync_currConf = {};
                }
            }
            if (blocksToSave.length) {
                yield this.saveBlocksInMainBranch(blocksToSave);
            }
        });
    }
}
exports.QuickSynchronizer = QuickSynchronizer;
//# sourceMappingURL=QuickSync.js.map