import { SQLiteDriver } from "./drivers/SQLiteDriver";
import { ConfDTO } from "../dto/ConfDTO";
import { BlockDTO } from "../dto/BlockDTO";
import { DBHead } from "../db/DBHead";
import { DBIdentity } from "./sqliteDAL/IdentityDAL";
import { CindexEntry, IndexEntry } from "../indexer";
import { DBPeer } from "./sqliteDAL/PeerDAL";
import { TransactionDTO } from "../dto/TransactionDTO";
import { DBCert } from "./sqliteDAL/CertDAL";
import { DBWallet } from "./sqliteDAL/WalletDAL";
import { DBTx } from "./sqliteDAL/TxsDAL";
import { DBBlock } from "../db/DBBlock";
import { DBMembership } from "./sqliteDAL/MembershipDAL";
import { MerkleDTO } from "../dto/MerkleDTO";
export interface FileDALParams {
    home: string;
    fs: any;
    dbf: () => SQLiteDriver;
    wotb: any;
}
export declare class FileDAL {
    rootPath: string;
    myFS: any;
    sqliteDriver: SQLiteDriver;
    wotb: any;
    profile: string;
    confDAL: any;
    metaDAL: any;
    peerDAL: any;
    blockDAL: any;
    txsDAL: any;
    statDAL: any;
    idtyDAL: any;
    certDAL: any;
    msDAL: any;
    walletDAL: any;
    bindexDAL: any;
    mindexDAL: any;
    iindexDAL: any;
    sindexDAL: any;
    cindexDAL: any;
    newDals: any;
    loadConfHook: (conf: ConfDTO) => Promise<void>;
    saveConfHook: (conf: ConfDTO) => Promise<ConfDTO>;
    constructor(params: FileDALParams);
    init(conf: ConfDTO): Promise<void>;
    getDBVersion(): any;
    writeFileOfBlock(block: DBBlock): any;
    writeSideFileOfBlock(block: DBBlock): any;
    listAllPeers(): any;
    getPeer(pubkey: string): Promise<any>;
    getWS2Peers(): Promise<any>;
    getBlock(number: number): Promise<any>;
    getAbsoluteBlockByNumberAndHash(number: number, hash: string): any;
    getAbsoluteBlockByBlockstamp(blockstamp: string): any;
    getBlockByBlockstampOrNull(blockstamp: string): Promise<any>;
    getBlockByBlockstamp(blockstamp: string): Promise<any>;
    getBlockByNumberAndHash(number: number, hash: string): Promise<any>;
    getBlockByNumberAndHashOrNull(number: number, hash: string): Promise<any>;
    existsNonChainableLink(from: string, vHEAD_1: DBHead, sigStock: number): Promise<boolean>;
    getCurrentBlockOrNull(): Promise<any>;
    getPromoted(number: number): Promise<any>;
    lastUDBlock(): any;
    getRootBlock(): Promise<any>;
    getPotentialRootBlocks(): any;
    lastBlockOfIssuer(issuer: string): any;
    getCountOfPoW(issuer: string): any;
    getBlocksBetween(start: number, end: number): Promise<any>;
    getForkBlocksFollowing(current: DBBlock): any;
    getPotentialForkBlocks(numberStart: number, medianTimeStart: number, maxNumber: number): any;
    getBlockCurrent(): Promise<any>;
    getValidLinksTo(to: string): any;
    getAvailableSourcesByPubkey(pubkey: string): any;
    getIdentityByHashOrNull(hash: string): Promise<any>;
    getMembers(): any;
    getWritten(pubkey: string): Promise<any>;
    getWrittenIdtyByPubkey(pubkey: string): any;
    getWrittenIdtyByUID(uid: string): any;
    fillInMembershipsOfIdentity(queryPromise: Promise<DBIdentity>): Promise<any>;
    findPeersWhoseHashIsIn(hashes: string[]): Promise<any>;
    getTxByHash(hash: string): any;
    removeTxByHash(hash: string): any;
    getTransactionsPending(versionMin?: number): any;
    getNonWritten(pubkey: string): Promise<any>;
    getRevocatingMembers(): Promise<any[]>;
    getToBeKickedPubkeys(): any;
    getRevokedPubkeys(): any;
    searchJustIdentities(search: string): Promise<[{}, {}, {}, {}, {}, {}, {}, {}, {}, {}]>;
    certsToTarget(pub: string, hash: string): Promise<any>;
    certsFrom(pubkey: string): Promise<any>;
    isSentry(pubkey: string, conf: ConfDTO): Promise<boolean>;
    certsFindNew(): Promise<any>;
    certsNotLinkedToTarget(hash: string): Promise<any>;
    getMostRecentMembershipNumberForIssuer(issuer: string): Promise<number>;
    lastJoinOfIdentity(target: string): Promise<any>;
    findNewcomers(blockMedianTime?: number): Promise<any>;
    findLeavers(): Promise<any>;
    existsNonReplayableLink(from: string, to: string): any;
    getSource(identifier: string, pos: number): any;
    isMember(pubkey: string): Promise<any>;
    isMemberAndNonLeaver(pubkey: string): Promise<boolean>;
    isLeaving(pubkey: string): Promise<any>;
    existsCert(cert: any): Promise<any>;
    deleteCert(cert: any): any;
    deleteMS(ms: any): any;
    setRevoked(pubkey: string): Promise<any>;
    setRevocating: (existing: DBIdentity, revocation_sig: string) => any;
    getPeerOrNull(pubkey: string): Promise<any>;
    removePeerByPubkey(pubkey: string): Promise<any>;
    findAllPeersNEWUPBut(pubkeys: string[]): Promise<any>;
    listAllPeersWithStatusNewUP(): Promise<any>;
    listAllPeersWithStatusNewUPWithtout(pub: string): Promise<any>;
    findPeers(pubkey: string): Promise<any[]>;
    getRandomlyUPsWithout(pubkeys: string[]): Promise<any>;
    setPeerUP(pubkey: string): Promise<any>;
    setPeerDown(pubkey: string): Promise<void>;
    saveBlock(block: BlockDTO): Promise<void>;
    generateIndexes(block: DBBlock, conf: ConfDTO, index: IndexEntry[], HEAD: DBHead): Promise<{
        HEAD: DBHead;
        mindex: any;
        iindex: any;
        sindex: any;
        cindex: any;
    }>;
    updateWotbLinks(cindex: CindexEntry[]): Promise<void>;
    trimIndexes(maxNumber: number): Promise<boolean>;
    trimSandboxes(block: DBBlock): Promise<boolean>;
    savePendingMembership(ms: DBMembership): any;
    saveBlockInFile(block: DBBlock): Promise<void>;
    saveSideBlockInFile(block: DBBlock): any;
    saveTxsInFiles(txs: TransactionDTO[], block_number: number, medianTime: number): Promise<any[]>;
    merkleForPeers(): Promise<MerkleDTO>;
    removeAllSourcesOfBlock(blockstamp: string): any;
    updateTransactions(txs: DBTx[]): any;
    savePendingIdentity(idty: DBIdentity): any;
    revokeIdentity(pubkey: string): any;
    removeUnWrittenWithPubkey(pubkey: string): Promise<any>;
    removeUnWrittenWithUID(pubkey: string): Promise<any>;
    registerNewCertification(cert: DBCert): any;
    saveTransaction(tx: DBTx): any;
    getTransactionsHistory(pubkey: string): Promise<{
        sent: DBTx[];
        received: DBTx[];
        sending: DBTx[];
        receiving: DBTx[];
        pending: DBTx[];
    }>;
    getUDHistory(pubkey: string): Promise<{
        history: any;
    }>;
    savePeer(peer: DBPeer): any;
    getUniqueIssuersBetween(start: number, end: number): Promise<any>;
    /**
     * Gets a range of entries for the last `start`th to the last `end`th HEAD entry.
     * @param start The starting entry number (min. 1)
     * @param end The ending entry (max. BINDEX length)
     * @param property If provided, transforms the range of entries into an array of the asked property.
     */
    range(start: number, end: number, property: string): Promise<any>;
    /**
     * Get the last `n`th entry from the BINDEX.
     * @param n The entry number (min. 1).
     */
    head(n: number): any;
    /***********************
     *    CONFIGURATION
     **********************/
    getParameters(): any;
    loadConf(overrideConf: ConfDTO, defaultConf?: boolean): Promise<any>;
    saveConf(confToSave: ConfDTO): Promise<any>;
    /***********************
     *     WALLETS
     **********************/
    getWallet(conditions: string): Promise<any>;
    saveWallet(wallet: DBWallet): any;
    /***********************
     *     STATISTICS
     **********************/
    loadStats(): any;
    getStat(name: string): any;
    pushStats(stats: any): any;
    cleanCaches(): Promise<void>;
    close(): Promise<void>;
    resetPeers(): Promise<void>;
    getLogContent(linesQuantity: number): Promise<{}>;
}
