"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractCFS_1 = require("./AbstractCFS");
const ConfDTO_1 = require("../../dto/ConfDTO");
const constants_1 = require("../../common-libs/constants");
const _ = require('underscore');
class ConfDAL extends AbstractCFS_1.AbstractCFS {
    constructor(rootPath, qioFS) {
        super(rootPath, qioFS);
        this.logger = require('../../logger').NewLogger();
    }
    init() {
        return Promise.resolve();
    }
    getParameters() {
        return __awaiter(this, void 0, void 0, function* () {
            const conf = yield this.loadConf();
            return {
                "currency": conf.currency,
                "c": parseFloat(conf.c),
                "dt": parseInt(conf.dt, 10),
                "ud0": parseInt(conf.ud0, 10),
                "sigPeriod": parseInt(conf.sigPeriod, 10),
                "sigStock": parseInt(conf.sigStock, 10),
                "sigWindow": parseInt(conf.sigWindow, 10),
                "sigValidity": parseInt(conf.sigValidity, 10),
                "sigQty": parseInt(conf.sigQty, 10),
                "idtyWindow": parseInt(conf.idtyWindow, 10),
                "msWindow": parseInt(conf.msWindow, 10),
                "xpercent": parseFloat(conf.xpercent),
                "msValidity": parseInt(conf.msValidity, 10),
                "stepMax": parseInt(conf.stepMax, 10),
                "medianTimeBlocks": parseInt(conf.medianTimeBlocks, 10),
                "avgGenTime": parseInt(conf.avgGenTime, 10),
                "dtDiffEval": parseInt(conf.dtDiffEval, 10),
                "percentRot": parseFloat(conf.percentRot),
                "udTime0": parseInt(conf.udTime0),
                "udReevalTime0": parseInt(conf.udReevalTime0),
                "dtReeval": parseInt(conf.dtReeval),
                "switchOnHeadAdvance": constants_1.CommonConstants.SWITCH_ON_BRANCH_AHEAD_BY_X_BLOCKS
            };
        });
    }
    loadConf() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = yield this.coreFS.readJSON('conf.json');
            if (data) {
                return _(ConfDTO_1.ConfDTO.defaultConf()).extend(data);
            }
            else {
                // Silent error
                this.logger.warn('No configuration loaded');
                return {};
            }
        });
    }
    saveConf(confToSave) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.coreFS.writeJSONDeep('conf.json', confToSave);
        });
    }
}
exports.ConfDAL = ConfDAL;
//# sourceMappingURL=ConfDAL.js.map