"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractSQLite_1 = require("./AbstractSQLite");
const indexer_1 = require("../../indexer");
const _ = require('underscore');
class AbstractIndex extends AbstractSQLite_1.AbstractSQLite {
    constructor(driver, table, pkFields = [], fields = [], arrays = [], booleans = [], bigintegers = [], transientFields = [], beforeSaveHook = null) {
        super(driver, table, pkFields, fields, arrays, booleans, bigintegers, transientFields, beforeSaveHook);
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    getWrittenOn(blockstamp) {
        return this.query('SELECT * FROM ' + this.table + ' WHERE written_on = ?', [blockstamp]);
    }
    trimRecords(belowNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            const belowRecords = yield this.query('SELECT COUNT(*) as nbRecords, pub FROM ' + this.table + ' ' +
                'WHERE CAST(written_on as int) < ? ' +
                'GROUP BY pub ' +
                'HAVING nbRecords > 1', [belowNumber]);
            const reducedByPub = indexer_1.Indexer.DUP_HELPERS.reduceBy(belowRecords, ['pub']);
            for (const record of reducedByPub) {
                const recordsOfPub = yield this.query('SELECT * FROM ' + this.table + ' WHERE pub = ?', [record.pub]);
                const toReduce = _.filter(recordsOfPub, (rec) => parseInt(rec.written_on) < belowNumber);
                if (toReduce.length) {
                    // Clean the records in the DB
                    yield this.exec('DELETE FROM ' + this.table + ' WHERE pub = \'' + record.pub + '\'');
                    const nonReduced = _.filter(recordsOfPub, (rec) => parseInt(rec.written_on) >= belowNumber);
                    const reduced = indexer_1.Indexer.DUP_HELPERS.reduce(toReduce);
                    // Persist
                    yield this.insertBatch([reduced].concat(nonReduced));
                }
            }
        });
    }
}
exports.AbstractIndex = AbstractIndex;
//# sourceMappingURL=AbstractIndex.js.map