"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractSQLite_1 = require("./AbstractSQLite");
const SandBox_1 = require("./SandBox");
const constants = require('../../constants');
class CertDAL extends AbstractSQLite_1.AbstractSQLite {
    constructor(driver) {
        super(driver, 'cert', 
        // PK fields
        ['from', 'target', 'sig'], 
        // Fields
        [
            'linked',
            'written',
            'written_block',
            'written_hash',
            'sig',
            'block_number',
            'block_hash',
            'target',
            'to',
            'from',
            'block',
            'expired',
            'expires_on'
        ], 
        // Arrays
        [], 
        // Booleans
        ['linked', 'written'], 
        // BigIntegers
        [], 
        // Transient
        [], (entity) => {
            entity.written = entity.written || !!(entity.written_hash);
        });
        /**************************
         * SANDBOX STUFF
         */
        this.getSandboxForKey = (pub) => {
            const getRecorded = () => this.query('SELECT * FROM cert WHERE `from` = ? ORDER BY block_number ASC LIMIT ' + constants.SANDBOX_SIZE_CERTIFICATIONS, [pub]);
            const compare = (compared, reference) => {
                if (compared.block_number < reference.block_number) {
                    return -1;
                }
                else if (compared.block_number > reference.block_number) {
                    return 1;
                }
                else {
                    return 0;
                }
            };
            return new SandBox_1.SandBox(constants.SANDBOX_SIZE_CERTIFICATIONS, getRecorded, compare);
        };
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.exec('BEGIN;' +
                'CREATE TABLE IF NOT EXISTS ' + this.table + ' (' +
                '`from` VARCHAR(50) NOT NULL,' +
                '`to` VARCHAR(50) NOT NULL,' +
                'target CHAR(64) NOT NULL,' +
                'sig VARCHAR(100) NOT NULL,' +
                'block_number INTEGER NOT NULL,' +
                'block_hash VARCHAR(64),' +
                'block INTEGER NOT NULL,' +
                'linked BOOLEAN NOT NULL,' +
                'written BOOLEAN NOT NULL,' +
                'written_block INTEGER,' +
                'written_hash VARCHAR(64),' +
                'expires_on INTEGER NULL,' +
                'PRIMARY KEY (`from`, target, sig, written_block)' +
                ');' +
                'CREATE INDEX IF NOT EXISTS idx_cert_from ON cert (`from`);' +
                'CREATE INDEX IF NOT EXISTS idx_cert_target ON cert (target);' +
                'CREATE INDEX IF NOT EXISTS idx_cert_linked ON cert (linked);' +
                'COMMIT;');
        });
    }
    getToTarget(hash) {
        return this.sqlFind({
            target: hash
        });
    }
    getFromPubkeyCerts(pubkey) {
        return this.sqlFind({
            from: pubkey
        });
    }
    getNotLinked() {
        return this.sqlFind({
            linked: false
        });
    }
    getNotLinkedToTarget(hash) {
        return this.sqlFind({
            target: hash,
            linked: false
        });
    }
    saveNewCertification(cert) {
        return this.saveEntity(cert);
    }
    existsGivenCert(cert) {
        return this.sqlExisting(cert);
    }
    deleteCert(cert) {
        return this.deleteEntity(cert);
    }
    trimExpiredCerts(medianTime) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.exec('DELETE FROM ' + this.table + ' WHERE expires_on IS NULL OR expires_on < ' + medianTime);
        });
    }
}
exports.CertDAL = CertDAL;
//# sourceMappingURL=CertDAL.js.map