"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractSQLite_1 = require("./AbstractSQLite");
const SandBox_1 = require("./SandBox");
const _ = require('underscore');
const constants = require('../../constants');
class MembershipDAL extends AbstractSQLite_1.AbstractSQLite {
    constructor(driver) {
        super(driver, 'membership', 
        // PK fields
        ['issuer', 'signature'], 
        // Fields
        [
            'membership',
            'issuer',
            'number',
            'blockNumber',
            'blockHash',
            'userid',
            'certts',
            'block',
            'fpr',
            'idtyHash',
            'written',
            'written_number',
            'expires_on',
            'signature',
            'expired'
        ], 
        // Arrays
        [], 
        // Booleans
        ['written'], 
        // BigIntegers
        [], 
        // Transient
        []);
        this.sandbox = new SandBox_1.SandBox(constants.SANDBOX_SIZE_MEMBERSHIPS, this.getSandboxMemberships.bind(this), (compared, reference) => {
            if (compared.block_number < reference.block_number) {
                return -1;
            }
            else if (compared.block_number > reference.block_number) {
                return 1;
            }
            else {
                return 0;
            }
        });
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.exec('BEGIN;' +
                'CREATE TABLE IF NOT EXISTS membership (' +
                'membership CHAR(2) NOT NULL,' +
                'issuer VARCHAR(50) NOT NULL,' +
                'number INTEGER NOT NULL,' +
                'blockNumber INTEGER,' +
                'blockHash VARCHAR(64) NOT NULL,' +
                'userid VARCHAR(255) NOT NULL,' +
                'certts VARCHAR(100) NOT NULL,' +
                'block INTEGER,' +
                'fpr VARCHAR(64),' +
                'idtyHash VARCHAR(64),' +
                'written BOOLEAN NOT NULL,' +
                'written_number INTEGER,' +
                'expires_on INTEGER NULL,' +
                'signature VARCHAR(50),' +
                'PRIMARY KEY (issuer,signature)' +
                ');' +
                'CREATE INDEX IF NOT EXISTS idx_mmembership_idtyHash ON membership (idtyHash);' +
                'CREATE INDEX IF NOT EXISTS idx_mmembership_membership ON membership (membership);' +
                'CREATE INDEX IF NOT EXISTS idx_mmembership_written ON membership (written);' +
                'COMMIT;');
        });
    }
    getMembershipsOfIssuer(issuer) {
        return this.sqlFind({
            issuer: issuer
        });
    }
    getPendingINOfTarget(hash) {
        return this.sqlFind({
            idtyHash: hash,
            membership: 'IN'
        });
    }
    getPendingIN() {
        return this.sqlFind({
            membership: 'IN'
        });
    }
    getPendingOUT() {
        return this.sqlFind({
            membership: 'OUT'
        });
    }
    savePendingMembership(ms) {
        ms.membership = ms.membership.toUpperCase();
        ms.written = false;
        return this.saveEntity(_.pick(ms, 'membership', 'issuer', 'number', 'blockNumber', 'blockHash', 'userid', 'certts', 'block', 'fpr', 'idtyHash', 'expires_on', 'written', 'written_number', 'signature'));
    }
    deleteMS(ms) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.deleteEntity(ms);
        });
    }
    trimExpiredMemberships(medianTime) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.exec('DELETE FROM ' + this.table + ' WHERE expires_on IS NULL OR expires_on < ' + medianTime);
        });
    }
    /**************************
     * SANDBOX STUFF
     */
    getSandboxMemberships() {
        return this.query('SELECT * FROM sandbox_memberships LIMIT ' + (this.sandbox.maxSize), []);
    }
    getSandboxRoom() {
        return this.sandbox.getSandboxRoom();
    }
    setSandboxSize(maxSize) {
        this.sandbox.maxSize = maxSize;
    }
}
exports.MembershipDAL = MembershipDAL;
//# sourceMappingURL=MembershipDAL.js.map