"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractSQLite_1 = require("./AbstractSQLite");
const common_1 = require("../../common");
const TransactionDTO_1 = require("../../dto/TransactionDTO");
const BlockDAL_1 = require("./BlockDAL");
const IdentityDAL_1 = require("./IdentityDAL");
const SIndexDAL_1 = require("./index/SIndexDAL");
const WalletDAL_1 = require("./WalletDAL");
const MIndexDAL_1 = require("./index/MIndexDAL");
const IdentityDTO_1 = require("../../dto/IdentityDTO");
const index_1 = require("../../common-libs/index");
const constants_1 = require("../../common-libs/constants");
const TxsDAL_1 = require("./TxsDAL");
const _ = require('underscore');
const logger = require('../../logger').NewLogger('metaDAL');
const constants = require('./../../constants');
class MetaDAL extends AbstractSQLite_1.AbstractSQLite {
    constructor(driver) {
        super(driver, 'meta', 
        // PK fields
        ['version'], 
        // Fields
        [
            'id',
            'version'
        ], 
        // Arrays
        [], 
        // Booleans
        [], 
        // BigIntegers
        [], 
        // Transient
        []);
        this.migrations = {
            // Test
            0: 'BEGIN; COMMIT;',
            // Test
            1: 'BEGIN;' +
                'CREATE VIEW IF NOT EXISTS identities_pending AS SELECT * FROM idty WHERE NOT written;' +
                'CREATE VIEW IF NOT EXISTS certifications_pending AS SELECT * FROM cert WHERE NOT written;' +
                'CREATE VIEW IF NOT EXISTS transactions_pending AS SELECT * FROM txs WHERE NOT written;' +
                'CREATE VIEW IF NOT EXISTS transactions_desc AS SELECT * FROM txs ORDER BY time DESC;' +
                'CREATE VIEW IF NOT EXISTS forks AS SELECT number, hash, issuer, monetaryMass, dividend, UDTime, membersCount, medianTime, time, * FROM block WHERE fork ORDER BY number DESC;' +
                'CREATE VIEW IF NOT EXISTS blockchain AS SELECT number, hash, issuer, monetaryMass, dividend, UDTime, membersCount, medianTime, time, * FROM block WHERE NOT fork ORDER BY number DESC;' +
                'CREATE VIEW IF NOT EXISTS network AS select i.uid, (last_try - first_down) / 1000 as down_delay_in_sec, p.* from peer p LEFT JOIN idty i on i.pubkey = p.pubkey ORDER by down_delay_in_sec;' +
                'COMMIT;',
            // New `receveid` column
            2: 'BEGIN; ALTER TABLE txs ADD COLUMN received INTEGER NULL; COMMIT;',
            // Update wrong recipients field (was not filled in)
            3: () => __awaiter(this, void 0, void 0, function* () { }),
            // Migrates wrong unitbases
            4: () => __awaiter(this, void 0, void 0, function* () { }),
            // Migrates wrong monetary masses
            5: () => __awaiter(this, void 0, void 0, function* () { }),
            6: 'BEGIN; ALTER TABLE idty ADD COLUMN expired INTEGER NULL; COMMIT;',
            7: 'BEGIN; ALTER TABLE cert ADD COLUMN expired INTEGER NULL; COMMIT;',
            8: 'BEGIN; ALTER TABLE membership ADD COLUMN expired INTEGER NULL; COMMIT;',
            9: 'BEGIN;' +
                'ALTER TABLE txs ADD COLUMN output_base INTEGER NULL;' +
                'ALTER TABLE txs ADD COLUMN output_amount INTEGER NULL;' +
                'COMMIT;',
            10: 'BEGIN; ALTER TABLE txs ADD COLUMN blockstamp VARCHAR(200) NULL; COMMIT;',
            11: 'BEGIN;' +
                'ALTER TABLE block ADD COLUMN issuersFrame INTEGER NULL;' +
                'ALTER TABLE block ADD COLUMN issuersFrameVar INTEGER NULL;' +
                'ALTER TABLE block ADD COLUMN issuersCount INTEGER NULL;' +
                'COMMIT;',
            12: () => __awaiter(this, void 0, void 0, function* () {
                let blockDAL = new BlockDAL_1.BlockDAL(this.driverCopy);
                yield blockDAL.exec('ALTER TABLE block ADD COLUMN len INTEGER NULL;');
                yield blockDAL.exec('ALTER TABLE txs ADD COLUMN len INTEGER NULL;');
            }),
            13: 'BEGIN; ALTER TABLE txs ADD COLUMN blockstampTime INTEGER NULL; COMMIT;',
            14: 'BEGIN; ' +
                'CREATE VIEW IF NOT EXISTS sandbox_txs AS SELECT * FROM txs WHERE NOT written AND NOT removed ORDER BY output_base DESC, output_amount DESC;' +
                'CREATE VIEW IF NOT EXISTS sandbox_idty AS SELECT ' +
                'I.*, ' +
                'I.hash, ' +
                '(SELECT COUNT(*) FROM cert C where C.target = I.hash) AS certsCount, ' +
                'CAST(SUBSTR(buid, 0, INSTR(buid, "-")) as number) AS ref_block ' +
                'FROM idty as I ' +
                'WHERE NOT I.member ' +
                'AND I.expired IS NULL ' +
                'ORDER BY certsCount DESC, ref_block DESC;' +
                'CREATE VIEW IF NOT EXISTS sandbox_memberships AS SELECT ' +
                '* ' +
                'FROM membership ' +
                'WHERE expired IS NULL ' +
                'AND written_number IS NULL ' +
                'ORDER BY blockNumber DESC;' +
                'CREATE VIEW IF NOT EXISTS sandbox_certs AS SELECT ' +
                '* ' +
                'FROM cert ' +
                'WHERE expired IS NULL ' +
                'AND written_block IS NULL ' +
                'ORDER BY block_number DESC;' +
                'COMMIT;',
            15: () => __awaiter(this, void 0, void 0, function* () {
                let idtyDAL = new IdentityDAL_1.IdentityDAL(this.driverCopy);
                yield idtyDAL.exec('ALTER TABLE idty ADD COLUMN revoked_on INTEGER NULL');
            }),
            16: () => __awaiter(this, void 0, void 0, function* () { }),
            17: () => __awaiter(this, void 0, void 0, function* () {
                let blockDAL = new BlockDAL_1.BlockDAL(this.driverCopy);
                let sindexDAL = new SIndexDAL_1.SIndexDAL(this.driverCopy);
                const blocks = yield blockDAL.query('SELECT * FROM block WHERE NOT fork');
                const amountsPerKey = {};
                const members = [];
                for (const b of blocks) {
                    const amountsInForBlockPerKey = {};
                    for (const idty of b.identities) {
                        members.push(IdentityDTO_1.IdentityDTO.fromInline(idty).pubkey);
                    }
                    if (b.dividend) {
                        for (const member of members) {
                            amountsInForBlockPerKey[member] = amountsInForBlockPerKey[member] || { amounts: [], sources: [] };
                            amountsInForBlockPerKey[member].amounts.push({ amount: b.dividend * Math.pow(10, b.unitbase), comment: 'Dividend' });
                            amountsInForBlockPerKey[member].sources.push({ amount: b.dividend, base: b.unitbase, identifier: member, pos: b.number, block: b, tx: null, conditions: 'SIG(' + member + ')' });
                        }
                    }
                    const txs = b.transactions;
                    for (let i = 0; i < txs.length; i++) {
                        const tx = txs[i];
                        tx.hash = common_1.hashf(index_1.rawer.getTransaction(b.transactions[i]));
                        for (const input of tx.inputsAsObjects()) {
                            amountsInForBlockPerKey[tx.issuers[0]] = amountsInForBlockPerKey[tx.issuers[0]] || { amounts: [], sources: [] };
                            amountsInForBlockPerKey[tx.issuers[0]].amounts.push({ amount: -input.amount * Math.pow(10, input.base), comment: tx.comment || '######' });
                            amountsInForBlockPerKey[tx.issuers[0]].sources.push({
                                amount: input.amount,
                                base: input.base,
                                identifier: input.identifier,
                                pos: input.pos,
                                conditions: "",
                                block: b,
                                tx: tx.hash
                            });
                        }
                        const outputObjects = tx.outputsAsObjects();
                        for (let j = 0; j < outputObjects.length; j++) {
                            const output = outputObjects[j];
                            const conditions = output.conditions.match(/^SIG\((.+)\)$/);
                            if (conditions) {
                                amountsInForBlockPerKey[conditions[1]] = amountsInForBlockPerKey[conditions[1]] || { amounts: [], sources: [] };
                                amountsInForBlockPerKey[conditions[1]].amounts.push({ amount: output.amount * Math.pow(10, output.base), comment: tx.comment || '######' });
                                amountsInForBlockPerKey[conditions[1]].sources.push({
                                    amount: output.amount,
                                    base: output.base,
                                    identifier: tx.hash,
                                    pos: j,
                                    conditions: output.conditions,
                                    block: b,
                                    tx: tx.hash
                                });
                            }
                        }
                    }
                    for (const key of Object.keys(amountsInForBlockPerKey)) {
                        amountsPerKey[key] = amountsPerKey[key] || [];
                        amountsPerKey[key].push(amountsInForBlockPerKey[key]);
                    }
                }
                const keysToSee = Object.keys(amountsPerKey);
                const sourcesMovements = [];
                for (const key of keysToSee) {
                    const allCreates = {};
                    const allUpdates = {};
                    const amounts = amountsPerKey[key];
                    let balance = 0;
                    for (let j = 0; j < amounts.length; j++) {
                        const amountsInBlock = amounts[j].amounts;
                        for (let i = 0; i < amountsInBlock.length; i++) {
                            const a = amountsInBlock[i].amount;
                            const id = [amounts[j].sources[i].identifier, amounts[j].sources[i].pos].join('-');
                            if (a < 0) {
                                allUpdates[id] = amounts[j].sources[i];
                                delete allCreates[id];
                            }
                            else {
                                allCreates[id] = amounts[j].sources[i];
                            }
                            balance += a;
                        }
                        if (balance > 0 && balance < 100) {
                            const sourcesToDelete = [];
                            for (const k of Object.keys(amountsPerKey)) {
                                for (const packet of amountsPerKey[k]) {
                                    for (const src of packet.sources) {
                                        const id = [src.identifier, src.pos].join('-');
                                        if (src.conditions == 'SIG(' + key + ')' && allCreates[id]) {
                                            sourcesToDelete.push(src);
                                        }
                                    }
                                }
                            }
                            const amountsToDelete = sourcesToDelete.map((src) => {
                                return {
                                    amount: -src.amount * Math.pow(10, src.base),
                                    comment: '--DESTRUCTION--'
                                };
                            });
                            amounts.splice(j + 1, 0, { amounts: amountsToDelete, sources: sourcesToDelete });
                        }
                    }
                    let amountMissing = 0;
                    yield Promise.all(_.values(allCreates).map((src) => __awaiter(this, void 0, void 0, function* () {
                        const exist = yield sindexDAL.getSource(src.identifier, src.pos);
                        if (!exist || exist.consumed) {
                            amountMissing += src.amount;
                            const block = src.block;
                            sourcesMovements.push({
                                index: constants_1.CommonConstants.I_INDEX,
                                op: constants_1.CommonConstants.IDX_CREATE,
                                tx: src.tx,
                                identifier: src.identifier,
                                pos: src.pos,
                                unlock: null,
                                age: 0,
                                txObj: TransactionDTO_1.TransactionDTO.mock(),
                                created_on: null,
                                written_on: [block.number, block.hash].join('-'),
                                writtenOn: block.number,
                                written_time: block.medianTime,
                                locktime: src.locktime,
                                amount: src.amount,
                                base: src.base,
                                conditions: src.conditions,
                                consumed: false
                            });
                        }
                    })));
                    let amountNotDestroyed = 0;
                    yield Promise.all(_.values(allUpdates).map((src) => __awaiter(this, void 0, void 0, function* () {
                        const exist = yield sindexDAL.getSource(src.identifier, src.pos);
                        if (exist && !exist.consumed) {
                            amountNotDestroyed += src.amount;
                        }
                    })));
                }
                yield sindexDAL.insertBatch(sourcesMovements);
            }),
            18: 'BEGIN;' +
                // Add a `massReeval` column
                'ALTER TABLE b_index ADD COLUMN massReeval VARCHAR(100) NOT NULL DEFAULT \'0\';' +
                'COMMIT;',
            19: 'BEGIN;' +
                // Add a `removed` column
                'ALTER TABLE idty ADD COLUMN removed BOOLEAN NULL DEFAULT 0;' +
                'COMMIT;',
            /**
             * Feeds the table of wallets with balances
             */
            20: () => __awaiter(this, void 0, void 0, function* () {
                let walletDAL = new WalletDAL_1.WalletDAL(this.driverCopy);
                let sindexDAL = new SIndexDAL_1.SIndexDAL(this.driverCopy);
                const conditions = yield sindexDAL.query('SELECT DISTINCT(conditions) FROM s_index');
                for (const row of conditions) {
                    const wallet = {
                        conditions: row.conditions,
                        balance: 0
                    };
                    const amountsRemaining = yield sindexDAL.getAvailableForConditions(row.conditions);
                    wallet.balance = amountsRemaining.reduce((sum, src) => sum + src.amount * Math.pow(10, src.base), 0);
                    yield walletDAL.saveWallet(wallet);
                }
            }),
            /**
             * Feeds the m_index.chainable_on
             */
            21: (conf) => __awaiter(this, void 0, void 0, function* () {
                let blockDAL = new BlockDAL_1.BlockDAL(this.driverCopy);
                let mindexDAL = new MIndexDAL_1.MIndexDAL(this.driverCopy);
                yield mindexDAL.exec('ALTER TABLE m_index ADD COLUMN chainable_on INTEGER NULL;');
                const memberships = yield mindexDAL.query('SELECT * FROM m_index WHERE op = ?', [constants_1.CommonConstants.IDX_CREATE]);
                for (const ms of memberships) {
                    const reference = yield blockDAL.getBlock(parseInt(ms.written_on.split('-')[0]));
                    const updateQuery = 'UPDATE m_index SET chainable_on = ' + (reference.medianTime + conf.msPeriod) + ' WHERE pub = \'' + ms.pub + '\' AND op = \'CREATE\'';
                    yield mindexDAL.exec(updateQuery);
                }
            }),
            // Replay the wallet table feeding, because of a potential bug
            22: () => {
                return this.migrations[20]();
            },
            23: 'BEGIN;' +
                // Add a `writtenOn` column for MISC Index
                'ALTER TABLE m_index ADD COLUMN writtenOn INTEGER NOT NULL DEFAULT 0;' +
                'ALTER TABLE i_index ADD COLUMN writtenOn INTEGER NOT NULL DEFAULT 0;' +
                'ALTER TABLE s_index ADD COLUMN writtenOn INTEGER NOT NULL DEFAULT 0;' +
                'ALTER TABLE c_index ADD COLUMN writtenOn INTEGER NOT NULL DEFAULT 0;' +
                'CREATE INDEX IF NOT EXISTS idx_mindex_writtenOn ON m_index (writtenOn);' +
                'CREATE INDEX IF NOT EXISTS idx_iindex_writtenOn ON i_index (writtenOn);' +
                'CREATE INDEX IF NOT EXISTS idx_sindex_writtenOn ON s_index (writtenOn);' +
                'CREATE INDEX IF NOT EXISTS idx_cindex_writtenOn ON c_index (writtenOn);' +
                'UPDATE m_index SET writtenOn = CAST(written_on as integer);' +
                'UPDATE i_index SET writtenOn = CAST(written_on as integer);' +
                'UPDATE s_index SET writtenOn = CAST(written_on as integer);' +
                'UPDATE c_index SET writtenOn = CAST(written_on as integer);' +
                'COMMIT;',
            /**
             * Feeds the m_index.chainable_on correctly
             */
            24: (conf) => __awaiter(this, void 0, void 0, function* () {
                let blockDAL = new BlockDAL_1.BlockDAL(this.driverCopy);
                let mindexDAL = new MIndexDAL_1.MIndexDAL(this.driverCopy);
                const memberships = yield mindexDAL.query('SELECT * FROM m_index');
                for (const ms of memberships) {
                    const reference = yield blockDAL.getBlock(parseInt(ms.written_on.split('-')[0]));
                    const msPeriod = conf.msWindow; // It has the same value, as it was not defined on currency init
                    const updateQuery = 'UPDATE m_index SET chainable_on = ' + (reference.medianTime + msPeriod) + ' WHERE pub = \'' + ms.pub + '\' AND written_on = \'' + ms.written_on + '\'';
                    yield mindexDAL.exec(updateQuery);
                }
            }),
            /**
             * Wrong transaction storage
             */
            25: () => __awaiter(this, void 0, void 0, function* () {
                const txsDAL = new TxsDAL_1.TxsDAL(this.driverCopy);
                const wrongTXS = yield txsDAL.query('SELECT * FROM txs WHERE outputs LIKE ? OR inputs LIKE ?', ['%amount%', '%amount%']);
                let i = 1;
                for (const tx of wrongTXS) {
                    logger.info('Updating incorrect transaction %s/%s.', i, wrongTXS.length);
                    i++;
                    const dto = TransactionDTO_1.TransactionDTO.fromJSONObject(tx);
                    dto.outputs = dto.outputs.map(o => {
                        if (typeof o === 'object') {
                            return TransactionDTO_1.TransactionDTO.outputObj2Str(o);
                        }
                        return o;
                    });
                    dto.inputs = dto.inputs.map(o => {
                        if (typeof o === 'object') {
                            return TransactionDTO_1.TransactionDTO.inputObj2Str(o);
                        }
                        return o;
                    });
                    yield txsDAL.exec('UPDATE txs SET ' +
                        'outputs = \'' + JSON.stringify(dto.outputs) + '\', ' +
                        'inputs = \'' + JSON.stringify(dto.inputs) + '\' ' +
                        'WHERE hash = \'' + tx.hash + '\'');
                }
            }),
        };
        this.driverCopy = driver;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.exec('BEGIN;' +
                'CREATE TABLE IF NOT EXISTS ' + this.table + ' (' +
                'id INTEGER NOT NULL,' +
                'version INTEGER NOT NULL,' +
                'PRIMARY KEY (id)' +
                ');' +
                'COMMIT;');
        });
    }
    executeMigration(migration, conf) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (typeof migration == "string") {
                    // Simple SQL script to pass
                    yield this.exec(migration);
                }
                else if (typeof migration == "function") {
                    // JS function to execute
                    yield migration(conf);
                }
            }
            catch (e) {
                logger.warn('An error occured during DB migration, continue.', e);
            }
        });
    }
    upgradeDatabase(conf) {
        return __awaiter(this, void 0, void 0, function* () {
            let version = yield this.getVersion();
            while (this.migrations[version]) {
                yield this.executeMigration(this.migrations[version], conf);
                // Automated increment
                yield this.exec('UPDATE meta SET version = version + 1');
                version++;
            }
        });
    }
    getRow() {
        return this.sqlFindOne({ id: 1 });
    }
    getVersion() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const row = yield this.getRow();
                return row.version;
            }
            catch (e) {
                yield this.exec('INSERT INTO ' + this.table + ' VALUES (1,0);');
                return 0;
            }
        });
    }
    cleanData() {
        // Never clean data of this table
        return Promise.resolve();
    }
}
exports.MetaDAL = MetaDAL;
//# sourceMappingURL=MetaDAL.js.map