"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractSQLite_1 = require("./AbstractSQLite");
class DBPeer {
    json() {
        return {
            version: this.version,
            currency: this.currency,
            endpoints: this.endpoints,
            status: this.status,
            block: this.block,
            signature: this.signature,
            raw: this.raw,
            pubkey: this.pubkey
        };
    }
}
exports.DBPeer = DBPeer;
class PeerDAL extends AbstractSQLite_1.AbstractSQLite {
    constructor(driver) {
        super(driver, 'peer', 
        // PK fields
        ['pubkey'], 
        // Fields
        [
            'version',
            'currency',
            'status',
            'statusTS',
            'hash',
            'first_down',
            'last_try',
            'pubkey',
            'block',
            'signature',
            'endpoints',
            'raw'
        ], 
        // Arrays
        ['endpoints'], 
        // Booleans
        [], 
        // BigIntegers
        [], 
        // Transient
        []);
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.exec('BEGIN;' +
                'CREATE TABLE IF NOT EXISTS ' + this.table + ' (' +
                'version INTEGER NOT NULL,' +
                'currency VARCHAR(50) NOT NULL,' +
                'status VARCHAR(10),' +
                'statusTS INTEGER NOT NULL,' +
                'hash CHAR(64),' +
                'first_down INTEGER,' +
                'last_try INTEGER,' +
                'pubkey VARCHAR(50) NOT NULL,' +
                'block VARCHAR(60) NOT NULL,' +
                'signature VARCHAR(100),' +
                'endpoints TEXT NOT NULL,' +
                'raw TEXT,' +
                'PRIMARY KEY (pubkey)' +
                ');' +
                'CREATE INDEX IF NOT EXISTS idx_link_source ON peer (pubkey);' +
                'COMMIT;');
        });
    }
    listAll() {
        return this.sqlListAll();
    }
    getPeer(pubkey) {
        return this.sqlFindOne({ pubkey: pubkey });
    }
    getPeersWithEndpointsLike(str) {
        return this.query('SELECT * FROM peer WHERE endpoints LIKE ?', ['%' + str + '%']);
    }
    savePeer(peer) {
        return this.saveEntity(peer);
    }
    removePeerByPubkey(pubkey) {
        return this.exec('DELETE FROM peer WHERE pubkey LIKE \'' + pubkey + '\'');
    }
    removeAll() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.sqlDeleteAll();
        });
    }
}
exports.PeerDAL = PeerDAL;
//# sourceMappingURL=PeerDAL.js.map