import { AbstractSQLite } from "./AbstractSQLite";
import { SQLiteDriver } from "../drivers/SQLiteDriver";
import { TransactionDTO } from "../../dto/TransactionDTO";
import { SandBox } from "./SandBox";
export declare class DBTx {
    hash: string;
    block_number: number | null;
    locktime: number;
    version: number;
    currency: string;
    comment: string;
    blockstamp: string;
    blockstampTime: number | null;
    time: number | null;
    inputs: string[];
    unlocks: string[];
    outputs: string[];
    issuers: string[];
    signatures: string[];
    recipients: string[];
    written: boolean;
    removed: boolean;
    received: number;
    output_base: number;
    output_amount: number;
    static fromTransactionDTO(tx: TransactionDTO): DBTx;
    static setRecipients(txs: DBTx[]): void;
    static outputs2recipients(tx: DBTx): string[];
}
export declare class TxsDAL extends AbstractSQLite<DBTx> {
    constructor(driver: SQLiteDriver);
    init(): Promise<void>;
    getAllPending(versionMin: number): Promise<DBTx[]>;
    getTX(hash: string): Promise<DBTx>;
    removeTX(hash: string): Promise<void>;
    addLinked(tx: TransactionDTO, block_number: number, time: number): Promise<void>;
    addPending(tx: TransactionDTO): Promise<void>;
    getLinkedWithIssuer(pubkey: string): Promise<DBTx[]>;
    getLinkedWithRecipient(pubkey: string): Promise<any>;
    getPendingWithIssuer(pubkey: string): Promise<DBTx[]>;
    getPendingWithRecipient(pubkey: string): Promise<DBTx[]>;
    insertBatchOfTxs(txs: DBTx[]): void;
    trimExpiredNonWrittenTxs(limitTime: number): Promise<void>;
    getTransactionByExtendedHash(hash: string): Promise<DBTx[]>;
    /**************************
     * SANDBOX STUFF
     */
    getSandboxTxs(): Promise<DBTx[]>;
    sandbox: SandBox<DBTx>;
    getSandboxRoom(): Promise<number>;
    setSandboxSize(maxSize: number): void;
}
