"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractIndex_1 = require("../AbstractIndex");
const indexer_1 = require("../../../indexer");
class MIndexDAL extends AbstractIndex_1.AbstractIndex {
    constructor(driver) {
        super(driver, 'm_index', 
        // PK fields
        ['op', 'pub', 'created_on', 'written_on'], 
        // Fields
        [
            'op',
            'pub',
            'created_on',
            'written_on',
            'writtenOn',
            'expires_on',
            'expired_on',
            'revokes_on',
            'revoked_on',
            'chainable_on',
            'leaving',
            'revocation'
        ], 
        // Arrays
        [], 
        // Booleans
        ['leaving'], 
        // BigIntegers
        [], 
        // Transient
        []);
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.exec('BEGIN;' +
                'CREATE TABLE IF NOT EXISTS ' + this.table + ' (' +
                'op VARCHAR(10) NOT NULL,' +
                'pub VARCHAR(50) NOT NULL,' +
                'created_on VARCHAR(80) NOT NULL,' +
                'written_on VARCHAR(80) NOT NULL,' +
                'expires_on INTEGER NULL,' +
                'expired_on INTEGER NULL,' +
                'revokes_on INTEGER NULL,' +
                'revoked_on INTEGER NULL,' +
                'leaving BOOLEAN NULL,' +
                'revocation VARCHAR(80) NULL,' +
                'PRIMARY KEY (op,pub,created_on,written_on)' +
                ');' +
                'CREATE INDEX IF NOT EXISTS idx_mindex_pub ON m_index (pub);' +
                'COMMIT;');
        });
    }
    getReducedMS(pub) {
        return __awaiter(this, void 0, void 0, function* () {
            const reducables = yield this.reducable(pub);
            if (reducables.length) {
                return indexer_1.Indexer.DUP_HELPERS.reduce(reducables);
            }
            return null;
        });
    }
    reducable(pub) {
        return this.query('SELECT * FROM ' + this.table + ' WHERE pub = ? ORDER BY CAST(written_on as integer) ASC', [pub]);
    }
    removeBlock(blockstamp) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.exec('DELETE FROM ' + this.table + ' WHERE written_on = \'' + blockstamp + '\'');
        });
    }
    getRevokedPubkeys() {
        return __awaiter(this, void 0, void 0, function* () {
            // All those who has been revoked. Make one result per pubkey.
            const revovedMemberships = yield this.sqlFind({ revoked_on: { $null: false } });
            // Filter on those to be revoked, return their pubkey
            return revovedMemberships.map((entry) => entry.pub);
        });
    }
}
exports.MIndexDAL = MIndexDAL;
//# sourceMappingURL=MIndexDAL.js.map