"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const indexer_1 = require("../../../indexer");
const AbstractIndex_1 = require("../AbstractIndex");
const constants_1 = require("../../../common-libs/constants");
const _ = require('underscore');
const constants = require('../../../constants');
class SIndexDAL extends AbstractIndex_1.AbstractIndex {
    constructor(driver) {
        super(driver, 's_index', 
        // PK fields
        ['op', 'identifier', 'pos', 'written_on'], 
        // Fields
        [
            'op',
            'tx',
            'identifier',
            'pos',
            'created_on',
            'written_on',
            'writtenOn',
            'written_time',
            'amount',
            'base',
            'locktime',
            'consumed',
            'conditions'
        ], 
        // Arrays
        [], 
        // Booleans
        ['consumed'], 
        // BigIntegers
        [], 
        // Transient
        []);
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.exec('BEGIN;' +
                'CREATE TABLE IF NOT EXISTS ' + this.table + ' (' +
                'op VARCHAR(10) NOT NULL,' +
                'tx VARCHAR(80) NULL,' +
                'identifier VARCHAR(64) NOT NULL,' +
                'pos INTEGER NOT NULL,' +
                'created_on VARCHAR(80) NULL,' +
                'written_on VARCHAR(80) NOT NULL,' +
                'written_time INTEGER NOT NULL,' +
                'amount INTEGER NULL,' +
                'base INTEGER NULL,' +
                'locktime INTEGER NULL,' +
                'consumed BOOLEAN NOT NULL,' +
                'conditions TEXT,' +
                'PRIMARY KEY (op,identifier,pos,written_on)' +
                ');' +
                'CREATE INDEX IF NOT EXISTS idx_sindex_identifier ON s_index (identifier);' +
                'CREATE INDEX IF NOT EXISTS idx_sindex_pos ON s_index (pos);' +
                'COMMIT;');
        });
    }
    removeBlock(blockstamp) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.exec('DELETE FROM ' + this.table + ' WHERE written_on = \'' + blockstamp + '\'');
        });
    }
    getSource(identifier, pos) {
        return __awaiter(this, void 0, void 0, function* () {
            const reducable = yield this.query('SELECT * FROM ' + this.table + ' s1 ' +
                'WHERE s1.identifier = ? ' +
                'AND s1.pos = ? ' +
                'ORDER BY op ASC', [identifier, pos]);
            if (reducable.length == 0) {
                return null;
            }
            else {
                const src = indexer_1.Indexer.DUP_HELPERS.reduce(reducable);
                src.type = src.tx ? 'T' : 'D';
                return src;
            }
        });
    }
    getUDSources(pubkey) {
        return __awaiter(this, void 0, void 0, function* () {
            const reducables = yield this.query('SELECT * FROM ' + this.table + ' s1 ' +
                'WHERE conditions = ? ' +
                'AND s1.tx IS NULL ' +
                'ORDER BY op ASC', ['SIG(' + pubkey + ')']);
            const reduced = indexer_1.Indexer.DUP_HELPERS.reduceBy(reducables, ['identifier', 'pos']).map((src) => {
                src.type = src.tx ? 'T' : 'D';
                return src;
            });
            return _.sortBy(reduced, (row) => row.type == 'D' ? 0 : 1);
        });
    }
    getAvailableForPubkey(pubkey) {
        return this.getAvailableForConditions('%SIG(' + pubkey + ')%');
    }
    getAvailableForConditions(conditionsStr) {
        return __awaiter(this, void 0, void 0, function* () {
            const potentials = yield this.query('SELECT * FROM ' + this.table + ' s1 ' +
                'WHERE s1.op = ? ' +
                'AND conditions LIKE ? ' +
                'AND NOT EXISTS (' +
                ' SELECT * ' +
                ' FROM s_index s2 ' +
                ' WHERE s2.identifier = s1.identifier ' +
                ' AND s2.pos = s1.pos ' +
                ' AND s2.op = ?' +
                ') ' +
                'ORDER BY CAST(SUBSTR(written_on, 0, INSTR(written_on, "-")) as number)', [constants_1.CommonConstants.IDX_CREATE, conditionsStr, constants_1.CommonConstants.IDX_UPDATE]);
            const sources = potentials.map((src) => {
                src.type = src.tx ? 'T' : 'D';
                return src;
            });
            return _.sortBy(sources, (row) => row.type == 'D' ? 0 : 1);
        });
    }
    trimConsumedSource(belowNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            const toDelete = yield this.query('SELECT * FROM ' + this.table + ' WHERE consumed AND CAST(written_on as int) < ?', [belowNumber]);
            const queries = [];
            for (const row of toDelete) {
                const sql = "DELETE FROM " + this.table + " " +
                    "WHERE identifier like '" + row.identifier + "' " +
                    "AND pos = " + row.pos;
                queries.push(sql);
            }
            yield this.exec(queries.join(';\n'));
        });
    }
}
exports.SIndexDAL = SIndexDAL;
//# sourceMappingURL=SIndexDAL.js.map