import { ProxiesConf } from '../proxy';
export interface Keypair {
    pub: string;
    sec: string;
}
export interface BranchingDTO {
    switchOnHeadAdvance: number;
    avgGenTime: number;
    forksize: number;
}
export interface CurrencyConfDTO {
    currency: string;
    c: number;
    dt: number;
    ud0: number;
    sigPeriod: number;
    sigStock: number;
    sigWindow: number;
    sigValidity: number;
    sigQty: number;
    idtyWindow: number;
    msWindow: number;
    msPeriod: number;
    xpercent: number;
    msValidity: number;
    stepMax: number;
    medianTimeBlocks: number;
    avgGenTime: number;
    dtDiffEval: number;
    percentRot: number;
    udTime0: number;
    udReevalTime0: number;
    dtReeval: number;
}
export interface KeypairConfDTO {
    pair: Keypair;
    oldPair: Keypair | null;
    salt: string;
    passwd: string;
}
export interface NetworkConfDTO {
    proxiesConf: ProxiesConf | undefined;
    nobma: boolean;
    bmaWithCrawler: boolean;
    remoteport: number;
    remotehost: string | null;
    remoteipv4: string | null;
    remoteipv6: string | null;
    port: number;
    ipv4: string;
    ipv6: string;
    dos: any;
    upnp: boolean;
    httplogs: boolean;
}
export interface WS2PConfDTO {
    ws2p?: {
        privateAccess: boolean;
        publicAccess: boolean;
        uuid: string;
        upnp?: boolean;
        remotehost?: string | null;
        remoteport?: number | null;
        remotepath?: string;
        port?: number;
        host?: string;
        maxPublic?: number;
        maxPrivate?: number;
        preferedNodes?: string[];
        preferedOnly: boolean;
        privilegedNodes?: string[];
        privilegedOnly: boolean;
    };
}
export declare class ConfDTO implements CurrencyConfDTO, KeypairConfDTO, NetworkConfDTO, BranchingDTO, WS2PConfDTO {
    loglevel: string;
    currency: string;
    endpoints: string[];
    rmEndpoints: string[];
    rootoffset: number;
    upInterval: number;
    cpu: number;
    nbCores: number;
    prefix: number;
    powSecurityRetryDelay: number;
    powMaxHandicap: number;
    c: number;
    dt: number;
    dtReeval: number;
    dtDiffEval: number;
    ud0: number;
    udTime0: number;
    udReevalTime0: number;
    stepMax: number;
    sigPeriod: number;
    msPeriod: number;
    sigValidity: number;
    msValidity: number;
    sigQty: number;
    sigStock: number;
    xpercent: number;
    percentRot: number;
    powDelay: number;
    avgGenTime: number;
    medianTimeBlocks: number;
    httplogs: boolean;
    timeout: number;
    isolate: boolean;
    forksize: number;
    idtyWindow: number;
    msWindow: number;
    sigWindow: number;
    switchOnHeadAdvance: number;
    pair: Keypair;
    oldPair: Keypair | null;
    salt: string;
    passwd: string;
    remoteport: number;
    remotehost: string | null;
    remoteipv4: string | null;
    remoteipv6: string | null;
    port: number;
    ipv4: string;
    ipv6: string;
    dos: any;
    upnp: boolean;
    homename: string;
    memory: boolean;
    nobma: boolean;
    bmaWithCrawler: boolean;
    proxiesConf: ProxiesConf | undefined;
    ws2p: {
        privateAccess: boolean;
        publicAccess: boolean;
        uuid: string;
        upnp?: boolean | undefined;
        remotehost?: string | null | undefined;
        remoteport?: number | null | undefined;
        remotepath?: string | undefined;
        port?: number | undefined;
        host?: string | undefined;
        preferedNodes?: string[] | undefined;
        preferedOnly: boolean;
        privilegedNodes?: string[] | undefined;
        privilegedOnly: boolean;
        maxPublic?: number | undefined;
        maxPrivate?: number | undefined;
    } | undefined;
    constructor(loglevel: string, currency: string, endpoints: string[], rmEndpoints: string[], rootoffset: number, upInterval: number, cpu: number, nbCores: number, prefix: number, powSecurityRetryDelay: number, powMaxHandicap: number, c: number, dt: number, dtReeval: number, dtDiffEval: number, ud0: number, udTime0: number, udReevalTime0: number, stepMax: number, sigPeriod: number, msPeriod: number, sigValidity: number, msValidity: number, sigQty: number, sigStock: number, xpercent: number, percentRot: number, powDelay: number, avgGenTime: number, medianTimeBlocks: number, httplogs: boolean, timeout: number, isolate: boolean, forksize: number, idtyWindow: number, msWindow: number, sigWindow: number, switchOnHeadAdvance: number, pair: Keypair, oldPair: Keypair | null, salt: string, passwd: string, remoteport: number, remotehost: string | null, remoteipv4: string | null, remoteipv6: string | null, port: number, ipv4: string, ipv6: string, dos: any, upnp: boolean, homename: string, memory: boolean, nobma: boolean, bmaWithCrawler: boolean, proxiesConf: ProxiesConf | undefined, ws2p?: {
        privateAccess: boolean;
        publicAccess: boolean;
        uuid: string;
        upnp?: boolean | undefined;
        remotehost?: string | null | undefined;
        remoteport?: number | null | undefined;
        remotepath?: string | undefined;
        port?: number | undefined;
        host?: string | undefined;
        preferedNodes?: string[] | undefined;
        preferedOnly: boolean;
        privilegedNodes?: string[] | undefined;
        privilegedOnly: boolean;
        maxPublic?: number | undefined;
        maxPrivate?: number | undefined;
    } | undefined);
    static mock(): ConfDTO;
    static defaultConf(): {
        "currency": null;
        "endpoints": never[];
        "rmEndpoints": never[];
        "upInterval": number;
        "c": any;
        "dt": any;
        "dtReeval": any;
        "ud0": any;
        "stepMax": any;
        "sigPeriod": any;
        "sigValidity": any;
        "msValidity": any;
        "sigQty": any;
        "xpercent": any;
        "percentRot": any;
        "powDelay": any;
        "avgGenTime": any;
        "dtDiffEval": any;
        "medianTimeBlocks": any;
        "httplogs": boolean;
        "udid2": boolean;
        "timeout": number;
        "isolate": boolean;
        "forksize": any;
        "switchOnHeadAdvance": number;
    };
    static complete(conf: any): any;
}
