"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("../common");
const DEFAULT_DOCUMENT_VERSION = 10;
class RevocationDTO {
    constructor(version, currency, pubkey, idty_uid, idty_buid, idty_sig, revocation) {
        this.version = version;
        this.currency = currency;
        this.pubkey = pubkey;
        this.idty_uid = idty_uid;
        this.idty_buid = idty_buid;
        this.idty_sig = idty_sig;
        this.revocation = revocation;
    }
    clone() {
        return RevocationDTO.fromJSONObject(this);
    }
    rawWithoutSig() {
        let raw = "";
        raw += "Version: " + this.version + "\n";
        raw += "Type: Revocation\n";
        raw += "Currency: " + this.currency + "\n";
        raw += "Issuer: " + this.pubkey + "\n";
        raw += "IdtyUniqueID: " + this.idty_uid + '\n';
        raw += "IdtyTimestamp: " + this.idty_buid + '\n';
        raw += "IdtySignature: " + this.idty_sig + '\n';
        return raw;
    }
    getRaw() {
        return this.rawWithoutSig() + this.revocation + "\n";
    }
    // TODO: to remove when BMA has been merged in duniter/duniter repo
    json() {
        return {
            result: true
        };
    }
    static fromInline(inline) {
        const [pubkey, revocation] = inline.split(':');
        return { pubkey, revocation };
    }
    static fromJSONObject(json) {
        return new RevocationDTO(json.version || DEFAULT_DOCUMENT_VERSION, json.currency, json.pubkey || json.issuer, json.idty_uid || json.uid, json.idty_buid || json.buid, json.idty_sig || json.sig, json.revocation || json.revocation);
    }
    getHash() {
        return common_1.hashf(this.getRaw());
    }
}
exports.RevocationDTO = RevocationDTO;
//# sourceMappingURL=RevocationDTO.js.map