import { BlockDTO } from "./dto/BlockDTO";
import { ConfDTO, CurrencyConfDTO } from "./dto/ConfDTO";
import { TransactionDTO } from "./dto/TransactionDTO";
import { DBHead } from "./db/DBHead";
export interface IndexEntry {
    index: string;
    op: string;
    writtenOn: number;
    written_on: string;
}
export interface MindexEntry extends IndexEntry {
    pub: string;
    created_on: string;
    type: string | null;
    expires_on: number | null;
    expired_on: number | null;
    revocation: string | null;
    revokes_on: number | null;
    chainable_on: number | null;
    revoked_on: string | null;
    leaving: boolean | null;
    age: number;
    isBeingRevoked?: boolean;
    unchainables: number;
    numberFollowing?: boolean;
    distanceOK?: boolean;
    onRevoked?: boolean;
    joinsTwice?: boolean;
    enoughCerts?: boolean;
    leaverIsMember?: boolean;
    activeIsMember?: boolean;
    revokedIsMember?: boolean;
    alreadyRevoked?: boolean;
    revocationSigOK?: boolean;
}
export interface IindexEntry extends IndexEntry {
    uid: string | null;
    pub: string;
    hash: string | null;
    sig: string | null;
    created_on: string | null;
    member: boolean;
    wasMember: boolean | null;
    kick: boolean | null;
    wotb_id: number | null;
    age: number;
    pubUnique?: boolean;
    excludedIsMember?: boolean;
    isBeingKicked?: boolean;
    uidUnique?: boolean;
    hasToBeExcluded?: boolean;
}
export interface CindexEntry extends IndexEntry {
    issuer: string;
    receiver: string;
    created_on: number;
    sig: string;
    chainable_on: number;
    expires_on: number;
    expired_on: number;
    from_wid: null;
    to_wid: null;
    unchainables: number;
    age: number;
    stock: number;
    fromMember?: boolean;
    toMember?: boolean;
    toNewcomer?: boolean;
    toLeaver?: boolean;
    isReplay?: boolean;
    sigOK?: boolean;
}
export interface SindexEntry extends IndexEntry {
    tx: string | null;
    identifier: string;
    pos: number;
    created_on: string | null;
    written_time: number;
    locktime: number;
    unlock: string | null;
    amount: number;
    base: number;
    conditions: string;
    consumed: boolean;
    txObj: TransactionDTO;
    age: number;
    type?: string;
    available?: boolean;
    isLocked?: boolean;
    isTimeLocked?: boolean;
}
export interface Ranger {
    (n: number, m: number, prop?: string): Promise<DBHead[]>;
}
export declare class Indexer {
    static localIndex(block: BlockDTO, conf: CurrencyConfDTO): IndexEntry[];
    static localSIndex(block: BlockDTO): SindexEntry[];
    static quickCompleteGlobalScope(block: BlockDTO, conf: CurrencyConfDTO, bindex: DBHead[], iindex: IindexEntry[], mindex: MindexEntry[], cindex: CindexEntry[], dal: any): Promise<DBHead>;
    static completeGlobalScope(block: BlockDTO, conf: ConfDTO, index: IndexEntry[], dal: any): Promise<DBHead>;
    static prepareNumber(HEAD: DBHead, HEAD_1: DBHead): void;
    static prepareIssuersCount(HEAD: DBHead, range: Ranger, HEAD_1: DBHead): Promise<void>;
    static prepareIssuersFrame(HEAD: DBHead, HEAD_1: DBHead): void;
    static prepareIssuersFrameVar(HEAD: DBHead, HEAD_1: DBHead): void;
    static prepareAvgBlockSize(HEAD: DBHead, range: (n: number, m: number, s: string) => Promise<number[]>): Promise<void>;
    static prepareDiffNumber(HEAD: DBHead, HEAD_1: DBHead, conf: CurrencyConfDTO): void;
    static prepareUDTime(HEAD: DBHead, HEAD_1: DBHead, conf: CurrencyConfDTO): void;
    static prepareDividend(HEAD: DBHead, HEAD_1: DBHead, conf: ConfDTO): void;
    static prepareUnitBase(HEAD: DBHead): void;
    static prepareMass(HEAD: DBHead, HEAD_1: DBHead): void;
    static prepareSpeed(HEAD: DBHead, head: (n: number) => Promise<BlockDTO>, conf: CurrencyConfDTO): Promise<void>;
    static preparePersonalizedPoW(HEAD: DBHead, HEAD_1: DBHead, range: (n: number, m: number) => Promise<BlockDTO>, conf: ConfDTO): Promise<void>;
    static prepareIdentitiesAge(iindex: IindexEntry[], HEAD: DBHead, HEAD_1: DBHead, conf: CurrencyConfDTO, dal: any): Promise<void>;
    static prepareMembershipsAge(mindex: MindexEntry[], HEAD: DBHead, HEAD_1: DBHead, conf: CurrencyConfDTO, dal: any): Promise<void>;
    static prepareCertificationsAge(cindex: CindexEntry[], HEAD: DBHead, HEAD_1: DBHead, conf: CurrencyConfDTO, dal: any): Promise<void>;
    static ruleVersion(HEAD: DBHead, HEAD_1: DBHead): boolean;
    static ruleBlockSize(HEAD: DBHead): boolean;
    static ruleCurrency(block: BlockDTO, HEAD: DBHead): boolean;
    static ruleNumber(block: BlockDTO, HEAD: DBHead): boolean;
    static rulePreviousHash(block: BlockDTO, HEAD: DBHead): boolean;
    static rulePreviousIssuer(block: BlockDTO, HEAD: DBHead): boolean;
    static ruleIssuerIsMember(HEAD: DBHead): boolean;
    static ruleIssuersCount(block: BlockDTO, HEAD: DBHead): boolean;
    static ruleIssuersFrame(block: BlockDTO, HEAD: DBHead): boolean;
    static ruleIssuersFrameVar(block: BlockDTO, HEAD: DBHead): boolean;
    static ruleMedianTime(block: BlockDTO, HEAD: DBHead): boolean;
    static ruleDividend(block: BlockDTO, HEAD: DBHead): boolean;
    static ruleUnitBase(block: BlockDTO, HEAD: DBHead): boolean;
    static ruleMembersCount(block: BlockDTO, HEAD: DBHead): boolean;
    static rulePowMin(block: BlockDTO, HEAD: DBHead): boolean;
    static ruleProofOfWork(HEAD: DBHead): boolean;
    static ruleIdentityWritability(iindex: IindexEntry[], conf: ConfDTO): boolean;
    static ruleMembershipWritability(mindex: MindexEntry[], conf: ConfDTO): boolean;
    static ruleMembershipPeriod(mindex: MindexEntry[]): boolean;
    static ruleCertificationWritability(cindex: CindexEntry[], conf: ConfDTO): boolean;
    static ruleCertificationStock(cindex: CindexEntry[], conf: ConfDTO): boolean;
    static ruleCertificationPeriod(cindex: CindexEntry[]): boolean;
    static ruleCertificationFromMember(HEAD: DBHead, cindex: CindexEntry[]): boolean;
    static ruleCertificationToMemberOrNewcomer(cindex: CindexEntry[]): boolean;
    static ruleCertificationToLeaver(cindex: CindexEntry[]): boolean;
    static ruleCertificationReplay(cindex: CindexEntry[]): boolean;
    static ruleCertificationSignature(cindex: CindexEntry[]): boolean;
    static ruleIdentityUIDUnicity(iindex: IindexEntry[]): boolean;
    static ruleIdentityPubkeyUnicity(iindex: IindexEntry[]): boolean;
    static ruleMembershipSuccession(mindex: MindexEntry[]): boolean;
    static ruleMembershipDistance(HEAD: DBHead, mindex: MindexEntry[]): boolean;
    static ruleMembershipOnRevoked(mindex: MindexEntry[]): boolean;
    static ruleMembershipJoinsTwice(mindex: MindexEntry[]): boolean;
    static ruleMembershipEnoughCerts(mindex: MindexEntry[]): boolean;
    static ruleMembershipLeaverIsMember(mindex: MindexEntry[]): boolean;
    static ruleMembershipActiveIsMember(mindex: MindexEntry[]): boolean;
    static ruleMembershipRevokedIsMember(mindex: MindexEntry[]): boolean;
    static ruleMembershipRevokedSingleton(mindex: MindexEntry[]): boolean;
    static ruleMembershipRevocationSignature(mindex: MindexEntry[]): boolean;
    static ruleMembershipExcludedIsMember(iindex: IindexEntry[]): boolean;
    static ruleToBeKickedArePresent(iindex: IindexEntry[], dal: any): Promise<boolean>;
    static ruleTxWritability(sindex: SindexEntry[]): boolean;
    static ruleInputIsAvailable(sindex: SindexEntry[]): boolean;
    static ruleInputIsUnlocked(sindex: SindexEntry[]): boolean;
    static ruleInputIsTimeUnlocked(sindex: SindexEntry[]): boolean;
    static ruleOutputBase(sindex: SindexEntry[], HEAD_1: DBHead): boolean;
    static ruleIndexGenDividend(HEAD: DBHead, dal: any): Promise<{
        op: string;
        identifier: any;
        pos: number;
        written_on: string;
        writtenOn: number;
        written_time: number;
        amount: number;
        base: number;
        locktime: null;
        conditions: string;
        consumed: boolean;
    }[]>;
    static ruleIndexGarbageSmallAccounts(HEAD: DBHead, sindex: SindexEntry[], dal: any): Promise<{
        op: string;
        tx: any;
        identifier: any;
        pos: any;
        amount: any;
        base: any;
        written_on: string;
        writtenOn: number;
        written_time: number;
        conditions: any;
        consumed: boolean;
    }[]>;
    static ruleIndexGenCertificationExpiry(HEAD: DBHead, dal: any): Promise<{
        op: string;
        issuer: any;
        receiver: any;
        created_on: any;
        written_on: string;
        writtenOn: number;
        expired_on: number;
    }[]>;
    static ruleIndexGenMembershipExpiry(HEAD: DBHead, dal: any): Promise<{
        op: string;
        pub: any;
        created_on: any;
        written_on: string;
        writtenOn: number;
        expired_on: number;
    }[]>;
    static ruleIndexGenExclusionByMembership(HEAD: DBHead, mindex: MindexEntry[], dal: any): Promise<{
        op: string;
        pub: any;
        written_on: string;
        writtenOn: number;
        kick: boolean;
    }[]>;
    static ruleIndexGenExclusionByCertificatons(HEAD: DBHead, cindex: CindexEntry[], iindex: IindexEntry[], conf: ConfDTO, dal: any): Promise<{
        op: string;
        pub: any;
        written_on: string;
        writtenOn: number;
        kick: boolean;
    }[]>;
    static ruleIndexGenImplicitRevocation(HEAD: DBHead, dal: any): Promise<{
        op: string;
        pub: any;
        created_on: any;
        written_on: string;
        writtenOn: number;
        revoked_on: number;
    }[]>;
    static ruleIndexCorrectMembershipExpiryDate(HEAD: DBHead, mindex: MindexEntry[], dal: any): Promise<void>;
    static ruleIndexCorrectCertificationExpiryDate(HEAD: DBHead, cindex: CindexEntry[], dal: any): Promise<void>;
    static iindexCreate(index: IndexEntry[]): IindexEntry[];
    static mindexCreate(index: IndexEntry[]): MindexEntry[];
    static iindex(index: IndexEntry[]): IindexEntry[];
    static mindex(index: IndexEntry[]): MindexEntry[];
    static cindex(index: IndexEntry[]): CindexEntry[];
    static sindex(index: IndexEntry[]): SindexEntry[];
    static DUP_HELPERS: {
        reduce: (records: any[]) => any;
        reduceBy: (reducables: IndexEntry[], properties: string[]) => any[];
        getMaxBlockSize: (HEAD: DBHead) => number;
        checkPeopleAreNotOudistanced: (pubkeys: string[], newLinks: any, newcomers: string[], conf: ConfDTO, dal: any) => Promise<boolean>;
    };
}
