"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const dos2unix_1 = require("../../../../lib/common-libs/dos2unix");
const constants_1 = require("../../../../lib/common-libs/constants");
class AbstractController {
    constructor(server) {
        this.server = server;
    }
    get conf() {
        return this.server.conf;
    }
    get logger() {
        return this.server.logger;
    }
    get BlockchainService() {
        return this.server.BlockchainService;
    }
    get IdentityService() {
        return this.server.IdentityService;
    }
    get PeeringService() {
        return this.server.PeeringService;
    }
    get MerkleService() {
        return this.server.MerkleService;
    }
    pushEntity(req, rawer, task) {
        return __awaiter(this, void 0, void 0, function* () {
            let rawDocument = rawer(req);
            rawDocument = dos2unix_1.dos2unix(rawDocument);
            try {
                return yield task(rawDocument);
            }
            catch (e) {
                const event = constants_1.CommonConstants.DocumentError;
                this.server.emit(event, e);
                if (e !== "Block already known" && (!e || !e.uerr || (e.uerr.ucode !== constants_1.CommonConstants.ERRORS.PEER_DOCUMENT_ALREADY_KNOWN.uerr.ucode
                    && e.uerr.ucode !== constants_1.CommonConstants.ERRORS.DOCUMENT_BEING_TREATED.uerr.ucode))) {
                    this.logger.error(e);
                }
                throw e;
            }
        });
    }
}
exports.AbstractController = AbstractController;
//# sourceMappingURL=AbstractController.js.map