"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractController_1 = require("./AbstractController");
const constants_1 = require("../constants");
const _ = require('underscore');
const http2raw = require('../http2raw');
class NetworkBinding extends AbstractController_1.AbstractController {
    peer() {
        return __awaiter(this, void 0, void 0, function* () {
            const p = yield this.PeeringService.peer();
            if (!p) {
                throw constants_1.BMAConstants.ERRORS.SELF_PEER_NOT_FOUND;
            }
            return p.json();
        });
    }
    peersGet(req) {
        return __awaiter(this, void 0, void 0, function* () {
            let merkle = yield this.server.dal.merkleForPeers();
            return yield this.MerkleService(req, merkle, (hashes) => __awaiter(this, void 0, void 0, function* () {
                try {
                    let peers = yield this.server.dal.findPeersWhoseHashIsIn(hashes);
                    const map = {};
                    peers.forEach((peer) => {
                        map[peer.hash] = peer;
                    });
                    if (peers.length == 0) {
                        throw constants_1.BMAConstants.ERRORS.PEER_NOT_FOUND;
                    }
                    return map;
                }
                catch (e) {
                    throw e;
                }
            }));
        });
    }
    peersPost(req) {
        return __awaiter(this, void 0, void 0, function* () {
            const peerDTO = yield this.pushEntity(req, http2raw.peer, (raw) => this.server.writeRawPeer(raw));
            return {
                version: peerDTO.version,
                currency: peerDTO.currency,
                pubkey: peerDTO.pubkey,
                block: peerDTO.blockstamp,
                endpoints: peerDTO.endpoints,
                signature: peerDTO.signature,
                raw: peerDTO.getRaw()
            };
        });
    }
    peers() {
        return __awaiter(this, void 0, void 0, function* () {
            let peers = yield this.server.dal.listAllPeers();
            return {
                peers: peers.map((p) => {
                    return _.pick(p, 'version', 'currency', 'status', 'first_down', 'last_try', 'pubkey', 'block', 'signature', 'endpoints');
                })
            };
        });
    }
    ws2pInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            const cluster = this.server.ws2pCluster;
            let level1 = 0;
            let level2 = 0;
            if (cluster) {
                level1 = yield cluster.clientsCount();
                level2 = yield cluster.servedCount();
            }
            return {
                peers: {
                    level1,
                    level2
                }
            };
        });
    }
    ws2pHeads() {
        return __awaiter(this, void 0, void 0, function* () {
            const cluster = this.server.ws2pCluster;
            let heads = [];
            if (cluster) {
                heads = yield cluster.getKnownHeads();
            }
            return { heads };
        });
    }
}
exports.NetworkBinding = NetworkBinding;
//# sourceMappingURL=network.js.map