"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractController_1 = require("./AbstractController");
const parameters_1 = require("../parameters");
const source_1 = require("../entity/source");
const _ = require('underscore');
class UDBinding extends AbstractController_1.AbstractController {
    getHistory(req) {
        return __awaiter(this, void 0, void 0, function* () {
            const pubkey = yield parameters_1.ParametersService.getPubkeyP(req);
            return this.getUDSources(pubkey, (results) => results);
        });
    }
    getHistoryBetweenBlocks(req) {
        return __awaiter(this, void 0, void 0, function* () {
            const pubkey = yield parameters_1.ParametersService.getPubkeyP(req);
            const from = yield parameters_1.ParametersService.getFromP(req);
            const to = yield parameters_1.ParametersService.getToP(req);
            return this.getUDSources(pubkey, (results) => {
                results.history.history = _.filter(results.history.history, function (ud) { return ud.block_number >= from && ud.block_number <= to; });
                return results;
            });
        });
    }
    getHistoryBetweenTimes(req) {
        return __awaiter(this, void 0, void 0, function* () {
            const pubkey = yield parameters_1.ParametersService.getPubkeyP(req);
            const from = yield parameters_1.ParametersService.getFromP(req);
            const to = yield parameters_1.ParametersService.getToP(req);
            return this.getUDSources(pubkey, (results) => {
                results.history.history = _.filter(results.history.history, function (ud) { return ud.time >= from && ud.time <= to; });
                return results;
            });
        });
    }
    getUDSources(pubkey, filter) {
        return __awaiter(this, void 0, void 0, function* () {
            const history = yield this.server.dal.getUDHistory(pubkey);
            const result = {
                "currency": this.conf.currency,
                "pubkey": pubkey,
                "history": history
            };
            _.keys(history).map((key) => {
                history[key].map((src, index) => {
                    history[key][index] = _.omit(new source_1.Source(src).UDjson(), 'currency', 'raw');
                    _.extend(history[key][index], { block_number: src && src.block_number, time: src && src.time });
                });
            });
            return filter(result);
        });
    }
}
exports.UDBinding = UDBinding;
//# sourceMappingURL=uds.js.map