"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("./constants");
const upnp = require('nnupnp');
const Q = require('q');
exports.Upnp = function (localPort, remotePort, logger) {
    "use strict";
    return __awaiter(this, void 0, void 0, function* () {
        logger.info('UPnP: configuring...');
        const api = new UpnpApi(localPort, remotePort, logger);
        try {
            yield api.openPort();
        }
        catch (e) {
            const client = upnp.createClient();
            try {
                yield Q.nbind(client.externalIp, client)();
            }
            catch (err) {
                if (err && err.message == 'timeout') {
                    throw 'No UPnP gateway found: your node won\'t be reachable from the Internet. Use --noupnp option to avoid this message.';
                }
                throw err;
            }
            finally {
                client.close();
            }
        }
        return api;
    });
};
class UpnpApi {
    constructor(localPort, remotePort, logger) {
        this.localPort = localPort;
        this.remotePort = remotePort;
        this.logger = logger;
    }
    openPort() {
        "use strict";
        return Q.Promise((resolve, reject) => {
            this.logger.trace('UPnP: mapping external port %s to local %s...', this.remotePort, this.localPort);
            const client = upnp.createClient();
            client.portMapping({
                'public': this.remotePort,
                'private': this.localPort,
                'ttl': constants_1.BMAConstants.UPNP_TTL,
                'description': 'duniter:bma:upnp'
            }, (err) => {
                client.close();
                if (err) {
                    this.logger.warn(err);
                    return reject(err);
                }
                resolve();
            });
        });
    }
    findGateway() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const client = upnp.createClient();
                const res = yield Q.nbind(client.findGateway, client)();
                const desc = res && res[0] && res[0].description;
                if (desc) {
                    const match = desc.match(/(\d+.\d+.\d+.\d+):/);
                    if (match) {
                        return match[1];
                    }
                }
                return null;
            }
            catch (e) {
                return null;
            }
        });
    }
    startRegular() {
        this.stopRegular();
        // Update UPnP IGD every INTERVAL seconds
        this.interval = setInterval(() => this.openPort(), 1000 * constants_1.BMAConstants.UPNP_INTERVAL);
    }
    stopRegular() {
        if (this.interval) {
            clearInterval(this.interval);
        }
    }
}
exports.UpnpApi = UpnpApi;
//# sourceMappingURL=upnp.js.map