"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const contacter_1 = require("./lib/contacter");
const crawler_1 = require("./lib/crawler");
const sync_1 = require("./lib/sync");
const req2fwd_1 = require("./lib/req2fwd");
const index_1 = require("../../lib/common-libs/index");
const PeerDTO_1 = require("../../lib/dto/PeerDTO");
const buid_1 = require("../../lib/common-libs/buid");
const BlockDTO_1 = require("../../lib/dto/BlockDTO");
exports.CrawlerDependency = {
    duniter: {
        service: {
            process: (server, conf, logger) => new crawler_1.Crawler(server, conf, logger)
        },
        methods: {
            contacter: (host, port, opts) => new contacter_1.Contacter(host, port, opts),
            pullBlocks: (server, pubkey) => __awaiter(this, void 0, void 0, function* () {
                const crawler = new crawler_1.Crawler(server, server.conf, server.logger);
                return crawler.pullBlocks(server, pubkey);
            }),
            pullSandbox: (server) => __awaiter(this, void 0, void 0, function* () {
                const crawler = new crawler_1.Crawler(server, server.conf, server.logger);
                return crawler.sandboxPull(server);
            }),
            synchronize: (server, onHost, onPort, upTo, chunkLength) => {
                const remote = new sync_1.Synchroniser(server, onHost, onPort);
                const syncPromise = remote.sync(upTo, chunkLength);
                return {
                    flow: remote,
                    syncPromise: syncPromise
                };
            },
            testForSync: (server, onHost, onPort) => {
                const remote = new sync_1.Synchroniser(server, onHost, onPort);
                return remote.test();
            }
        },
        cliOptions: [
            { value: '--nointeractive', desc: 'Disable interactive sync UI.' },
            { value: '--nocautious', desc: 'Do not check blocks validity during sync.' },
            { value: '--cautious', desc: 'Check blocks validity during sync (overrides --nocautious option).' },
            { value: '--nopeers', desc: 'Do not retrieve peers during sync.' },
            { value: '--onlypeers', desc: 'Will only try to sync peers.' },
            { value: '--slow', desc: 'Download slowly the blokchcain (for low connnections).' },
            { value: '--minsig <minsig>', desc: 'Minimum pending signatures count for `crawl-lookup`. Default is 5.' }
        ],
        cli: [{
                name: 'sync [host] [port] [to]',
                desc: 'Synchronize blockchain from a remote Duniter node',
                preventIfRunning: true,
                onDatabaseExecute: (server, conf, program, params) => __awaiter(this, void 0, void 0, function* () {
                    const host = params[0];
                    const port = params[1];
                    const to = params[2];
                    if (!host) {
                        throw 'Host is required.';
                    }
                    if (!port) {
                        throw 'Port is required.';
                    }
                    let cautious;
                    if (program.nocautious) {
                        cautious = false;
                    }
                    if (program.cautious) {
                        cautious = true;
                    }
                    const onHost = host;
                    const onPort = port;
                    const upTo = parseInt(to);
                    const chunkLength = 0;
                    const interactive = !program.nointeractive;
                    const askedCautious = cautious;
                    const nopeers = program.nopeers;
                    const noShufflePeers = program.noshuffle;
                    const remote = new sync_1.Synchroniser(server, onHost, onPort, interactive === true, program.slow === true);
                    if (program.onlypeers === true) {
                        return remote.syncPeers(nopeers, true, onHost, onPort);
                    }
                    else {
                        return remote.sync(upTo, chunkLength, askedCautious, nopeers, noShufflePeers === true);
                    }
                })
            }, {
                name: 'peer [host] [port]',
                desc: 'Exchange peerings with another node',
                preventIfRunning: true,
                onDatabaseExecute: (server, conf, program, params) => __awaiter(this, void 0, void 0, function* () {
                    const host = params[0];
                    const port = params[1];
                    const logger = server.logger;
                    try {
                        const ERASE_IF_ALREADY_RECORDED = true;
                        logger.info('Fetching peering record at %s:%s...', host, port);
                        let peering = yield contacter_1.Contacter.fetchPeer(host, port);
                        logger.info('Apply peering ...');
                        yield server.PeeringService.submitP(peering, ERASE_IF_ALREADY_RECORDED, !program.nocautious);
                        logger.info('Applied');
                        let selfPeer = yield server.dal.getPeer(server.PeeringService.pubkey);
                        if (!selfPeer) {
                            yield server.PeeringService.generateSelfPeer(server.conf);
                            selfPeer = yield server.dal.getPeer(server.PeeringService.pubkey);
                        }
                        logger.info('Send self peering ...');
                        const p = PeerDTO_1.PeerDTO.fromJSONObject(peering);
                        const contact = new contacter_1.Contacter(p.getHostPreferDNS(), p.getPort(), {});
                        yield contact.postPeer(PeerDTO_1.PeerDTO.fromJSONObject(selfPeer));
                        logger.info('Sent.');
                        yield server.disconnect();
                    }
                    catch (e) {
                        logger.error(e.code || e.message || e);
                        throw Error("Exiting");
                    }
                })
            }, {
                name: 'import <fromHost> <fromPort> <search> <toHost> <toPort>',
                desc: 'Import all pending data from matching <search>',
                onDatabaseExecute: (server, conf, program, params) => __awaiter(this, void 0, void 0, function* () {
                    const fromHost = params[0];
                    const fromPort = params[1];
                    const search = params[2];
                    const toHost = params[3];
                    const toPort = params[4];
                    const logger = server.logger;
                    try {
                        const peers = fromHost && fromPort ? [{ endpoints: [['BASIC_MERKLED_API', fromHost, fromPort].join(' ')] }] : yield server.dal.peerDAL.query('SELECT * FROM peer WHERE status = ?', ['UP']);
                        // Memberships
                        for (const p of peers) {
                            const peer = PeerDTO_1.PeerDTO.fromJSONObject(p);
                            const fromHost = peer.getHostPreferDNS();
                            const fromPort = peer.getPort();
                            logger.info('Looking at %s:%s...', fromHost, fromPort);
                            try {
                                const node = new contacter_1.Contacter(fromHost, fromPort, { timeout: 10000 });
                                const requirements = yield node.getRequirements(search);
                                yield req2fwd_1.req2fwd(requirements, toHost, toPort, logger);
                            }
                            catch (e) {
                                logger.error(e);
                            }
                        }
                        yield server.disconnect();
                    }
                    catch (e) {
                        logger.error(e);
                        throw Error("Exiting");
                    }
                })
            }, {
                name: 'forward <number> <fromHost> <fromPort> <toHost> <toPort>',
                desc: 'Forward existing block <number> from a host to another',
                onDatabaseExecute: (server, conf, program, params) => __awaiter(this, void 0, void 0, function* () {
                    const number = params[0];
                    const fromHost = params[1];
                    const fromPort = params[2];
                    const toHost = params[3];
                    const toPort = params[4];
                    const logger = server.logger;
                    try {
                        logger.info('Looking at %s:%s...', fromHost, fromPort);
                        try {
                            const source = new contacter_1.Contacter(fromHost, fromPort, { timeout: 10000 });
                            const target = new contacter_1.Contacter(toHost, toPort, { timeout: 10000 });
                            const block = yield source.getBlock(number);
                            const raw = BlockDTO_1.BlockDTO.fromJSONObject(block).getRawSigned();
                            yield target.postBlock(raw);
                        }
                        catch (e) {
                            logger.error(e);
                        }
                        yield server.disconnect();
                    }
                    catch (e) {
                        logger.error(e);
                        throw Error("Exiting");
                    }
                })
            }, {
                name: 'import-lookup [search] [fromhost] [fromport] [tohost] [toport]',
                desc: 'Exchange peerings with another node',
                onDatabaseExecute: (server, conf, program, params) => __awaiter(this, void 0, void 0, function* () {
                    const search = params[0];
                    const fromhost = params[1];
                    const fromport = params[2];
                    const tohost = params[3];
                    const toport = params[4];
                    const logger = server.logger;
                    try {
                        logger.info('Looking for "%s" at %s:%s...', search, fromhost, fromport);
                        const sourcePeer = new contacter_1.Contacter(fromhost, fromport);
                        const targetPeer = new contacter_1.Contacter(tohost, toport);
                        const lookup = yield sourcePeer.getLookup(search);
                        for (const res of lookup.results) {
                            for (const uid of res.uids) {
                                const rawIdty = index_1.rawer.getOfficialIdentity({
                                    currency: 'g1',
                                    issuer: res.pubkey,
                                    uid: uid.uid,
                                    buid: uid.meta.timestamp,
                                    sig: uid.self
                                });
                                logger.info('Success idty %s', uid.uid);
                                try {
                                    yield targetPeer.postIdentity(rawIdty);
                                }
                                catch (e) {
                                    logger.error(e);
                                }
                                for (const received of uid.others) {
                                    const rawCert = index_1.rawer.getOfficialCertification({
                                        currency: 'g1',
                                        issuer: received.pubkey,
                                        idty_issuer: res.pubkey,
                                        idty_uid: uid.uid,
                                        idty_buid: uid.meta.timestamp,
                                        idty_sig: uid.self,
                                        buid: buid_1.Buid.format.buid(received.meta.block_number, received.meta.block_hash),
                                        sig: received.signature
                                    });
                                    try {
                                        logger.info('Success cert %s -> %s', received.pubkey.slice(0, 8), uid.uid);
                                        yield targetPeer.postCert(rawCert);
                                    }
                                    catch (e) {
                                        logger.error(e);
                                    }
                                }
                            }
                        }
                        const certBy = yield sourcePeer.getCertifiedBy(search);
                        const mapBlocks = {};
                        for (const signed of certBy.certifications) {
                            if (signed.written) {
                                logger.info('Already written cert %s -> %s', certBy.pubkey.slice(0, 8), signed.uid);
                            }
                            else {
                                const lookupIdty = yield sourcePeer.getLookup(signed.pubkey);
                                let idty = null;
                                for (const result of lookupIdty.results) {
                                    for (const uid of result.uids) {
                                        if (uid.uid === signed.uid && result.pubkey === signed.pubkey && uid.meta.timestamp === signed.sigDate) {
                                            idty = uid;
                                        }
                                    }
                                }
                                let block = mapBlocks[signed.cert_time.block];
                                if (!block) {
                                    block = yield sourcePeer.getBlock(signed.cert_time.block);
                                    mapBlocks[block.number] = block;
                                }
                                const rawCert = index_1.rawer.getOfficialCertification({
                                    currency: 'g1',
                                    issuer: certBy.pubkey,
                                    idty_issuer: signed.pubkey,
                                    idty_uid: signed.uid,
                                    idty_buid: idty.meta.timestamp,
                                    idty_sig: idty.self,
                                    buid: buid_1.Buid.format.buid(block.number, block.hash),
                                    sig: signed.signature
                                });
                                try {
                                    logger.info('Success cert %s -> %s', certBy.pubkey.slice(0, 8), signed.uid);
                                    yield targetPeer.postCert(rawCert);
                                }
                                catch (e) {
                                    logger.error(e);
                                }
                            }
                        }
                        logger.info('Sent.');
                        yield server.disconnect();
                    }
                    catch (e) {
                        logger.error(e);
                        throw Error("Exiting");
                    }
                })
            }, {
                name: 'crawl-lookup <toHost> <toPort> [<fromHost> [<fromPort>]]',
                desc: 'Make a full network scan and rebroadcast every WoT pending document (identity, certification, membership)',
                onDatabaseExecute: (server, conf, program, params) => __awaiter(this, void 0, void 0, function* () {
                    const toHost = params[0];
                    const toPort = params[1];
                    const fromHost = params[2];
                    const fromPort = params[3];
                    const logger = server.logger;
                    try {
                        const peers = fromHost && fromPort ? [{ endpoints: [['BASIC_MERKLED_API', fromHost, fromPort].join(' ')] }] : yield server.dal.peerDAL.query('SELECT * FROM peer WHERE status = ?', ['UP']);
                        // Memberships
                        for (const p of peers) {
                            const peer = PeerDTO_1.PeerDTO.fromJSONObject(p);
                            const fromHost = peer.getHostPreferDNS();
                            const fromPort = peer.getPort();
                            logger.info('Looking at %s:%s...', fromHost, fromPort);
                            try {
                                const node = new contacter_1.Contacter(fromHost, fromPort, { timeout: 10000 });
                                const requirements = yield node.getRequirementsPending(program.minsig || 5);
                                yield req2fwd_1.req2fwd(requirements, toHost, toPort, logger);
                            }
                            catch (e) {
                                logger.error(e);
                            }
                        }
                        yield server.disconnect();
                    }
                    catch (e) {
                        logger.error(e);
                        throw Error("Exiting");
                    }
                })
            }, {
                name: 'fwd-pending-ms',
                desc: 'Forwards all the local pending memberships to target node',
                onDatabaseExecute: (server, conf, program, params) => __awaiter(this, void 0, void 0, function* () {
                    const logger = server.logger;
                    try {
                        const pendingMSS = yield server.dal.msDAL.getPendingIN();
                        const targetPeer = new contacter_1.Contacter('g1.cgeek.fr', 80, { timeout: 5000 });
                        // Membership
                        let rawMS;
                        for (const theMS of pendingMSS) {
                            console.log('New membership pending for %s', theMS.uid);
                            try {
                                rawMS = index_1.rawer.getMembership({
                                    currency: 'g1',
                                    issuer: theMS.issuer,
                                    block: theMS.block,
                                    membership: theMS.membership,
                                    userid: theMS.userid,
                                    certts: theMS.certts,
                                    signature: theMS.signature
                                });
                                yield targetPeer.postRenew(rawMS);
                                logger.info('Success ms idty %s', theMS.userid);
                            }
                            catch (e) {
                                logger.warn(e);
                            }
                        }
                        yield server.disconnect();
                    }
                    catch (e) {
                        logger.error(e);
                        throw Error("Exiting");
                    }
                })
            }]
    }
};
//# sourceMappingURL=index.js.map