"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("./constants");
const rp = require('request-promise');
const sanitize = require('../../../modules/bma/lib/sanitize');
const dtos = require('../../../modules/bma').BmaDependency.duniter.methods.dtos;
class Contacter {
    constructor(host, port, opts = {}) {
        this.host = host;
        this.port = port;
        this.options = {
            timeout: (opts && opts.timeout) || constants_1.CrawlerConstants.DEFAULT_TIMEOUT
        };
        // We suppose that IPv6 is already wrapped by [], for example 'http://[::1]:80/index.html'
        this.fullyQualifiedHost = [host, port].join(':');
    }
    getSummary() {
        return this.get('/node/summary/', dtos.Summary);
    }
    getCertifiedBy(search) {
        return this.get('/wot/certified-by/' + search, dtos.Certifications);
    }
    getRequirements(search) {
        return this.get('/wot/requirements/' + search, dtos.Requirements);
    }
    getRequirementsPending(minsig) {
        return this.get('/wot/requirements-of-pending/' + minsig, dtos.Requirements);
    }
    getLookup(search) {
        return this.get('/wot/lookup/', dtos.Lookup, search);
    }
    getBlock(number) {
        return this.get('/blockchain/block/', dtos.Block, number);
    }
    getCurrent() {
        return this.get('/blockchain/current', dtos.Block);
    }
    getPeer() {
        return this.get('/network/peering', dtos.Peer);
    }
    getPeers(obj) {
        return this.get('/network/peering/peers', dtos.MerkleOfPeers, obj);
    }
    getSources(pubkey) {
        return this.get('/tx/sources/', dtos.Sources, pubkey);
    }
    getBlocks(count, fromNumber) {
        return this.get('/blockchain/blocks/', dtos.Blocks, [count, fromNumber].join('/'));
    }
    postPeer(peer) {
        return this.post('/network/peering/peers', dtos.Peer, { peer: peer });
    }
    postIdentity(raw) {
        return this.post('/wot/add', dtos.Identity, { identity: raw });
    }
    postCert(cert) {
        return this.post('/wot/certify', dtos.Cert, { cert: cert });
    }
    postRenew(ms) {
        return this.post('/blockchain/membership', dtos.Membership, { membership: ms });
    }
    wotPending() {
        return this.get('/wot/pending', dtos.MembershipList);
    }
    wotMembers() {
        return this.get('/wot/members', dtos.Members);
    }
    postBlock(rawBlock) {
        return this.post('/blockchain/block', dtos.Block, { block: rawBlock });
    }
    processTransaction(rawTX) {
        return this.post('/tx/process', dtos.Transaction, { transaction: rawTX });
    }
    get(url, dtoContract, param) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof param === 'object') {
                // Classical URL params (a=1&b=2&...)
                param = '?' + Object.keys(param).map((k) => [k, param[k]].join('=')).join('&');
            }
            try {
                const json = yield rp.get({
                    url: Contacter.protocol(this.port) + this.fullyQualifiedHost + url + (param !== undefined ? param : ''),
                    json: true,
                    timeout: this.options.timeout
                });
                // Prevent JSON injection
                return sanitize(json, dtoContract);
            }
            catch (e) {
                throw e.error;
            }
        });
    }
    post(url, dtoContract, data) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const json = yield rp.post({
                    url: Contacter.protocol(this.port) + this.fullyQualifiedHost + url,
                    body: data,
                    json: true,
                    timeout: this.options.timeout
                });
                // Prevent JSON injection
                return sanitize(json, dtoContract);
            }
            catch (e) {
                throw e.error;
            }
        });
    }
    static protocol(port) {
        return port == 443 ? 'https://' : 'http://';
    }
    static quickly(host, port, opts, callbackPromise) {
        return __awaiter(this, void 0, void 0, function* () {
            const node = new Contacter(host, port, opts);
            return callbackPromise(node);
        });
    }
    static quickly2(peer, opts, callbackPromise) {
        return __awaiter(this, void 0, void 0, function* () {
            const Peer = require('./entity/peer');
            const p = Peer.fromJSON(peer);
            const node = new Contacter(p.getHostPreferDNS(), p.getPort(), opts);
            return callbackPromise(node);
        });
    }
    static fetchPeer(host, port, opts = {}) {
        return Contacter.quickly(host, port, opts, (node) => node.getPeer());
    }
    static fetchBlock(number, peer, opts = {}) {
        return Contacter.quickly2(peer, opts, (node) => node.getBlock(number));
    }
    static isReachableFromTheInternet(peer, opts) {
        return __awaiter(this, void 0, void 0, function* () {
            const Peer = require('./entity/peer');
            const p = Peer.fromJSON(peer);
            const node = new Contacter(p.getHostPreferDNS(), p.getPort(), opts);
            try {
                yield node.getPeer();
                return true;
            }
            catch (e) {
                return false;
            }
        });
    }
}
exports.Contacter = Contacter;
//# sourceMappingURL=contacter.js.map