"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const contacter_1 = require("./contacter");
const keyring_1 = require("../../../lib/common-libs/crypto/keyring");
const index_1 = require("../../../lib/common-libs/index");
exports.req2fwd = (requirements, toHost, toPort, logger) => __awaiter(this, void 0, void 0, function* () {
    const mss = {};
    const identities = {};
    const certs = {};
    const targetPeer = new contacter_1.Contacter(toHost, toPort, { timeout: 10000 });
    // Identities
    for (const idty of requirements.identities) {
        try {
            const iid = [idty.pubkey, idty.uid, idty.meta.timestamp].join('-');
            if (!identities[iid]) {
                logger.info('New identity %s', idty.uid);
                identities[iid] = idty;
                try {
                    const rawIdty = index_1.rawer.getOfficialIdentity({
                        currency: 'g1',
                        issuer: idty.pubkey,
                        uid: idty.uid,
                        buid: idty.meta.timestamp,
                        sig: idty.sig
                    });
                    yield targetPeer.postIdentity(rawIdty);
                    logger.info('Success idty %s', idty.uid);
                }
                catch (e) {
                    logger.warn('Rejected idty %s...', idty.uid, e);
                }
            }
            for (const received of idty.pendingCerts) {
                const cid = [received.from, iid].join('-');
                if (!certs[cid]) {
                    yield new Promise((res) => setTimeout(res, 300));
                    certs[cid] = received;
                    const rawCert = index_1.rawer.getOfficialCertification({
                        currency: 'g1',
                        issuer: received.from,
                        idty_issuer: idty.pubkey,
                        idty_uid: idty.uid,
                        idty_buid: idty.meta.timestamp,
                        idty_sig: idty.sig,
                        buid: received.blockstamp,
                        sig: received.sig
                    });
                    const rawCertNoSig = index_1.rawer.getOfficialCertification({
                        currency: 'g1',
                        issuer: received.from,
                        idty_issuer: idty.pubkey,
                        idty_uid: idty.uid,
                        idty_buid: idty.meta.timestamp,
                        idty_sig: idty.sig,
                        buid: received.blockstamp
                    });
                    try {
                        const chkSig = keyring_1.verify(rawCertNoSig, received.sig, received.from);
                        if (!chkSig) {
                            throw "Wrong signature for certification?!";
                        }
                        yield targetPeer.postCert(rawCert);
                        logger.info('Success cert %s -> %s', received.from, idty.uid);
                    }
                    catch (e) {
                        logger.warn('Rejected cert %s -> %s', received.from, idty.uid, received.blockstamp.substr(0, 18), e);
                    }
                }
            }
            for (const theMS of idty.pendingMemberships) {
                // + Membership
                const id = [idty.pubkey, idty.uid, theMS.blockstamp].join('-');
                if (!mss[id]) {
                    mss[id] = theMS;
                    try {
                        const rawMS = index_1.rawer.getMembership({
                            currency: 'g1',
                            issuer: idty.pubkey,
                            userid: idty.uid,
                            block: theMS.blockstamp,
                            membership: theMS.type,
                            certts: idty.meta.timestamp,
                            signature: theMS.sig
                        });
                        yield targetPeer.postRenew(rawMS);
                        logger.info('Success ms idty %s', idty.uid);
                    }
                    catch (e) {
                        logger.warn('Rejected ms idty %s', idty.uid, e);
                    }
                }
            }
        }
        catch (e) {
            logger.warn(e);
        }
    }
});
//# sourceMappingURL=req2fwd.js.map