"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const base58_1 = require("../../../lib/common-libs/crypto/base58");
const nacl_util_1 = require("../../../lib/common-libs/crypto/nacl-util");
const nacl = require('tweetnacl');
const scrypt = require('scryptb');
const SEED_LENGTH = 32; // Length of the key
/**
 * Generates a new keypair object from salt + password strings.
 * @param salt
 * @param key
 * @param N Scrypt parameter N. Defaults to 4096.
 * @param r Scrypt parameter r. Defaults to 16.
 * @param p Scrypt parameter p. Defaults to 1.
 * @return keyPair An object containing the public and private keys, base58 encoded.
 */
exports.Scrypt = (salt, key, N = 4096, r = 16, p = 1) => __awaiter(this, void 0, void 0, function* () {
    const keyBytes = yield getScryptKey(key, salt, N, r, p);
    const pair = nacl.sign.keyPair.fromSeed(keyBytes);
    return {
        pub: base58_1.Base58encode(new Buffer(pair.publicKey, 'hex')),
        sec: base58_1.Base58encode(new Buffer(pair.secretKey, 'hex'))
    };
});
const getScryptKey = (key, salt, N, r, p) => __awaiter(this, void 0, void 0, function* () {
    const res = yield new Promise((resolve, reject) => {
        scrypt.hash(key, { N, r, p }, SEED_LENGTH, salt, (err, res) => {
            if (err)
                return reject(err);
            resolve(res);
        });
    });
    return nacl_util_1.decodeBase64(res.toString("base64"));
});
//# sourceMappingURL=scrypt.js.map