"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const pulling_1 = require("../../crawler/lib/pulling");
const constants_1 = require("../../crawler/lib/constants");
const tx_cleaner_1 = require("../../crawler/lib/tx_cleaner");
const WS2PRequester_1 = require("./WS2PRequester");
class WS2PBlockPuller {
    constructor(server, connection) {
        this.server = server;
        this.connection = connection;
    }
    pull() {
        return __awaiter(this, void 0, void 0, function* () {
            const requester = WS2PRequester_1.WS2PRequester.fromConnection(this.connection);
            // node.pubkey = p.pubkey;
            let dao = new WS2PDao(this.server, requester);
            yield dao.pull(this.server.conf, this.server.logger);
        });
    }
}
exports.WS2PBlockPuller = WS2PBlockPuller;
class WS2PDao extends pulling_1.AbstractDAO {
    constructor(server, requester) {
        super();
        this.server = server;
        this.requester = requester;
        this.nodeCurrent = null;
        this.newCurrent = null;
        this.node = {
            getCurrent: () => __awaiter(this, void 0, void 0, function* () {
                return this.requester.getCurrent();
            }),
            getBlock: (number) => __awaiter(this, void 0, void 0, function* () {
                return this.requester.getBlock(number);
            }),
            getBlocks: (count, fromNumber) => __awaiter(this, void 0, void 0, function* () {
                return this.requester.getBlocks(count, fromNumber);
            }),
            pubkey: this.requester.getPubkey()
        };
    }
    localCurrent() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.server.dal.getCurrentBlockOrNull();
        });
    }
    remoteCurrent(source) {
        return __awaiter(this, void 0, void 0, function* () {
            this.nodeCurrent = yield source.getCurrent();
            return this.nodeCurrent;
        });
    }
    remotePeers(source) {
        return __awaiter(this, void 0, void 0, function* () {
            const peer = this.node;
            return Promise.resolve([peer]);
        });
    }
    getLocalBlock(number) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.server.dal.getBlock(number);
        });
    }
    getRemoteBlock(thePeer, number) {
        return __awaiter(this, void 0, void 0, function* () {
            let block = null;
            try {
                block = yield thePeer.getBlock(number);
                tx_cleaner_1.tx_cleaner(block.transactions);
            }
            catch (e) {
                if (e.httpCode != 404) {
                    throw e;
                }
            }
            return block;
        });
    }
    applyMainBranch(block) {
        return __awaiter(this, void 0, void 0, function* () {
            const existing = yield this.server.dal.getAbsoluteBlockByNumberAndHash(block.number, block.hash);
            if (!existing) {
                let addedBlock = yield this.server.writeBlock(block, false, true);
                if (!this.lastDownloaded) {
                    this.lastDownloaded = yield this.remoteCurrent(this.node);
                }
                this.server.pullingEvent('applying', { number: block.number, last: this.lastDownloaded && this.lastDownloaded.number });
                if (addedBlock) {
                    this.newCurrent = addedBlock;
                    // Emit block events (for sharing with the network) only in forkWindowSize
                    if (this.nodeCurrent && this.nodeCurrent.number - addedBlock.number < this.server.conf.forksize) {
                        this.server.streamPush(addedBlock);
                    }
                }
            }
            return true;
        });
    }
    removeForks() {
        return __awaiter(this, void 0, void 0, function* () {
            return true;
        });
    }
    isMemberPeer(thePeer) {
        return __awaiter(this, void 0, void 0, function* () {
            return true;
        });
    }
    downloadBlocks(thePeer, fromNumber, count) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!count) {
                count = constants_1.CrawlerConstants.CRAWL_BLOCK_CHUNK;
            }
            let blocks = yield thePeer.getBlocks(count, fromNumber);
            // Fix for #734
            for (const block of blocks) {
                for (const tx of block.transactions) {
                    tx.version = constants_1.CrawlerConstants.TRANSACTION_VERSION;
                }
            }
            return blocks;
        });
    }
}
//# sourceMappingURL=WS2PBlockPuller.js.map