"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const WS2PConnection_1 = require("./WS2PConnection");
const keyring_1 = require("../../../lib/common-libs/crypto/keyring");
const constants_1 = require("./constants");
const WS2PStreamer_1 = require("./WS2PStreamer");
const WS2PSingleWriteStream_1 = require("./WS2PSingleWriteStream");
const proxy_1 = require("../../../lib/proxy");
class WS2PClient {
    constructor(connection) {
        this.connection = connection;
    }
    static connectTo(server, fullEndpointAddress, uuid, messageHandler, expectedPub, allowKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const k2 = new keyring_1.Key(server.conf.pair.pub, server.conf.pair.sec);
            const c = WS2PConnection_1.WS2PConnection.newConnectionToAddress(fullEndpointAddress, messageHandler, new WS2PConnection_1.WS2PPubkeyLocalAuth(server.conf.currency, k2, allowKey), new WS2PConnection_1.WS2PPubkeyRemoteAuth(server.conf.currency, k2, allowKey), proxy_1.ProxiesConf.wsProxy(fullEndpointAddress, server.conf.proxiesConf), {
                connectionTimeout: constants_1.WS2PConstants.REQUEST_TIMEOUT,
                requestTimeout: constants_1.WS2PConstants.REQUEST_TIMEOUT
            }, expectedPub);
            const singleWriteProtection = new WS2PSingleWriteStream_1.WS2PSingleWriteStream();
            const streamer = new WS2PStreamer_1.WS2PStreamer(c);
            c.connected
                .then(() => {
                // Streaming
                server
                    .pipe(singleWriteProtection)
                    .pipe(streamer);
            })
                .catch(() => {
                server.unpipe(singleWriteProtection);
                singleWriteProtection.unpipe(streamer);
            });
            c.closed.then(() => {
                server.unpipe(singleWriteProtection);
                singleWriteProtection.unpipe(streamer);
            });
            // Connecting
            try {
                yield c.connect();
            }
            catch (e) {
                // Immediately close the connection
                c.close();
                throw e;
            }
            return new WS2PClient(c);
        });
    }
}
exports.WS2PClient = WS2PClient;
//# sourceMappingURL=WS2PClient.js.map