import { WS2PServer } from "./WS2PServer";
import { Server } from "../../../../server";
import { WS2PConnection } from "./WS2PConnection";
import { WS2PMessageHandler } from "./impl/WS2PMessageHandler";
export interface WS2PHead {
    message: string;
    sig: string;
}
export declare class WS2PCluster {
    private server;
    static getFullAddress(host: string, port: number, path?: string | null | undefined): string;
    private ws2pServer;
    private ws2pClients;
    private host;
    private port;
    private syncBlockInterval;
    private syncDocpoolInterval;
    private fifo;
    private maxLevel1Size;
    private messageHandler;
    private banned;
    private blockstampsCache;
    private memberkeysCache;
    private headsCache;
    private newHeads;
    private headsTimeout;
    private reconnectionInteval;
    private constructor();
    getKnownHeads(): Promise<WS2PHead[]>;
    headsReceived(heads: [{
        message: string;
        sig: string;
    }]): Promise<{
        message: string;
        sig: string;
    }[]>;
    private isMemberKey(pub);
    private isConnectedKey(pub);
    private existsBlock(blockstamp);
    static plugOn(server: Server): WS2PCluster;
    maxLevel1Peers: number;
    maxLevel2Peers: number;
    listen(host: string, port: number): Promise<WS2PServer>;
    close(): Promise<void>;
    clientsCount(): number;
    servedCount(): number;
    connectToRemoteWS(host: string, port: number, path: string, messageHandler: WS2PMessageHandler, expectedPub: string, ws2pEndpointUUID?: string): Promise<WS2PConnection>;
    connectToWS2Peers(): Promise<void>;
    listenServerFlow(): void;
    private broadcastHead(message, sig);
    private spreadNewHeads(heads);
    private sayHeadChangedTo(number, hash);
    trimClientConnections(): Promise<void>;
    private getPreferedNodes();
    protected acceptPubkey(pub: string, connectedPubkeys: string[], getConcurrentConnexionsCount: () => number, maxConcurrentConnexionsSize: number, priorityKeys: string[], priorityKeysOnly: boolean, targetWS2PUID?: string): Promise<boolean>;
    isSiblingNode(pub: string, uuid: string): boolean;
    getLevel1Connections(): Promise<WS2PConnection[]>;
    getLevel2Connections(): Promise<WS2PConnection[]>;
    getAllConnections(): WS2PConnection[];
    startCrawling(waitConnection?: boolean): Promise<void>;
    stopCrawling(): Promise<void>;
    pullBlocks(): Promise<void>;
    private makeApullShot();
    pullDocpool(): Promise<void>;
    getConnectedPubkeys(): string[];
    banConnection(c: WS2PConnection, reason: string): void;
}
