"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const WS2PServer_1 = require("./WS2PServer");
const WS2PClient_1 = require("./WS2PClient");
const randomPick_1 = require("../../../lib/common-libs/randomPick");
const constants_1 = require("../../crawler/lib/constants");
const WS2PBlockPuller_1 = require("./WS2PBlockPuller");
const WS2PDocpoolPuller_1 = require("./WS2PDocpoolPuller");
const constants_2 = require("./constants");
const PeerDTO_1 = require("../../../lib/dto/PeerDTO");
const GlobalFifoPromise_1 = require("../../../service/GlobalFifoPromise");
const other_constants_1 = require("../../../lib/other_constants");
const keyring_1 = require("../../../lib/common-libs/crypto/keyring");
const WS2PServerMessageHandler_1 = require("./interface/WS2PServerMessageHandler");
const constants_3 = require("../../../lib/common-libs/constants");
const package_1 = require("../../../lib/common/package");
const constants_4 = require("../../prover/lib/constants");
const proxy_1 = require("../../../lib/proxy");
const es = require('event-stream');
const nuuid = require('node-uuid');
const _ = require('underscore');
class WS2PCluster {
    constructor(server) {
        this.server = server;
        this.ws2pServer = null;
        this.ws2pClients = {};
        this.host = null;
        this.port = null;
        this.fifo = new GlobalFifoPromise_1.GlobalFifoPromise();
        this.maxLevel1Size = constants_2.WS2PConstants.MAX_LEVEL_1_PEERS;
        // A cache to remember the banned keys
        this.banned = {};
        // A cache to know if a block exists or not in the DB
        this.blockstampsCache = {};
        // A cache to know wether a pubkey is a member or not
        this.memberkeysCache = {};
        // A cache of the current HEAD for a given pubkey
        this.headsCache = {};
        // A buffer of "to be sent" heads
        this.newHeads = [];
        // The triggerer of a buffer of heads' sending
        this.headsTimeout = null;
        // A timer to regularly reconnect to the network in case we are below the minimum connections' count
        this.reconnectionInteval = null;
        this.messageHandler = new WS2PServerMessageHandler_1.WS2PServerMessageHandler(this.server, this);
        // Conf: max private connections
        if (this.server.conf.ws2p && this.server.conf.ws2p.maxPrivate !== undefined) {
            this.maxLevel1Size = this.server.conf.ws2p.maxPrivate;
        }
    }
    static getFullAddress(host, port, path = null) {
        if (host.match(constants_3.CommonConstants.IPV6_REGEXP)) {
            host = "[" + host + "]";
        }
        // Make the path be a string
        path = path || '';
        // delete the space at the beginning of the path
        if (path.match(/^ /)) {
            path = path.substr(1);
        }
        // Check that the path starts well with / (added if not)
        if (path.length > 0 && !path.match(/^\//)) {
            path = '/' + path;
        }
        // Choose the web protocol depending on the port
        const protocol = port == 443 ? "wss://" : "ws://";
        return [protocol, host, ':', port, path].join('');
    }
    getKnownHeads() {
        return __awaiter(this, void 0, void 0, function* () {
            const heads = [];
            const ws2pId = (this.server.conf.ws2p && this.server.conf.ws2p.uuid) || '000000';
            const localPub = this.server.conf.pair.pub;
            const fullId = [localPub, ws2pId].join('-');
            if (!this.headsCache[fullId]) {
                const current = yield this.server.dal.getCurrentBlockOrNull();
                if (current) {
                    const { sig, message } = this.sayHeadChangedTo(current.number, current.hash);
                    const blockstamp = [current.number, current.hash].join('-');
                    this.headsCache[fullId] = { blockstamp, message, sig };
                }
            }
            for (const ws2pFullId of Object.keys(this.headsCache)) {
                heads.push({
                    message: this.headsCache[ws2pFullId].message,
                    sig: this.headsCache[ws2pFullId].sig
                });
            }
            return heads;
        });
    }
    headsReceived(heads) {
        return __awaiter(this, void 0, void 0, function* () {
            const added = [];
            yield Promise.all(heads.map((h) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const message = h.message;
                    const sig = h.sig;
                    if (!message) {
                        throw "EMPTY_MESSAGE_FOR_HEAD";
                    }
                    if (message.match(constants_2.WS2PConstants.HEAD_V0_REGEXP)) {
                        const [, , pub, blockstamp] = message.split(':');
                        const ws2pId = (this.server.conf.ws2p && this.server.conf.ws2p.uuid) || '000000';
                        const fullId = [pub, ws2pId].join('-');
                        const sigOK = keyring_1.verify(message, sig, pub);
                        if (sigOK) {
                            // Already known?
                            if (!this.headsCache[fullId] || this.headsCache[fullId].blockstamp !== blockstamp) {
                                // More recent?
                                if (!this.headsCache[fullId] || parseInt(this.headsCache[fullId].blockstamp) < parseInt(blockstamp)) {
                                    // Check that issuer is a member and that the block exists
                                    const isAllowed = pub === this.server.conf.pair.pub || this.isConnectedKey(pub) || (yield this.isMemberKey(pub));
                                    if (isAllowed) {
                                        const exists = yield this.existsBlock(blockstamp);
                                        if (exists) {
                                            this.headsCache[fullId] = { blockstamp, message, sig };
                                            this.newHeads.push({ message, sig });
                                            added.push({ message, sig });
                                            // Cancel a pending "heads" to be spread
                                            if (this.headsTimeout) {
                                                clearTimeout(this.headsTimeout);
                                            }
                                            // Reprogram it a few moments later
                                            this.headsTimeout = setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                                                const heads = this.newHeads.splice(0, this.newHeads.length);
                                                if (heads.length) {
                                                    yield this.spreadNewHeads(heads);
                                                }
                                            }), constants_2.WS2PConstants.HEADS_SPREAD_TIMEOUT);
                                        }
                                    }
                                }
                            }
                        }
                        else {
                            throw "HEAD_MESSAGE_WRONGLY_SIGNED";
                        }
                    }
                    else if (message.match(constants_2.WS2PConstants.HEAD_V1_REGEXP)) {
                        const [, , , pub, blockstamp, ws2pId, software, softVersion, prefix] = message.split(':');
                        const sigOK = keyring_1.verify(message, sig, pub);
                        const fullId = [pub, ws2pId].join('-');
                        if (sigOK) {
                            // Already known?
                            if (!this.headsCache[fullId] || this.headsCache[fullId].blockstamp !== blockstamp) {
                                // More recent?
                                if (!this.headsCache[fullId] || parseInt(this.headsCache[fullId].blockstamp) < parseInt(blockstamp)) {
                                    // Check that issuer is a member and that the block exists
                                    const isAllowed = pub === this.server.conf.pair.pub || this.isConnectedKey(pub) || (yield this.isMemberKey(pub));
                                    if (isAllowed) {
                                        const exists = yield this.existsBlock(blockstamp);
                                        if (exists) {
                                            this.headsCache[fullId] = { blockstamp, message, sig };
                                            this.newHeads.push({ message, sig });
                                            added.push({ message, sig });
                                            // Cancel a pending "heads" to be spread
                                            if (this.headsTimeout) {
                                                clearTimeout(this.headsTimeout);
                                            }
                                            // Reprogram it a few moments later
                                            this.headsTimeout = setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                                                const heads = this.newHeads.splice(0, this.newHeads.length);
                                                if (heads.length) {
                                                    yield this.spreadNewHeads(heads);
                                                }
                                            }), constants_2.WS2PConstants.HEADS_SPREAD_TIMEOUT);
                                        }
                                    }
                                }
                            }
                        }
                        else {
                            throw "HEAD_MESSAGE_WRONGLY_SIGNED";
                        }
                    }
                }
                catch (e) {
                    this.server.logger.trace(e);
                }
            })));
            this.server.push({
                ws2p: 'heads',
                added
            });
            return added;
        });
    }
    isMemberKey(pub) {
        return __awaiter(this, void 0, void 0, function* () {
            let isMember = false;
            if (this.memberkeysCache[pub]) {
                isMember = true;
            }
            if (!isMember) {
                // Do we have this block in the DB?
                isMember = !!(yield this.server.dal.isMember(pub));
            }
            if (isMember) {
                // Update the last time it was checked
                this.memberkeysCache[pub] = Date.now();
            }
            return isMember;
        });
    }
    isConnectedKey(pub) {
        return this.getConnectedPubkeys().indexOf(pub) !== -1;
    }
    existsBlock(blockstamp) {
        return __awaiter(this, void 0, void 0, function* () {
            let exists = false;
            if (this.blockstampsCache[blockstamp]) {
                exists = true;
            }
            if (!exists) {
                // Do we have this block in the DB?
                exists = !!(yield this.server.dal.getAbsoluteBlockByBlockstamp(blockstamp));
            }
            // Update the last time it was checked
            this.blockstampsCache[blockstamp] = Date.now();
            return exists;
        });
    }
    static plugOn(server) {
        const cluster = new WS2PCluster(server);
        server.ws2pCluster = cluster;
        return cluster;
    }
    set maxLevel1Peers(newValue) {
        this.maxLevel1Size = Math.max(newValue, 0) || 0;
    }
    set maxLevel2Peers(newValue) {
        if (this.ws2pServer) {
            this.ws2pServer.maxLevel2Peers = Math.max(newValue, 0);
        }
    }
    get maxLevel2Peers() {
        if (this.ws2pServer) {
            return this.ws2pServer.maxLevel2Peers || 0;
        }
        return 0;
    }
    listen(host, port) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.ws2pServer) {
                yield this.ws2pServer.close();
            }
            this.ws2pServer = yield WS2PServer_1.WS2PServer.bindOn(this.server, host, port, this.fifo, (pubkey, connectedPubkeys) => {
                const privilegedNodes = (this.server.conf.ws2p && this.server.conf.ws2p.privilegedNodes) ? this.server.conf.ws2p.privilegedNodes : [];
                return this.acceptPubkey(pubkey, connectedPubkeys, () => this.servedCount(), this.maxLevel2Peers, privilegedNodes, (this.server.conf.ws2p && this.server.conf.ws2p.privilegedOnly || false));
            }, this.messageHandler);
            this.host = host;
            this.port = port;
            return this.ws2pServer;
        });
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.ws2pServer) {
                yield this.ws2pServer.close();
            }
            const connections = this.getAllConnections();
            yield Promise.all(connections.map(c => c.close()));
        });
    }
    clientsCount() {
        return Object.keys(this.ws2pClients).length;
    }
    servedCount() {
        return this.ws2pServer ? this.ws2pServer.getConnexions().length : 0;
    }
    connectToRemoteWS(host, port, path, messageHandler, expectedPub, ws2pEndpointUUID = "") {
        return __awaiter(this, void 0, void 0, function* () {
            const uuid = nuuid.v4();
            let pub = expectedPub.slice(0, 8);
            const api = (host.match(constants_2.WS2PConstants.HOST_ONION_REGEX) !== null) ? 'WS2PTOR' : 'WS2P';
            try {
                const fullEndpointAddress = WS2PCluster.getFullAddress(host, port, path);
                const ws2pc = yield WS2PClient_1.WS2PClient.connectTo(this.server, fullEndpointAddress, ws2pEndpointUUID, messageHandler, expectedPub, (pub) => {
                    const connectedPubkeys = this.getConnectedPubkeys();
                    const preferedNodes = (this.server.conf.ws2p && this.server.conf.ws2p.preferedNodes) ? this.server.conf.ws2p.preferedNodes : [];
                    return this.acceptPubkey(expectedPub, connectedPubkeys, () => this.clientsCount(), this.maxLevel1Size, preferedNodes, (this.server.conf.ws2p && this.server.conf.ws2p.preferedOnly) || false, ws2pEndpointUUID);
                });
                this.ws2pClients[uuid] = ws2pc;
                pub = ws2pc.connection.pubkey;
                ws2pc.connection.closed.then(() => {
                    this.server.logger.info(api + ': connection [%s `' + api + ' %s %s`] has been closed', pub.slice(0, 8), host, port);
                    this.server.push({
                        ws2p: 'disconnected',
                        peer: {
                            pub: ws2pc.connection.pubkey
                        }
                    });
                    if (this.ws2pClients[uuid]) {
                        delete this.ws2pClients[uuid];
                    }
                });
                this.server.logger.info(api + ': connected to peer %s using `' + api + ' %s %s`!', pub.slice(0, 8), host, port);
                this.server.push({
                    ws2p: 'connected',
                    to: { host, port, pubkey: pub }
                });
                yield this.server.dal.setPeerUP(pub);
                return ws2pc.connection;
            }
            catch (e) {
                this.server.logger.info(api + ': Could not connect to peer %s using `' + api + ' %s %s: %s`', pub.slice(0, 8), host, port, (e && e.message || e));
                throw e;
            }
        });
    }
    connectToWS2Peers() {
        return __awaiter(this, void 0, void 0, function* () {
            const potentials = yield this.server.dal.getWS2Peers();
            const peers = potentials.map((p) => PeerDTO_1.PeerDTO.fromJSONObject(p));
            const prefered = ((this.server.conf.ws2p && this.server.conf.ws2p.preferedNodes) || []).slice(); // Copy
            // Our key is also a prefered one, so we connect to our siblings
            prefered.push(this.server.conf.pair.pub);
            const canReachTorEndpoint = proxy_1.ProxiesConf.canReachTorEndpoint(this.server.conf.proxiesConf);
            peers.sort((a, b) => {
                const aIsPrefered = prefered.indexOf(a.pubkey) !== -1;
                const bIsPrefered = prefered.indexOf(b.pubkey) !== -1;
                if (canReachTorEndpoint) {
                    const aAtWs2pTorEnpoint = a.endpoints.filter(function (element) { return element.match(constants_3.CommonConstants.WS2PTOR_REGEXP); }).length > 0;
                    const bAtWs2pTorEnpoint = b.endpoints.filter(function (element) { return element.match(constants_3.CommonConstants.WS2PTOR_REGEXP); }).length > 0;
                    if ((aAtWs2pTorEnpoint && bAtWs2pTorEnpoint) || (!aAtWs2pTorEnpoint && !bAtWs2pTorEnpoint)) {
                        if ((aIsPrefered && bIsPrefered) || (!aIsPrefered && !bIsPrefered)) {
                            return 0;
                        }
                        else if (aIsPrefered) {
                            return -1;
                        }
                        else {
                            return 1;
                        }
                    }
                    else {
                        if (aAtWs2pTorEnpoint) {
                            return -1;
                        }
                        else {
                            return 1;
                        }
                    }
                }
                else {
                    if ((aIsPrefered && bIsPrefered) || (!aIsPrefered && !bIsPrefered)) {
                        return 0;
                    }
                    else if (aIsPrefered) {
                        return -1;
                    }
                    else {
                        return 1;
                    }
                }
            });
            let i = 0;
            const canReachClearEndpoint = proxy_1.ProxiesConf.canReachClearEndpoint(this.server.conf.proxiesConf);
            while (i < peers.length && this.clientsCount() < this.maxLevel1Size) {
                const p = peers[i];
                const api = p.getWS2P(canReachTorEndpoint, canReachClearEndpoint);
                if (api) {
                    try {
                        // We do not connect to local host
                        if (!this.server.conf.ws2p || api.uuid !== this.server.conf.ws2p.uuid || p.pubkey !== this.server.conf.pair.pub || api.uuid === '11111111') {
                            yield this.connectToRemoteWS(api.host, api.port, api.path, this.messageHandler, p.pubkey, api.uuid);
                        }
                    }
                    catch (e) {
                        this.server.logger.debug('WS2P: init: failed connection');
                    }
                }
                i++;
                // Trim the eventual extra connections
                setTimeout(() => this.trimClientConnections(), constants_2.WS2PConstants.CONNEXION_TIMEOUT);
            }
        });
    }
    listenServerFlow() {
        let connectingToNodesByFlow = false;
        // Also listen for network updates, and connect to new nodes
        this.server.pipe(es.mapSync((data) => {
            (() => __awaiter(this, void 0, void 0, function* () {
                // New peer
                if (data.endpoints) {
                    const peer = PeerDTO_1.PeerDTO.fromJSONObject(data);
                    const ws2pEnpoint = peer.getWS2P(proxy_1.ProxiesConf.canReachTorEndpoint(this.server.conf.proxiesConf), proxy_1.ProxiesConf.canReachClearEndpoint(this.server.conf.proxiesConf));
                    if (ws2pEnpoint) {
                        // Check if already connected to the pubkey (in any way: server or client)
                        const connectedPubkeys = this.getConnectedPubkeys();
                        const preferedNodes = (this.server.conf.ws2p && this.server.conf.ws2p.preferedNodes) ? this.server.conf.ws2p.preferedNodes : [];
                        const shouldAccept = yield this.acceptPubkey(peer.pubkey, connectedPubkeys, () => this.clientsCount(), this.maxLevel1Size, preferedNodes, (this.server.conf.ws2p && this.server.conf.ws2p.preferedOnly) || false, ws2pEnpoint.uuid);
                        if (shouldAccept && (!this.server.conf.ws2p || ws2pEnpoint.uuid !== this.server.conf.ws2p.uuid || peer.pubkey !== this.server.conf.pair.pub || ws2pEnpoint.uuid === '11111111')) {
                            yield this.connectToRemoteWS(ws2pEnpoint.host, ws2pEnpoint.port, ws2pEnpoint.path, this.messageHandler, peer.pubkey);
                            yield this.trimClientConnections();
                        }
                    }
                }
                else if (data.joiners) {
                    // Update the cache
                    this.blockstampsCache[[data.number, data.hash].join('-')] = Date.now();
                }
                else if (data.bcEvent === other_constants_1.OtherConstants.BC_EVENT.HEAD_CHANGED || data.bcEvent === other_constants_1.OtherConstants.BC_EVENT.SWITCHED) {
                    // Propagate this change to the network
                    const { sig, message } = this.sayHeadChangedTo(data.block.number, data.block.hash);
                    try {
                        yield this.broadcastHead(message, sig);
                    }
                    catch (e) {
                        this.server.logger.warn(e);
                    }
                }
                else if (data.ws2p === 'disconnected') {
                    const nbConnections = this.getAllConnections().length;
                    if (nbConnections < constants_2.WS2PConstants.CONNECTIONS_LOW_LEVEL && !connectingToNodesByFlow) {
                        try {
                            connectingToNodesByFlow = true;
                            yield this.connectToWS2Peers();
                        }
                        catch (e) {
                            throw e;
                        }
                        finally {
                            connectingToNodesByFlow = false;
                        }
                    }
                }
            }))();
            return data;
        }));
    }
    broadcastHead(message, sig) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.headsReceived([{ message, sig }]);
            return this.spreadNewHeads([{ message, sig }]);
        });
    }
    spreadNewHeads(heads) {
        return __awaiter(this, void 0, void 0, function* () {
            const connexions = this.getAllConnections();
            return Promise.all(connexions.map((c) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield c.pushHeads(heads);
                }
                catch (e) {
                    this.server.logger.warn('Could not spread new HEAD info to %s WS2P %s %s', c.pubkey);
                }
            })));
        });
    }
    sayHeadChangedTo(number, hash) {
        const api = (this.server.conf.ws2p && this.server.conf.ws2p.remotehost && this.server.conf.ws2p.remotehost.match(constants_2.WS2PConstants.HOST_ONION_REGEX)) ? 'WS2P' : 'WS2P';
        const key = new keyring_1.Key(this.server.conf.pair.pub, this.server.conf.pair.sec);
        const pub = key.publicKey;
        const software = 'duniter';
        const softVersion = package_1.Package.getInstance().version;
        const ws2pId = (this.server.conf.ws2p && this.server.conf.ws2p.uuid) || '00000000';
        const prefix = this.server.conf.prefix || constants_4.Constants.DEFAULT_PEER_ID;
        const message = `${api}:HEAD:1:${pub}:${number}-${hash}:${ws2pId}:${software}:${softVersion}:${prefix}`;
        const sig = key.signSync(message);
        return { sig, message, pub };
    }
    trimClientConnections() {
        return __awaiter(this, void 0, void 0, function* () {
            let serverPubkeys = [];
            if (this.ws2pServer) {
                serverPubkeys = this.ws2pServer.getConnexions().map(c => c.pubkey);
            }
            let disconnectedOne = true;
            // Disconnect Private connexions already present under Public
            while (disconnectedOne) {
                disconnectedOne = false;
                let uuids = Object.keys(this.ws2pClients);
                uuids = _.shuffle(uuids);
                for (const uuid of uuids) {
                    const client = this.ws2pClients[uuid];
                    const pub = client.connection.pubkey;
                    const isNotOurself = pub !== this.server.conf.pair.pub;
                    const isAlreadyInPublic = serverPubkeys.indexOf(pub) !== -1;
                    if (isNotOurself && isAlreadyInPublic) {
                        client.connection.close();
                        yield client.connection.closed;
                        disconnectedOne = true;
                        if (this.ws2pClients[uuid]) {
                            delete this.ws2pClients[uuid];
                        }
                    }
                }
            }
            // Disconnect non-members
            while (disconnectedOne && this.clientsCount() > this.maxLevel1Size) {
                disconnectedOne = false;
                let uuids = Object.keys(this.ws2pClients);
                uuids = _.shuffle(uuids);
                for (const uuid of uuids) {
                    const client = this.ws2pClients[uuid];
                    const pub = client.connection.pubkey;
                    const isNotOurself = pub !== this.server.conf.pair.pub;
                    const isMember = yield this.server.dal.isMember(pub);
                    const isPrefered = this.getPreferedNodes().indexOf(pub) !== -1;
                    if (isNotOurself && !isMember && !disconnectedOne && !isPrefered) {
                        client.connection.close();
                        yield client.connection.closed;
                        disconnectedOne = true;
                        if (this.ws2pClients[uuid]) {
                            delete this.ws2pClients[uuid];
                        }
                    }
                }
            }
            disconnectedOne = true;
            // Disconnect non-prefered members
            while (disconnectedOne && this.clientsCount() > this.maxLevel1Size) {
                disconnectedOne = false;
                let uuids = Object.keys(this.ws2pClients);
                uuids = _.shuffle(uuids);
                for (const uuid of uuids) {
                    const client = this.ws2pClients[uuid];
                    const pub = client.connection.pubkey;
                    const isNotOurself = pub !== this.server.conf.pair.pub;
                    const isPrefered = this.getPreferedNodes().indexOf(pub) !== -1;
                    if (isNotOurself && !disconnectedOne && !isPrefered) {
                        client.connection.close();
                        disconnectedOne = true;
                        yield client.connection.closed;
                        if (this.ws2pClients[uuid]) {
                            delete this.ws2pClients[uuid];
                        }
                    }
                }
            }
            // Disconnect anything
            disconnectedOne = true;
            while (disconnectedOne && this.clientsCount() > this.maxLevel1Size) {
                disconnectedOne = false;
                let uuids = Object.keys(this.ws2pClients);
                uuids = _.shuffle(uuids);
                for (const uuid of uuids) {
                    const client = this.ws2pClients[uuid];
                    if (!disconnectedOne) {
                        client.connection.close();
                        disconnectedOne = true;
                        yield client.connection.closed;
                        if (this.ws2pClients[uuid]) {
                            delete this.ws2pClients[uuid];
                        }
                    }
                }
            }
        });
    }
    getPreferedNodes() {
        return (this.server.conf.ws2p && this.server.conf.ws2p.preferedNodes) || [];
    }
    acceptPubkey(pub, connectedPubkeys, getConcurrentConnexionsCount, maxConcurrentConnexionsSize, priorityKeys, priorityKeysOnly, targetWS2PUID = "") {
        return __awaiter(this, void 0, void 0, function* () {
            // We do not accept banned keys
            if (this.banned[pub]) {
                this.server.logger.warn('Connection to %s refused, reason: %s', pub.slice(0, 8), this.banned[pub]);
                return false;
            }
            let accept = priorityKeys.indexOf(pub) !== -1;
            if (!accept && !priorityKeysOnly && connectedPubkeys.indexOf(pub) === -1) {
                // Do we have room?
                if (this.server.conf.pair.pub === pub && this.server.conf.ws2p && this.server.conf.ws2p.uuid === targetWS2PUID) {
                    accept = false;
                }
                else if (getConcurrentConnexionsCount() < maxConcurrentConnexionsSize) {
                    // Yes: just connect to it
                    accept = true;
                }
                else {
                    // No: let's verify some peer has a lower priority
                    if (connectedPubkeys.indexOf(this.server.conf.pair.pub) !== -1) {
                        // Yes, we are connected to ourself. Let's replace this connexion
                        accept = true;
                    }
                    else {
                        // Does this node have the priority over at least one node?
                        const isMemberPeer = yield this.server.dal.isMember(pub);
                        if (isMemberPeer) {
                            // The node may have the priority over at least 1 other node
                            let i = 0, existsOneNonMemberNode = false;
                            while (!existsOneNonMemberNode && i < connectedPubkeys.length) {
                                const isAlsoAMemberPeer = yield this.server.dal.isMember(connectedPubkeys[i]);
                                existsOneNonMemberNode = !isAlsoAMemberPeer;
                                i++;
                            }
                            if (existsOneNonMemberNode) {
                                // The node has the priority over a non-member peer: try to connect
                                accept = true;
                            }
                        }
                    }
                }
            }
            else {
                // The pubkey is already connected: we accept only self nodes, and they have a supreme priority (these are siblings)
                if (targetWS2PUID) {
                    if (this.isSiblingNode(pub, targetWS2PUID)) {
                        accept = true;
                    }
                }
            }
            return accept;
        });
    }
    isSiblingNode(pub, uuid) {
        return !!(this.server.conf.pair.pub === pub && this.server.conf.ws2p && this.server.conf.ws2p.uuid !== uuid);
    }
    getLevel1Connections() {
        return __awaiter(this, void 0, void 0, function* () {
            const all = [];
            for (const uuid of Object.keys(this.ws2pClients)) {
                all.push(this.ws2pClients[uuid].connection);
            }
            return all;
        });
    }
    getLevel2Connections() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.ws2pServer ? this.ws2pServer.getConnexions() : [];
        });
    }
    getAllConnections() {
        const all = this.ws2pServer ? this.ws2pServer.getConnexions() : [];
        for (const uuid of Object.keys(this.ws2pClients)) {
            all.push(this.ws2pClients[uuid].connection);
        }
        return all;
    }
    startCrawling(waitConnection = false) {
        return __awaiter(this, void 0, void 0, function* () {
            // For connectivity
            this.reconnectionInteval = setInterval(() => this.server.push({ ws2p: 'disconnected' }), 1000 * constants_2.WS2PConstants.RECONNEXION_INTERVAL_IN_SEC);
            // For blocks
            if (this.syncBlockInterval)
                clearInterval(this.syncBlockInterval);
            this.syncBlockInterval = setInterval(() => this.pullBlocks(), 1000 * constants_2.WS2PConstants.BLOCK_PULLING_INTERVAL);
            // Pull blocks right on start
            const init = () => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.listenServerFlow();
                    yield this.connectToWS2Peers();
                    yield this.pullBlocks();
                }
                catch (e) {
                    this.server.logger.error(e);
                }
            });
            if (waitConnection) {
                yield init();
            }
            else {
                init();
            }
            // For docpool
            if (this.syncDocpoolInterval)
                clearInterval(this.syncDocpoolInterval);
            this.syncDocpoolInterval = setInterval(() => this.pullDocpool(), 1000 * constants_2.WS2PConstants.DOCPOOL_PULLING_INTERVAL);
            // The first pulling occurs 10 minutes after the start
            setTimeout(() => this.pullDocpool(), constants_2.WS2PConstants.SANDBOX_FIRST_PULL_DELAY);
        });
    }
    stopCrawling() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.reconnectionInteval) {
                clearInterval(this.reconnectionInteval);
            }
            if (this.syncBlockInterval) {
                clearInterval(this.syncBlockInterval);
            }
            if (this.syncDocpoolInterval) {
                clearInterval(this.syncDocpoolInterval);
            }
        });
    }
    pullBlocks() {
        return __awaiter(this, void 0, void 0, function* () {
            let current = { number: -1 };
            let newCurrent = { number: 0 };
            while (current && newCurrent && newCurrent.number > current.number) {
                current = newCurrent;
                yield this.makeApullShot();
                newCurrent = yield this.server.dal.getCurrentBlockOrNull();
            }
            if (current) {
                this.server.pullingEvent('end', current.number);
            }
        });
    }
    makeApullShot() {
        return __awaiter(this, void 0, void 0, function* () {
            const connections = this.getAllConnections();
            const chosen = randomPick_1.randomPick(connections, constants_1.CrawlerConstants.CRAWL_PEERS_COUNT);
            yield Promise.all(chosen.map((conn) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const puller = new WS2PBlockPuller_1.WS2PBlockPuller(this.server, conn);
                    yield puller.pull();
                }
                catch (e) {
                    this.server.logger.warn(e);
                }
            })));
            yield this.server.BlockchainService.pushFIFO("WS2PCrawlerResolution", () => __awaiter(this, void 0, void 0, function* () {
                yield this.server.BlockchainService.blockResolution();
                yield this.server.BlockchainService.forkResolution();
            }));
        });
    }
    pullDocpool() {
        return __awaiter(this, void 0, void 0, function* () {
            const connections = this.getAllConnections();
            const chosen = randomPick_1.randomPick(connections, constants_1.CrawlerConstants.CRAWL_PEERS_COUNT);
            yield Promise.all(chosen.map((conn) => __awaiter(this, void 0, void 0, function* () {
                const puller = new WS2PDocpoolPuller_1.WS2PDocpoolPuller(this.server, conn);
                yield puller.pull();
            })));
        });
    }
    getConnectedPubkeys() {
        const clients = Object.keys(this.ws2pClients).map(k => this.ws2pClients[k].connection.pubkey);
        const served = this.ws2pServer ? this.ws2pServer.getConnexions().map(c => c.pubkey) : [];
        return clients.concat(served);
    }
    banConnection(c, reason) {
        this.server.logger.warn('Banning connections of %s for %ss, reason: %s', c.pubkey.slice(0, 8), constants_2.WS2PConstants.BAN_DURATION_IN_SECONDS, reason);
        if (c.pubkey) {
            this.banned[c.pubkey] = reason;
            setTimeout(() => {
                delete this.banned[c.pubkey];
            }, 1000 * constants_2.WS2PConstants.BAN_DURATION_IN_SECONDS);
            const connections = this.getAllConnections();
            for (const connection of connections) {
                if (c.pubkey == connection.pubkey) {
                    connection.close();
                }
            }
        }
    }
}
exports.WS2PCluster = WS2PCluster;
//# sourceMappingURL=WS2PCluster.js.map