"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const keyring_1 = require("../../../lib/common-libs/crypto/keyring");
const constants_1 = require("./constants");
const ws = require('ws');
const SocksProxyAgent = require('socks-proxy-agent');
const nuuid = require('node-uuid');
const logger = require('../../../lib/logger').NewLogger('ws2p');
const MAXIMUM_ERRORS_COUNT = 5;
const REQUEST_TIMEOUT_VALUE = constants_1.WS2PConstants.REQUEST_TIMEOUT;
var WS2P_ERR;
(function (WS2P_ERR) {
    WS2P_ERR[WS2P_ERR["REJECTED_PUBKEY_OR_INCORRECT_ASK_SIGNATURE_FROM_REMOTE"] = 0] = "REJECTED_PUBKEY_OR_INCORRECT_ASK_SIGNATURE_FROM_REMOTE";
    WS2P_ERR[WS2P_ERR["AUTH_INVALID_ASK_FIELDS"] = 1] = "AUTH_INVALID_ASK_FIELDS";
    WS2P_ERR[WS2P_ERR["AUTH_INVALID_ACK_FIELDS"] = 2] = "AUTH_INVALID_ACK_FIELDS";
    WS2P_ERR[WS2P_ERR["AUTH_INVALID_OK_FIELDS"] = 3] = "AUTH_INVALID_OK_FIELDS";
    WS2P_ERR[WS2P_ERR["INCORRECT_ACK_SIGNATURE_FROM_REMOTE"] = 4] = "INCORRECT_ACK_SIGNATURE_FROM_REMOTE";
    WS2P_ERR[WS2P_ERR["INCORRECT_ASK_SIGNATURE_FROM_REMOTE"] = 5] = "INCORRECT_ASK_SIGNATURE_FROM_REMOTE";
    WS2P_ERR[WS2P_ERR["UNKNOWN_AUTH_MESSAGE"] = 6] = "UNKNOWN_AUTH_MESSAGE";
    WS2P_ERR[WS2P_ERR["ALREADY_AUTHENTICATED_AND_CONFIRMED_BY_REMOTE"] = 7] = "ALREADY_AUTHENTICATED_AND_CONFIRMED_BY_REMOTE";
    WS2P_ERR[WS2P_ERR["ALREADY_AUTHENTICATED_REMOTE"] = 8] = "ALREADY_AUTHENTICATED_REMOTE";
    WS2P_ERR[WS2P_ERR["ALREADY_AUTHENTICATED_BY_REMOTE"] = 9] = "ALREADY_AUTHENTICATED_BY_REMOTE";
    WS2P_ERR[WS2P_ERR["INCORRECT_PUBKEY_FOR_REMOTE"] = 10] = "INCORRECT_PUBKEY_FOR_REMOTE";
    WS2P_ERR[WS2P_ERR["MUST_BE_AUTHENTICATED_FIRST"] = 11] = "MUST_BE_AUTHENTICATED_FIRST";
    WS2P_ERR[WS2P_ERR["REQUEST_FAILED"] = 12] = "REQUEST_FAILED";
    WS2P_ERR[WS2P_ERR["MESSAGE_MUST_BE_AN_OBJECT"] = 13] = "MESSAGE_MUST_BE_AN_OBJECT";
    WS2P_ERR[WS2P_ERR["ANSWER_TO_UNDEFINED_REQUEST"] = 14] = "ANSWER_TO_UNDEFINED_REQUEST";
})(WS2P_ERR || (WS2P_ERR = {}));
var WS2P_PUSH;
(function (WS2P_PUSH) {
    WS2P_PUSH[WS2P_PUSH["PEER"] = 0] = "PEER";
    WS2P_PUSH[WS2P_PUSH["TRANSACTION"] = 1] = "TRANSACTION";
    WS2P_PUSH[WS2P_PUSH["MEMBERSHIP"] = 2] = "MEMBERSHIP";
    WS2P_PUSH[WS2P_PUSH["CERTIFICATION"] = 3] = "CERTIFICATION";
    WS2P_PUSH[WS2P_PUSH["IDENTITY"] = 4] = "IDENTITY";
    WS2P_PUSH[WS2P_PUSH["BLOCK"] = 5] = "BLOCK";
    WS2P_PUSH[WS2P_PUSH["HEAD"] = 6] = "HEAD";
})(WS2P_PUSH = exports.WS2P_PUSH || (exports.WS2P_PUSH = {}));
/**
 * A passive authenticator based on our keyring.
 */
class WS2PPubkeyRemoteAuth {
    constructor(currency, pair, tellIsAuthorizedPubkey = () => Promise.resolve(true)) {
        this.currency = currency;
        this.pair = pair;
        this.tellIsAuthorizedPubkey = tellIsAuthorizedPubkey;
        this.authenticatedByRemote = false;
        this.remotePub = "";
        this.challenge = nuuid.v4() + nuuid.v4();
        this.serverAuth = new Promise((resolve, reject) => {
            this.serverAuthResolve = resolve;
            this.serverAuthReject = reject;
        });
    }
    getPubkey() {
        return this.remotePub;
    }
    sendACK(ws) {
        return __awaiter(this, void 0, void 0, function* () {
            const challengeMessage = `WS2P:ACK:${this.currency}:${this.pair.pub}:${this.challenge}`;
            Logger.log('sendACK >>> ' + challengeMessage);
            const sig = this.pair.signSync(challengeMessage);
            yield ws.send(JSON.stringify({
                auth: 'ACK',
                pub: this.pair.pub,
                sig
            }));
        });
    }
    registerCONNECT(challenge, sig, pub) {
        return __awaiter(this, void 0, void 0, function* () {
            const allow = yield this.tellIsAuthorizedPubkey(pub);
            if (!allow) {
                return false;
            }
            const challengeMessage = `WS2P:CONNECT:${this.currency}:${pub}:${challenge}`;
            Logger.log('registerCONNECT >>> ' + challengeMessage);
            const verified = keyring_1.verify(challengeMessage, sig, pub);
            if (verified) {
                this.challenge = challenge;
                this.remotePub = pub;
            }
            return verified;
        });
    }
    registerOK(sig) {
        return __awaiter(this, void 0, void 0, function* () {
            const challengeMessage = `WS2P:OK:${this.currency}:${this.remotePub}:${this.challenge}`;
            Logger.log('registerOK >>> ' + challengeMessage);
            this.authenticatedByRemote = keyring_1.verify(challengeMessage, sig, this.remotePub);
            if (!this.authenticatedByRemote) {
                this.serverAuthReject("Wrong signature from remote OK");
            }
            else {
                this.serverAuthResolve();
            }
            return this.authenticatedByRemote;
        });
    }
    isAuthenticatedByRemote() {
        return this.authenticatedByRemote;
    }
    authenticationIsDone() {
        return this.serverAuth;
    }
}
exports.WS2PPubkeyRemoteAuth = WS2PPubkeyRemoteAuth;
/**
 * An connecting authenticator based on our keyring.
 */
class WS2PPubkeyLocalAuth {
    constructor(currency, pair, tellIsAuthorizedPubkey = () => Promise.resolve(true)) {
        this.currency = currency;
        this.pair = pair;
        this.tellIsAuthorizedPubkey = tellIsAuthorizedPubkey;
        this.authenticated = false;
        this.challenge = nuuid.v4() + nuuid.v4();
        this.serverAuth = new Promise((resolve, reject) => {
            this.serverAuthResolve = resolve;
            this.serverAuthReject = reject;
        });
    }
    sendCONNECT(ws) {
        return __awaiter(this, void 0, void 0, function* () {
            const challengeMessage = `WS2P:CONNECT:${this.currency}:${this.pair.pub}:${this.challenge}`;
            Logger.log('sendCONNECT >>> ' + challengeMessage);
            const sig = this.pair.signSync(challengeMessage);
            yield ws.send(JSON.stringify({
                auth: 'CONNECT',
                pub: this.pair.pub,
                challenge: this.challenge,
                sig
            }));
            return this.serverAuth;
        });
    }
    registerACK(sig, pub) {
        return __awaiter(this, void 0, void 0, function* () {
            const allow = yield this.tellIsAuthorizedPubkey(pub);
            if (!allow) {
                return false;
            }
            const challengeMessage = `WS2P:ACK:${this.currency}:${pub}:${this.challenge}`;
            Logger.log('registerACK >>> ' + challengeMessage);
            this.authenticated = keyring_1.verify(challengeMessage, sig, pub);
            if (!this.authenticated) {
                this.serverAuthReject("Wrong signature from server ACK");
            }
            else {
                this.serverAuthResolve();
            }
            return this.authenticated;
        });
    }
    sendOK(ws) {
        return __awaiter(this, void 0, void 0, function* () {
            const challengeMessage = `WS2P:OK:${this.currency}:${this.pair.pub}:${this.challenge}`;
            Logger.log('sendOK >>> ' + challengeMessage);
            const sig = this.pair.signSync(challengeMessage);
            yield ws.send(JSON.stringify({
                auth: 'OK',
                sig
            }));
            return this.serverAuth;
        });
    }
    isRemoteAuthenticated() {
        return this.authenticated;
    }
    authenticationIsDone() {
        return this.serverAuth;
    }
}
exports.WS2PPubkeyLocalAuth = WS2PPubkeyLocalAuth;
/**
 * The handler of a WS2P connection.
 *
 * Goal: operating an authenticated bidirectionnal communication over a WebSocket connection.
 * Requires an established WebSocket connection in order to work.
 */
class WS2PConnection {
    constructor(ws, onWsOpened, onWsClosed, messageHandler, localAuth, remoteAuth, options = {
            connectionTimeout: REQUEST_TIMEOUT_VALUE,
            requestTimeout: REQUEST_TIMEOUT_VALUE
        }, expectedPub = "") {
        this.ws = ws;
        this.onWsOpened = onWsOpened;
        this.onWsClosed = onWsClosed;
        this.messageHandler = messageHandler;
        this.localAuth = localAuth;
        this.remoteAuth = remoteAuth;
        this.options = options;
        this.expectedPub = expectedPub;
        this.nbErrors = 0;
        this.nbRequestsCount = 0;
        this.nbResponsesCount = 0;
        this.nbPushsToRemoteCount = 0;
        this.nbPushsByRemoteCount = 0;
        this.exchanges = {};
        this.connectedp = new Promise((resolve, reject) => {
            this.connectedResolve = resolve;
            this.connectedReject = reject;
        });
    }
    static newConnectionToAddress(address, messageHandler, localAuth, remoteAuth, proxySocksAddress = undefined, options = {
            connectionTimeout: REQUEST_TIMEOUT_VALUE,
            requestTimeout: REQUEST_TIMEOUT_VALUE
        }, expectedPub = "") {
        if (address.match(constants_1.WS2PConstants.FULL_ADDRESS_ONION_REGEX)) {
            options = {
                connectionTimeout: constants_1.WS2PConstants.CONNEXION_TOR_TIMEOUT,
                requestTimeout: constants_1.WS2PConstants.REQUEST_TOR_TIMEOUT
            };
        }
        const websocket = (proxySocksAddress !== undefined) ? new ws(address, { agent: SocksProxyAgent("socks://" + proxySocksAddress) }) : new ws(address);
        const onWsOpened = new Promise(res => {
            websocket.on('open', () => res());
        });
        const onWsClosed = new Promise(res => {
            websocket.on('close', () => res());
        });
        websocket.on('error', () => websocket.close());
        return new WS2PConnection(websocket, onWsOpened, onWsClosed, messageHandler, localAuth, remoteAuth, options, expectedPub);
    }
    static newConnectionFromWebSocketServer(websocket, messageHandler, localAuth, remoteAuth, options = {
            connectionTimeout: REQUEST_TIMEOUT_VALUE,
            requestTimeout: REQUEST_TIMEOUT_VALUE
        }, expectedPub = "") {
        const onWsOpened = Promise.resolve();
        const onWsClosed = new Promise(res => {
            websocket.on('close', () => res());
        });
        return new WS2PConnection(websocket, onWsOpened, onWsClosed, messageHandler, localAuth, remoteAuth, options, expectedPub);
    }
    get pubkey() {
        return this.remoteAuth.getPubkey();
    }
    get nbRequests() {
        return this.nbRequestsCount;
    }
    get nbResponses() {
        return this.nbResponsesCount;
    }
    get nbPushsToRemote() {
        return this.nbPushsToRemoteCount;
    }
    get nbPushsByRemote() {
        return this.nbPushsByRemoteCount;
    }
    get connected() {
        return this.connectedp;
    }
    get closed() {
        return this.onWsClosed;
    }
    close() {
        return this.ws.close();
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.connectp) {
                this.connectp = (() => __awaiter(this, void 0, void 0, function* () {
                    const connectionTimeout = new Promise((res, rej) => {
                        setTimeout(() => {
                            rej("WS2P connection timeout");
                        }, this.options.connectionTimeout);
                    });
                    try {
                        yield Promise.race([connectionTimeout, new Promise((resolve, reject) => {
                                (() => __awaiter(this, void 0, void 0, function* () {
                                    yield this.onWsOpened;
                                    try {
                                        yield this.localAuth.sendCONNECT(this.ws);
                                        yield Promise.all([
                                            this.localAuth.authenticationIsDone(),
                                            this.remoteAuth.authenticationIsDone()
                                        ]);
                                        resolve();
                                    }
                                    catch (e) {
                                        reject(e);
                                    }
                                }))();
                                this.ws.on('message', (msg) => __awaiter(this, void 0, void 0, function* () {
                                    const data = JSON.parse(msg);
                                    // Incorrect data
                                    if (typeof data !== 'object') {
                                        // We only accept JSON objects
                                        yield this.errorDetected(WS2P_ERR.MESSAGE_MUST_BE_AN_OBJECT);
                                    }
                                    else {
                                        /************************
                                         * CONNECTION STUFF
                                         ************************/
                                        if (data.auth && typeof data.auth === "string") {
                                            if (data.auth === "CONNECT") {
                                                if (this.remoteAuth.isAuthenticatedByRemote()) {
                                                    return this.errorDetected(WS2P_ERR.ALREADY_AUTHENTICATED_BY_REMOTE);
                                                }
                                                else if (typeof data.pub !== "string" || typeof data.sig !== "string" || typeof data.challenge !== "string") {
                                                    yield this.errorDetected(WS2P_ERR.AUTH_INVALID_ASK_FIELDS);
                                                }
                                                else {
                                                    if (this.expectedPub && data.pub !== this.expectedPub) {
                                                        yield this.errorDetected(WS2P_ERR.INCORRECT_PUBKEY_FOR_REMOTE);
                                                    }
                                                    else {
                                                        const valid = yield this.remoteAuth.registerCONNECT(data.challenge, data.sig, data.pub);
                                                        if (valid) {
                                                            yield this.remoteAuth.sendACK(this.ws);
                                                        }
                                                        else {
                                                            yield this.errorDetected(WS2P_ERR.REJECTED_PUBKEY_OR_INCORRECT_ASK_SIGNATURE_FROM_REMOTE);
                                                        }
                                                    }
                                                }
                                            }
                                            else if (data.auth === "ACK") {
                                                if (this.localAuth.isRemoteAuthenticated()) {
                                                    return this.errorDetected(WS2P_ERR.ALREADY_AUTHENTICATED_REMOTE);
                                                }
                                                if (typeof data.pub !== "string" || typeof data.sig !== "string") {
                                                    yield this.errorDetected(WS2P_ERR.AUTH_INVALID_ACK_FIELDS);
                                                }
                                                else {
                                                    if (this.expectedPub && data.pub !== this.expectedPub) {
                                                        yield this.errorDetected(WS2P_ERR.INCORRECT_PUBKEY_FOR_REMOTE);
                                                    }
                                                    else {
                                                        try {
                                                            const valid = yield this.localAuth.registerACK(data.sig, data.pub);
                                                            if (valid) {
                                                                yield this.localAuth.sendOK(this.ws);
                                                            }
                                                        }
                                                        catch (e) {
                                                            yield this.errorDetected(WS2P_ERR.INCORRECT_ACK_SIGNATURE_FROM_REMOTE);
                                                        }
                                                    }
                                                }
                                            }
                                            else if (data.auth === "OK") {
                                                if (this.remoteAuth.isAuthenticatedByRemote()) {
                                                    return this.errorDetected(WS2P_ERR.ALREADY_AUTHENTICATED_AND_CONFIRMED_BY_REMOTE);
                                                }
                                                if (typeof data.sig !== "string") {
                                                    yield this.errorDetected(WS2P_ERR.AUTH_INVALID_OK_FIELDS);
                                                }
                                                else {
                                                    yield this.remoteAuth.registerOK(data.sig);
                                                }
                                            }
                                            else {
                                                yield this.errorDetected(WS2P_ERR.UNKNOWN_AUTH_MESSAGE);
                                            }
                                        }
                                        else {
                                            if (!this.localAuth.isRemoteAuthenticated()) {
                                                yield this.errorDetected(WS2P_ERR.MUST_BE_AUTHENTICATED_FIRST);
                                            }
                                            else if (data.reqId && typeof data.reqId === "string") {
                                                try {
                                                    const answer = yield this.messageHandler.answerToRequest(data.body);
                                                    this.ws.send(JSON.stringify({ resId: data.reqId, body: answer }));
                                                }
                                                catch (e) {
                                                    this.ws.send(JSON.stringify({ resId: data.reqId, err: e }));
                                                }
                                            }
                                            else if (data.resId && typeof data.resId === "string") {
                                                // An answer
                                                const request = this.exchanges[data.resId];
                                                this.nbResponsesCount++;
                                                if (request !== undefined) {
                                                    request.extras.resolve(data.body);
                                                }
                                                else {
                                                    yield this.errorDetected(WS2P_ERR.ANSWER_TO_UNDEFINED_REQUEST);
                                                }
                                            }
                                            else {
                                                this.nbPushsByRemoteCount++;
                                                yield this.messageHandler.handlePushMessage(data, this);
                                            }
                                        }
                                    }
                                }));
                            })]);
                        this.connectedResolve(this.remoteAuth.getPubkey());
                    }
                    catch (e) {
                        this.connectedReject(e);
                        yield this.connectedp;
                    }
                }))();
            }
            return this.connectp;
        });
    }
    request(body) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.connect();
            const uuid = nuuid.v4();
            return new Promise((resolve, reject) => {
                this.nbRequestsCount++;
                this.ws.send(JSON.stringify({
                    reqId: uuid,
                    body
                }), (err) => __awaiter(this, void 0, void 0, function* () {
                    if (err) {
                        return reject(err);
                    }
                    else {
                        // The request was successfully sent. Now wait for the answer.
                        const extras = {
                            resolve: () => { console.error('resolution not implemented'); },
                            reject: () => { console.error('rejection not implemented'); }
                        };
                        this.exchanges[uuid] = {
                            extras,
                            promise: Promise.race([
                                // The answer
                                new Promise((res, rej) => {
                                    extras.resolve = res;
                                    extras.reject = () => {
                                        this.errorDetected(WS2P_ERR.REQUEST_FAILED);
                                        rej();
                                    };
                                }),
                                // Timeout
                                new Promise((res, rej) => {
                                    setTimeout(() => {
                                        rej("WS2P request timeout");
                                    }, this.options.requestTimeout);
                                })
                            ])
                        };
                        try {
                            resolve(yield this.exchanges[uuid].promise);
                        }
                        catch (e) {
                            reject(e);
                        }
                    }
                }));
            });
        });
    }
    pushBlock(block) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pushData(WS2P_PUSH.BLOCK, 'block', block);
        });
    }
    pushIdentity(idty) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pushData(WS2P_PUSH.IDENTITY, 'identity', idty);
        });
    }
    pushCertification(cert) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pushData(WS2P_PUSH.CERTIFICATION, 'certification', cert);
        });
    }
    pushMembership(ms) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pushData(WS2P_PUSH.MEMBERSHIP, 'membership', ms);
        });
    }
    pushTransaction(tx) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pushData(WS2P_PUSH.TRANSACTION, 'transaction', tx);
        });
    }
    pushPeer(peer) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pushData(WS2P_PUSH.PEER, 'peer', peer);
        });
    }
    pushHeads(heads) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pushData(WS2P_PUSH.HEAD, 'heads', heads);
        });
    }
    pushData(type, key, data) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.connect();
            return new Promise((resolve, reject) => {
                this.nbPushsToRemoteCount++;
                try {
                    this.ws.send(JSON.stringify({
                        body: {
                            name: WS2P_PUSH[type],
                            [key]: data
                        }
                    }), (err) => __awaiter(this, void 0, void 0, function* () {
                        if (err) {
                            return reject(err);
                        }
                        resolve();
                    }));
                }
                catch (e) {
                    reject(e);
                }
            });
        });
    }
    errorDetected(cause) {
        this.nbErrors++;
        Logger.error('>>> WS ERROR: %s', WS2P_ERR[cause]);
        if (this.nbErrors >= MAXIMUM_ERRORS_COUNT) {
            this.ws.terminate();
        }
    }
}
exports.WS2PConnection = WS2PConnection;
class Logger {
    static log(message) {
        logger.trace('WS2P >>> ' + message);
    }
    static error(message, obj) {
        logger.error('WS2P >>> ' + message, obj);
    }
}
//# sourceMappingURL=WS2PConnection.js.map