"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const stream = require("stream");
const logger_1 = require("../../../lib/logger");
const CertificationDTO_1 = require("../../../lib/dto/CertificationDTO");
const IdentityDTO_1 = require("../../../lib/dto/IdentityDTO");
const BlockDTO_1 = require("../../../lib/dto/BlockDTO");
const MembershipDTO_1 = require("../../../lib/dto/MembershipDTO");
const TransactionDTO_1 = require("../../../lib/dto/TransactionDTO");
const PeerDTO_1 = require("../../../lib/dto/PeerDTO");
const constants_1 = require("./constants");
const logger = logger_1.NewLogger();
class WS2PSingleWriteStream extends stream.Transform {
    constructor(protectionDuration = 1000 * constants_1.WS2PConstants.SINGLE_RECORD_PROTECTION_IN_SECONDS) {
        super({ objectMode: true });
        this.protectionDuration = protectionDuration;
        this.detections = {};
    }
    getNbProtectionsCurrently() {
        return Object.keys(this.detections).length;
    }
    _write(obj, enc, done) {
        return __awaiter(this, void 0, void 0, function* () {
            let documentHash = '';
            let doStream = false;
            try {
                if (obj.joiners) {
                    const dto = BlockDTO_1.BlockDTO.fromJSONObject(obj);
                    documentHash = dto.getHash();
                }
                else if (obj.pubkey && obj.uid) {
                    const dto = IdentityDTO_1.IdentityDTO.fromJSONObject(obj);
                    documentHash = dto.getHash();
                }
                else if (obj.idty_uid) {
                    const dto = CertificationDTO_1.CertificationDTO.fromJSONObject(obj);
                    documentHash = dto.getHash();
                }
                else if (obj.userid) {
                    const dto = MembershipDTO_1.MembershipDTO.fromJSONObject(obj);
                    documentHash = dto.getHash();
                }
                else if (obj.issuers) {
                    const dto = TransactionDTO_1.TransactionDTO.fromJSONObject(obj);
                    documentHash = dto.getHash();
                }
                else if (obj.endpoints) {
                    const dto = PeerDTO_1.PeerDTO.fromJSONObject(obj);
                    documentHash = dto.getHash();
                }
                if (documentHash) {
                    if (!this.detections[documentHash]) {
                        doStream = true;
                        this.detections[documentHash] = 1;
                    }
                    else {
                        this.detections[documentHash]++;
                        logger.warn('WS2P OUT => Document detected %s times: %s', this.detections[documentHash], JSON.stringify(obj));
                    }
                    setTimeout(() => {
                        delete this.detections[documentHash];
                    }, this.protectionDuration);
                }
                if (doStream) {
                    this.push(obj);
                }
            }
            catch (e) {
                logger.warn('WS2P >> SingleWrite >>', e);
            }
            done && done();
        });
    }
}
exports.WS2PSingleWriteStream = WS2PSingleWriteStream;
//# sourceMappingURL=WS2PSingleWriteStream.js.map