"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class WS2PReqMapperByServer {
    constructor(server) {
        this.server = server;
    }
    getCurrent() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.server.BlockchainService.current();
        });
    }
    getBlock(number) {
        return this.server.dal.getBlock(number);
    }
    getBlocks(count, from) {
        return __awaiter(this, void 0, void 0, function* () {
            if (count > 5000) {
                throw 'Count is too high';
            }
            const current = yield this.server.dal.getCurrentBlockOrNull();
            count = Math.min(current.number - from + 1, count);
            if (!current || current.number < from) {
                return [];
            }
            return this.server.dal.getBlocksBetween(from, from + count - 1);
        });
    }
    getRequirementsOfPending(minsig) {
        return __awaiter(this, void 0, void 0, function* () {
            let identities = yield this.server.dal.idtyDAL.query('SELECT i.*, count(c.sig) as nbSig ' +
                'FROM idty i, cert c ' +
                'WHERE c.target = i.hash group by i.hash having nbSig >= ?', minsig);
            const members = (yield this.server.dal.idtyDAL.query('SELECT i.*, count(c.sig) as nbSig ' +
                'FROM i_index i, cert c ' +
                'WHERE c.`to` = i.pub group by i.pub having nbSig >= ?', minsig)).map((i) => {
                return {
                    hash: i.hash || "",
                    member: i.member || false,
                    wasMember: i.wasMember || false,
                    pubkey: i.pub,
                    uid: i.uid || "",
                    buid: i.created_on || "",
                    sig: i.sig || "",
                    revocation_sig: "",
                    revoked: false,
                    revoked_on: 0
                };
            });
            identities = identities.concat(members);
            const all = yield this.server.BlockchainService.requirementsOfIdentities(identities, false);
            return {
                identities: all
            };
        });
    }
}
exports.WS2PReqMapperByServer = WS2PReqMapperByServer;
//# sourceMappingURL=WS2PReqMapperByServer.js.map