"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const WS2PReqMapperByServer_1 = require("../impl/WS2PReqMapperByServer");
const BlockDTO_1 = require("../../../../lib/dto/BlockDTO");
const IdentityDTO_1 = require("../../../../lib/dto/IdentityDTO");
const CertificationDTO_1 = require("../../../../lib/dto/CertificationDTO");
const MembershipDTO_1 = require("../../../../lib/dto/MembershipDTO");
const TransactionDTO_1 = require("../../../../lib/dto/TransactionDTO");
const PeerDTO_1 = require("../../../../lib/dto/PeerDTO");
const WS2PRequester_1 = require("../WS2PRequester");
const constants_1 = require("../constants");
const constants_2 = require("../../../../lib/common-libs/constants");
var WS2P_REQERROR;
(function (WS2P_REQERROR) {
    WS2P_REQERROR[WS2P_REQERROR["UNKNOWN_REQUEST"] = 0] = "UNKNOWN_REQUEST";
})(WS2P_REQERROR = exports.WS2P_REQERROR || (exports.WS2P_REQERROR = {}));
class WS2PServerMessageHandler {
    constructor(server, cluster) {
        this.server = server;
        this.cluster = cluster;
        this.errors = {};
        this.mapper = new WS2PReqMapperByServer_1.WS2PReqMapperByServer(server);
    }
    handlePushMessage(json, c) {
        return __awaiter(this, void 0, void 0, function* () {
            let documentHash = '';
            try {
                if (json.body) {
                    if (json.body.block) {
                        const dto = BlockDTO_1.BlockDTO.fromJSONObject(json.body.block);
                        const raw = dto.getRawSigned();
                        documentHash = dto.getHash();
                        yield this.server.writeRawBlock(raw);
                    }
                    else if (json.body.identity) {
                        const dto = IdentityDTO_1.IdentityDTO.fromJSONObject(json.body.identity);
                        const raw = dto.getRawSigned();
                        documentHash = dto.getHash();
                        yield this.server.writeRawIdentity(raw);
                    }
                    else if (json.body.certification) {
                        const dto = CertificationDTO_1.CertificationDTO.fromJSONObject(json.body.certification);
                        const raw = dto.getRawSigned();
                        documentHash = dto.getHash();
                        yield this.server.writeRawCertification(raw);
                    }
                    else if (json.body.membership) {
                        const dto = MembershipDTO_1.MembershipDTO.fromJSONObject(json.body.membership);
                        const raw = dto.getRawSigned();
                        documentHash = dto.getHash();
                        yield this.server.writeRawMembership(raw);
                    }
                    else if (json.body.transaction) {
                        const dto = TransactionDTO_1.TransactionDTO.fromJSONObject(json.body.transaction);
                        const raw = dto.getRaw();
                        documentHash = dto.getHash();
                        yield this.server.writeRawTransaction(raw);
                    }
                    else if (json.body.peer) {
                        const dto = PeerDTO_1.PeerDTO.fromJSONObject(json.body.peer);
                        const raw = dto.getRawSigned();
                        documentHash = dto.getHash();
                        yield this.server.writeRawPeer(raw);
                    }
                    else if (json.body.heads && typeof json.body.heads === "object" && json.body.heads.length !== undefined) {
                        if (!json.body.heads.length) {
                            documentHash = 'HEADs';
                            throw "Heads empty HEADs received";
                        }
                        yield this.cluster.headsReceived(json.body.heads || []);
                    }
                }
            }
            catch (e) {
                if (documentHash
                    && this.errors[documentHash]
                    && this.errors[documentHash].pubkeys[c.pubkey] !== undefined
                    && this.server.conf.pair.pub !== c.pubkey) {
                    this.errors[documentHash].pubkeys[c.pubkey].push(json.body);
                    if (this.errors[documentHash].pubkeys[c.pubkey].length >= constants_1.WS2PConstants.BAN_ON_REPEAT_THRESHOLD) {
                        let message = "peer " + (c.pubkey || '--unknown--') + " sent " + constants_1.WS2PConstants.BAN_ON_REPEAT_THRESHOLD + " times a same wrong document: " + (e && (e.message || (e.uerr && e.uerr.message)) || JSON.stringify(e));
                        this.cluster.banConnection(c, message);
                        for (const body of this.errors[documentHash].pubkeys[c.pubkey]) {
                            message += '\n => ' + JSON.stringify(body);
                        }
                    }
                    else {
                        let message = "WS2P IN => " + (c.pubkey || '--unknown--') + " sent " + this.errors[documentHash].pubkeys[c.pubkey].length + " times a same wrong document: " + (e && (e.message || (e.uerr && e.uerr.message)) || JSON.stringify(e));
                        for (const body of this.errors[documentHash].pubkeys[c.pubkey]) {
                            message += '\n => ' + JSON.stringify(body);
                        }
                        this.server.logger.warn(message);
                    }
                }
                else {
                    // Remember the error for some time
                    if (!this.errors[documentHash]) {
                        this.errors[documentHash] = {
                            createdOn: Date.now(),
                            pubkeys: {}
                        };
                    }
                    this.errors[documentHash].pubkeys[c.pubkey] = [json.body];
                    setTimeout(() => {
                        delete this.errors[documentHash];
                    }, 1000 * constants_1.WS2PConstants.ERROR_RECALL_DURATION_IN_SECONDS);
                }
                if (e !== "Block already known"
                    && (!e.uerr
                        || !(e.uerr.ucode == constants_2.CommonConstants.ERRORS.DOCUMENT_BEING_TREATED.uerr.ucode
                            || e.uerr.ucode == constants_2.CommonConstants.ERRORS.PEER_DOCUMENT_ALREADY_KNOWN.uerr.ucode))) {
                    this.server.logger.warn(e);
                }
            }
        });
    }
    answerToRequest(data) {
        return __awaiter(this, void 0, void 0, function* () {
            /**********
             * REQUEST
             *********/
            let body = {};
            if (data && data.name) {
                switch (data.name) {
                    case WS2PRequester_1.WS2P_REQ[WS2PRequester_1.WS2P_REQ.CURRENT]:
                        body = yield this.mapper.getCurrent();
                        break;
                    case WS2PRequester_1.WS2P_REQ[WS2PRequester_1.WS2P_REQ.BLOCK_BY_NUMBER]:
                        if (isNaN(data.params.number)) {
                            throw "Wrong param `number`";
                        }
                        const number = data.params.number;
                        body = yield this.mapper.getBlock(number);
                        break;
                    case WS2PRequester_1.WS2P_REQ[WS2PRequester_1.WS2P_REQ.BLOCKS_CHUNK]:
                        if (isNaN(data.params.count)) {
                            throw "Wrong param `count`";
                        }
                        if (isNaN(data.params.fromNumber)) {
                            throw "Wrong param `fromNumber`";
                        }
                        const count = data.params.count;
                        const fromNumber = data.params.fromNumber;
                        body = yield this.mapper.getBlocks(count, fromNumber);
                        break;
                    case WS2PRequester_1.WS2P_REQ[WS2PRequester_1.WS2P_REQ.WOT_REQUIREMENTS_OF_PENDING]:
                        if (isNaN(data.params.minCert)) {
                            throw "Wrong param `minCert`";
                        }
                        const minCert = data.params.minCert;
                        body = yield this.mapper.getRequirementsOfPending(minCert);
                        break;
                    default:
                        throw Error(WS2P_REQERROR[WS2P_REQERROR.UNKNOWN_REQUEST]);
                }
            }
            return body;
        });
    }
}
exports.WS2PServerMessageHandler = WS2PServerMessageHandler;
//# sourceMappingURL=WS2PServerMessageHandler.js.map