import { IdentityForRequirements } from './BlockchainService';
import { Server } from "../../server";
import { GlobalFifoPromise } from "./GlobalFifoPromise";
import { BlockchainContext } from "../lib/computation/BlockchainContext";
import { ConfDTO } from "../lib/dto/ConfDTO";
import { FileDAL } from "../lib/dal/fileDAL";
import { QuickSynchronizer } from "../lib/computation/QuickSync";
import { BlockDTO } from "../lib/dto/BlockDTO";
import { DBBlock } from "../lib/db/DBBlock";
import { HttpIdentityRequirement } from "../modules/bma/lib/dtos";
import { FIFOService } from "./FIFOService";
import { SwitcherDao } from "../lib/blockchain/Switcher";
export interface IdentityForRequirements {
    hash: string;
    member: boolean;
    wasMember: boolean;
    pubkey: string;
    uid: string;
    buid: string;
    sig: string;
    revocation_sig: string;
    revoked: boolean;
    revoked_on: number;
}
export declare class BlockchainService extends FIFOService {
    private server;
    mainContext: BlockchainContext;
    conf: ConfDTO;
    dal: FileDAL;
    logger: any;
    selfPubkey: string;
    quickSynchronizer: QuickSynchronizer;
    switcherDao: SwitcherDao<BlockDTO>;
    invalidForks: string[];
    constructor(server: Server, fifoPromiseHandler: GlobalFifoPromise);
    /**
     * Mandatory with stream.Readable
     * @private
     */
    _read(): void;
    getContext(): BlockchainContext;
    setConfDAL(newConf: ConfDTO, newDAL: FileDAL, newKeyPair: any): void;
    current(): Promise<any>;
    promoted(number: number): Promise<any>;
    checkBlock(block: any, withPoWAndSignature?: boolean): Promise<any>;
    branches(): Promise<BlockDTO[]>;
    submitBlock(blockToAdd: any, noResolution?: boolean): Promise<BlockDTO>;
    blockResolution(): Promise<void>;
    forkResolution(): Promise<void>;
    revertCurrentBlock(): Promise<BlockDTO>;
    applyNextAvailableFork(): Promise<any>;
    requirementsOfIdentities(identities: IdentityForRequirements[], computeDistance?: boolean): Promise<HttpIdentityRequirement[]>;
    requirementsOfIdentity(idty: IdentityForRequirements, current: DBBlock, computeDistance?: boolean): Promise<HttpIdentityRequirement>;
    getValidCerts(newcomer: string, newCerts: any): Promise<any>;
    isMember(): Promise<any>;
    getCountOfSelfMadePoW(): any;
    saveParametersForRootBlock(block: BlockDTO): any;
    blocksBetween(from: number, count: number): Promise<any>;
    /**
     * Allows to quickly insert a bunch of blocks. To reach such speed, this method skips global rules and buffers changes.
     *
     * **This method should be used ONLY when a node is really far away from current blockchain HEAD (i.e several hundreds of blocks late).
     *
     * This method is called by duniter-crawler 1.3.x.
     *
     * @param blocks An array of blocks to insert.
     * @param to The final block number of the fast insertion.
     */
    fastBlockInsertions(blocks: BlockDTO[], to: number): Promise<any>;
}
