"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../lib/common-libs/constants");
const logger_1 = require("../lib/logger");
const querablep = require('querablep');
const async = require('async');
const logger = logger_1.NewLogger();
class GlobalFifoPromise {
    constructor() {
        this.fifo = async.queue(function (task, callback) {
            task(callback);
        }, 1);
        this.operations = {};
    }
    /**
     * Adds a promise to a FIFO stack of promises, so the given promise will be executed against a shared FIFO stack.
     * @param operationId The ID of the operation, which indicates which task to reject if the FIFO already contains it
     * @param p
     */
    pushFIFOPromise(operationId, p) {
        // Return a promise that will be done after the fifo has executed the given promise
        return new Promise((resolve, reject) => {
            if (this.operations[operationId]) {
                throw constants_1.CommonConstants.ERRORS.DOCUMENT_BEING_TREATED;
            }
            this.operations[operationId] = true;
            // Push the promise on the stack
            this.fifo.push((cb) => __awaiter(this, void 0, void 0, function* () {
                // OK its the turn of given promise, execute it
                try {
                    this.currentPromise = querablep(p());
                    const res = yield this.currentPromise;
                    delete this.operations[operationId];
                    // Finished, we end the function in the FIFO
                    cb(null, res);
                }
                catch (e) {
                    delete this.operations[operationId];
                    // Errored, we end the function with an error
                    cb(e);
                }
            }), (err, res) => {
                // An error occured => reject promise
                if (err)
                    return reject(err);
                // Success => we resolve with given promise result
                resolve(res);
            });
        });
    }
    closeFIFO() {
        return __awaiter(this, void 0, void 0, function* () {
            this.fifo.pause();
            if (this.currentPromise && !this.currentPromise.isFulfilled()) {
                logger.info('Waiting current task of documentFIFO to be finished...');
                yield this.currentPromise;
            }
        });
    }
    remainingTasksCount() {
        return this.fifo.length();
    }
}
exports.GlobalFifoPromise = GlobalFifoPromise;
//# sourceMappingURL=GlobalFifoPromise.js.map