"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const local_rules_1 = require("../lib/rules/local_rules");
const global_rules_1 = require("../lib/rules/global_rules");
const MembershipDTO_1 = require("../lib/dto/MembershipDTO");
const FIFOService_1 = require("./FIFOService");
const constants = require('../lib/constants');
class MembershipService extends FIFOService_1.FIFOService {
    constructor(fifoPromiseHandler) {
        super(fifoPromiseHandler);
    }
    setConfDAL(newConf, newDAL) {
        this.dal = newDAL;
        this.conf = newConf;
        this.logger = require('../lib/logger').NewLogger(this.dal.profile);
    }
    current() {
        return this.dal.getCurrentBlockOrNull();
    }
    submitMembership(ms) {
        const entry = MembershipDTO_1.MembershipDTO.fromJSONObject(ms);
        const hash = entry.getHash();
        return this.pushFIFO(hash, () => __awaiter(this, void 0, void 0, function* () {
            // Force usage of local currency name, do not accept other currencies documents
            entry.currency = this.conf.currency || entry.currency;
            this.logger.info('⬇ %s %s', entry.issuer, entry.membership);
            if (!local_rules_1.LOCAL_RULES_HELPERS.checkSingleMembershipSignature(entry)) {
                throw constants.ERRORS.WRONG_SIGNATURE_MEMBERSHIP;
            }
            // Get already existing Membership with same parameters
            const mostRecentNumber = yield this.dal.getMostRecentMembershipNumberForIssuer(entry.issuer);
            const thisNumber = entry.number;
            if (mostRecentNumber == thisNumber) {
                throw constants.ERRORS.ALREADY_RECEIVED_MEMBERSHIP;
            }
            else if (mostRecentNumber > thisNumber) {
                throw constants.ERRORS.A_MORE_RECENT_MEMBERSHIP_EXISTS;
            }
            const isMember = yield this.dal.isMember(entry.issuer);
            const isJoin = entry.membership == 'IN';
            if (!isMember && !isJoin) {
                // LEAVE
                throw constants.ERRORS.MEMBERSHIP_A_NON_MEMBER_CANNOT_LEAVE;
            }
            const current = yield this.dal.getCurrentBlockOrNull();
            const basedBlock = yield global_rules_1.GLOBAL_RULES_HELPERS.checkMembershipBlock(entry, current, this.conf, this.dal);
            if (!(yield this.dal.msDAL.sandbox.acceptNewSandBoxEntry({
                pubkey: entry.pubkey,
                block_number: entry.block_number
            }, this.conf.pair && this.conf.pair.pub))) {
                throw constants.ERRORS.SANDBOX_FOR_MEMERSHIP_IS_FULL;
            }
            // Saves entry
            yield this.dal.savePendingMembership({
                membership: entry.membership,
                issuer: entry.issuer,
                number: entry.number,
                blockNumber: entry.number,
                blockHash: entry.fpr,
                userid: entry.userid,
                certts: entry.certts,
                block: entry.blockstamp,
                fpr: entry.fpr,
                idtyHash: entry.getIdtyHash(),
                written: false,
                written_number: null,
                expires_on: basedBlock ? basedBlock.medianTime + this.conf.msWindow : null,
                signature: entry.signature,
                expired: false,
                block_number: entry.number
            });
            this.logger.info('✔ %s %s', entry.issuer, entry.membership);
            return entry;
        }));
    }
}
exports.MembershipService = MembershipService;
//# sourceMappingURL=MembershipService.js.map