"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const TransactionDTO_1 = require("../lib/dto/TransactionDTO");
const local_rules_1 = require("../lib/rules/local_rules");
const global_rules_1 = require("../lib/rules/global_rules");
const TxsDAL_1 = require("../lib/dal/sqliteDAL/TxsDAL");
const FIFOService_1 = require("./FIFOService");
const constants = require('../lib/constants');
const CHECK_PENDING_TRANSACTIONS = true;
class TransactionService extends FIFOService_1.FIFOService {
    constructor(fifoPromiseHandler) {
        super(fifoPromiseHandler);
    }
    setConfDAL(newConf, newDAL) {
        this.dal = newDAL;
        this.conf = newConf;
        this.logger = require('../lib/logger').NewLogger(this.dal.profile);
    }
    processTx(txObj) {
        const tx = TransactionDTO_1.TransactionDTO.fromJSONObject(txObj, this.conf.currency);
        const hash = tx.getHash();
        return this.pushFIFO(hash, () => __awaiter(this, void 0, void 0, function* () {
            try {
                this.logger.info('⬇ TX %s:%s from %s', tx.output_amount, tx.output_base, tx.issuers);
                const existing = yield this.dal.getTxByHash(tx.hash);
                const current = yield this.dal.getCurrentBlockOrNull();
                if (existing) {
                    throw constants.ERRORS.TX_ALREADY_PROCESSED;
                }
                // Start checks...
                const nextBlockWithFakeTimeVariation = { medianTime: current.medianTime + 1 };
                const dto = TransactionDTO_1.TransactionDTO.fromJSONObject(tx);
                yield local_rules_1.LOCAL_RULES_HELPERS.checkSingleTransactionLocally(dto);
                yield global_rules_1.GLOBAL_RULES_HELPERS.checkTxBlockStamp(tx, this.dal);
                yield global_rules_1.GLOBAL_RULES_HELPERS.checkSingleTransaction(dto, nextBlockWithFakeTimeVariation, this.conf, this.dal, CHECK_PENDING_TRANSACTIONS);
                const server_pubkey = this.conf.pair && this.conf.pair.pub;
                if (!(yield this.dal.txsDAL.sandbox.acceptNewSandBoxEntry({
                    pubkey: tx.issuers.indexOf(server_pubkey) !== -1 ? server_pubkey : '',
                    output_base: tx.output_base,
                    output_amount: tx.output_amount
                }, server_pubkey))) {
                    throw constants.ERRORS.SANDBOX_FOR_TRANSACTION_IS_FULL;
                }
                yield this.dal.saveTransaction(TxsDAL_1.DBTx.fromTransactionDTO(tx));
                this.logger.info('✔ TX %s:%s from %s', tx.output_amount, tx.output_base, tx.issuers);
                return tx;
            }
            catch (e) {
                this.logger.info('✘ TX %s:%s from %s', tx.output_amount, tx.output_base, tx.issuers);
                throw e;
            }
        }));
    }
}
exports.TransactionService = TransactionService;
//# sourceMappingURL=TransactionsService.js.map