"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require('underscore');
class MemoryIndex {
    constructor() {
        // The blockchain storage
        this.indexStorage = {};
    }
    initIndexer(pkFields) {
        return Promise.resolve();
    }
    getSubIndexes() {
        return Promise.resolve(_.keys(this.indexStorage));
    }
    findTrimable(subIndex, numberField, maxNumber) {
        const criterias = {};
        criterias[numberField] = { $lt: maxNumber };
        return this.findWhere(subIndex, criterias);
    }
    removeWhere(subIndex, criterias) {
        let i = 0;
        let rows = this.indexStorage[subIndex];
        while (i < rows.length) {
            if (MemoryIndex.matchComplexCriterias(criterias, rows[i])) {
                rows.splice(i, 1);
            }
            else {
                i++;
            }
        }
        return Promise.resolve();
    }
    recordIndex(index) {
        const subIndexes = _.keys(index);
        // Create subIndexes if they do not exist
        for (const subIndex of subIndexes) {
            this.indexStorage[subIndex] = this.indexStorage[subIndex] || [];
        }
        // Feed the subIndexes
        for (const subIndex of subIndexes) {
            this.indexStorage[subIndex] = this.indexStorage[subIndex].concat(index[subIndex]);
        }
        return Promise.resolve();
    }
    static matchComplexCriterias(criterias, row) {
        const criteriaKeys = _.keys(criterias);
        let matches = true;
        let i = 0;
        while (matches && i < criteriaKeys.length) {
            const k = criteriaKeys[i];
            if (typeof criterias[k] === 'function') {
                matches = criterias[k](row[k]);
            }
            else if (typeof criterias[k] === 'object') {
                if (criterias[k].$lt) {
                    matches = row[k] < criterias[k].$lt;
                }
                else if (criterias[k].$gt) {
                    matches = row[k] > criterias[k].$gt;
                }
                else if (criterias[k].$lte) {
                    matches = row[k] <= criterias[k].$lte;
                }
                else if (criterias[k].$gte) {
                    matches = row[k] >= criterias[k].$gte;
                }
                else {
                    // Unknown predicate
                    matches = false;
                }
            }
            else {
                matches = row[k] === criterias[k];
            }
            i++;
        }
        return matches;
    }
    findWhere(subIndex, criterias) {
        let res = [];
        const areBasicCriterias = _.values(criterias).reduce((are, criteria) => are && typeof criteria !== 'function' && typeof criteria !== 'object', true);
        if (areBasicCriterias) {
            res = _.where(this.indexStorage[subIndex], criterias);
        }
        else {
            // Slower test, with specific criterias
            for (const row of this.indexStorage[subIndex]) {
                if (MemoryIndex.matchComplexCriterias(criterias, row)) {
                    res.push(row);
                }
            }
        }
        return Promise.resolve(res);
    }
}
exports.MemoryIndex = MemoryIndex;
//# sourceMappingURL=MemoryIndex.js.map