"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const toolbox_1 = require("./tools/toolbox");
const _ = require('underscore');
const user = require('./tools/user');
describe("Revoked pubkey replay", function () {
    const now = 1500000000;
    const DONT_WAIT_FOR_BLOCKCHAIN_CHANGE = true;
    let s1, cat, tic;
    const conf = { nbCores: 1, sigQty: 1 };
    before(() => __awaiter(this, void 0, void 0, function* () {
        const res1 = yield toolbox_1.simpleNodeWith2Users(conf);
        s1 = res1.s1;
        cat = res1.cat;
        tic = user('tic', { pub: 'DNann1Lh55eZMEDXeYt59bzHbA3NJR46DeQYCS2qQdLV', sec: '468Q1XtTq7h84NorZdWBZFJrGkB18CbmbHr9tkp9snt5GiERP7ySs3wM8myLccbAAGejgMRC9rqnXuW3iAfZACm7' }, { server: s1 });
        yield s1.commit({ time: now });
        yield s1.commit({ time: now });
        // Create the tested identity « tic »
        yield tic.createIdentity();
    }));
    it('should exist tic as pending identity', () => s1.expect('/wot/lookup/tic', (res) => {
        res.should.have.property('results').length(1);
        res.results[0].should.have.property('uids').length(1);
        res.results[0].uids[0].should.have.property('uid').equal('tic');
    }));
    it('should be able to make tic become a member', () => __awaiter(this, void 0, void 0, function* () {
        yield tic.join();
        yield cat.cert(tic);
        yield s1.commit();
        yield s1.expect('/wot/members', (res) => {
            res.should.have.property('results').length(3);
            const ticEntries = _.filter(res.results, (entry) => entry.uid === 'tic');
            ticEntries.should.have.length(1);
        });
    }));
    it('should be able to revoke tic', () => __awaiter(this, void 0, void 0, function* () {
        yield tic.revoke();
        yield s1.expect('/wot/lookup/tic', (res) => {
            res.should.have.property('results').length(1);
            res.results[0].should.have.property('uids').length(1);
            res.results[0].uids[0].should.have.property('uid').equal('tic');
            res.results[0].uids[0].should.have.property('revocation_sig').not.equal(null);
        });
        yield s1.commit();
        yield s1.expect('/wot/members', (res) => {
            res.should.have.property('results').length(2);
            const ticEntries = _.filter(res.results, (entry) => entry.uid === 'tic');
            ticEntries.should.have.length(0);
        });
    }));
    it('should not try to include tic2 in a new block', () => __awaiter(this, void 0, void 0, function* () {
        yield s1.commit();
        yield tic.join();
        const block = yield s1.commit(null, DONT_WAIT_FOR_BLOCKCHAIN_CHANGE);
        block.should.have.property('joiners').length(0);
    }));
    after(() => __awaiter(this, void 0, void 0, function* () {
        yield s1.closeCluster();
    }));
});
//# sourceMappingURL=revoked_pubkey_replay.js.map