"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const toolbox_1 = require("./tools/toolbox");
const common_1 = require("../../app/lib/common");
const buid_1 = require("../../app/lib/common-libs/buid");
describe("Transaction: CSV+CLTV+1of2SIG", function () {
    const now = 1500000000;
    const DONT_WAIT_FOR_BLOCKCHAIN_CHANGE = true;
    let s1;
    let cat;
    let tac;
    let txHash;
    const conf = {
        nbCores: 1,
        sigQty: 1,
        udTime0: now,
        medianTimeBlocks: 1 // MedianTime(t) = Time(t-1)
    };
    before(() => __awaiter(this, void 0, void 0, function* () {
        const res1 = yield toolbox_1.simpleNodeWith2Users(conf);
        s1 = res1.s1;
        cat = res1.cat;
        tac = res1.tac;
        yield s1.commit({ time: now });
        yield s1.commit({ time: now });
    }));
    it('cat should have 100 units', () => __awaiter(this, void 0, void 0, function* () {
        const sources = yield s1.get('/tx/sources/' + cat.pub);
        sources.should.have.property('sources').length(1);
    }));
    it('cat should be able to make a CSV+CLTV+1of2SIG tx', () => __awaiter(this, void 0, void 0, function* () {
        const current = yield s1.dal.getCurrentBlockOrNull();
        const tx = yield cat.makeTX([{
                src: '100:0:D:HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd:1',
                unlock: 'SIG(0)'
            }], [{
                qty: 100,
                base: 0,
                lock: '(SIG(' + cat.pub + ') || SIG(' + tac.pub + ')) && (CSV(10) || CLTV(' + now + '))'
            }], {
            blockstamp: buid_1.Buid.format.buid(current)
        });
        txHash = common_1.hashf(tx);
        yield cat.sendTX(tx);
        const block = yield s1.commit({ time: now }, DONT_WAIT_FOR_BLOCKCHAIN_CHANGE);
        block.should.have.property('transactions').length(1);
        yield s1.waitToHaveBlock(2);
    }));
    it('tac should be able to consume the tx after 10s', () => __awaiter(this, void 0, void 0, function* () {
        yield s1.commit({ time: now + 10 });
        yield s1.commit({ time: now + 10 });
        const current = yield s1.dal.getCurrentBlockOrNull();
        const tx = yield tac.makeTX([{
                src: '100:0:T:' + txHash + ':0',
                unlock: 'SIG(0)'
            }], [{
                qty: 100,
                base: 0,
                lock: 'SIG(' + tac.pub + ')'
            }], {
            blockstamp: buid_1.Buid.format.buid(current)
        });
        yield tac.sendTX(tx);
        const block = yield s1.commit(null, DONT_WAIT_FOR_BLOCKCHAIN_CHANGE);
        block.should.have.property('transactions').length(1);
    }));
    after(() => __awaiter(this, void 0, void 0, function* () {
        yield s1.closeCluster();
    }));
});
//# sourceMappingURL=transactions-csv-cltv-sig.js.map