"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const toolbox_1 = require("./tools/toolbox");
const constants_1 = require("../../app/modules/ws2p/lib/constants");
const assert = require('assert');
describe("WS2P doc sharing", function () {
    constants_1.WS2PConstants.CONNEXION_TIMEOUT = 100;
    constants_1.WS2PConstants.REQUEST_TIMEOUT = 100;
    const now = 1500000000;
    let s1, s2, wss;
    let cat, tac, toc;
    const catKeyring = { pub: 'HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd', sec: '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP' };
    const tacKeyring = { pub: '2LvDg21dVXvetTD9GdkPLURavLYEqP3whauvPWX4c2qc', sec: '2HuRLWgKgED1bVio1tdpeXrf7zuUszv1yPHDsDj7kcMC4rVSN9RC58ogjtKNfTbH1eFz7rn38U1PywNs3m6Q7UxE' };
    const tocKeyring = { pub: 'DKpQPUL4ckzXYdnDRvCRKAm1gNvSdmAXnTrJZ7LvM5Qo', sec: '64EYRvdPpTfLGGmaX5nijLXRqWXaVz8r1Z1GtaahXwVSJGQRn7tqkxLb288zwSYzELMEG5ZhXSBYSxsTsz1m9y8F' };
    before(() => __awaiter(this, void 0, void 0, function* () {
        const conf1 = toolbox_1.simpleTestingConf(now, catKeyring);
        const conf2 = toolbox_1.simpleTestingConf(now, tacKeyring);
        s1 = toolbox_1.simpleTestingServer(conf1);
        s2 = toolbox_1.simpleTestingServer(conf2);
        cat = toolbox_1.simpleUser('cat', catKeyring, s1);
        tac = toolbox_1.simpleUser('tac', tacKeyring, s1);
        toc = toolbox_1.simpleUser('toc', tocKeyring, s1);
        yield s1.initDalBmaConnections();
        yield s2.initDalBmaConnections();
        const network = yield toolbox_1.simpleWS2PNetwork(s1, s2);
        yield cat.createIdentity();
        yield tac.createIdentity();
        yield cat.cert(tac);
        yield tac.cert(cat);
        yield cat.join();
        yield tac.join();
        wss = network.wss;
    }));
    after(() => wss.close());
    it('should see the identity and certs of initial members in the docpool', () => __awaiter(this, void 0, void 0, function* () {
        yield s2.expect('/wot/lookup/cat', (res) => {
            assert.equal(res.results.length, 1);
            assert.equal(res.results[0].uids[0].others.length, 1);
        });
        yield s2.expect('/wot/lookup/tac', (res) => {
            assert.equal(res.results.length, 1);
            assert.equal(res.results[0].uids[0].others.length, 1);
        });
    }));
    it('should have the same block#0 if we commit', () => __awaiter(this, void 0, void 0, function* () {
        yield s1.commit({ time: now });
        yield s1.commit({ time: now });
        yield s1.waitToHaveBlock(1);
        yield s2.waitToHaveBlock(1);
        const b1s1 = yield s1.BlockchainService.current();
        const b1s2 = yield s2.BlockchainService.current();
        assert.equal(b1s1.number, 1);
        assert.equal(b1s2.number, 1);
        assert.equal(b1s1.hash, b1s2.hash);
    }));
    it('should see the identity, certs and memberships in the docpool', () => __awaiter(this, void 0, void 0, function* () {
        yield toc.createIdentity();
        yield cat.cert(toc);
        yield toc.join();
        yield s2.expect('/wot/lookup/toc', (res) => {
            assert.equal(res.results.length, 1);
            assert.equal(res.results[0].uids[0].others.length, 1);
        });
        yield s2.commit({ time: now });
        yield s1.waitToHaveBlock(2);
        yield s2.waitToHaveBlock(2);
        const b2s1 = yield s1.BlockchainService.current();
        const b2s2 = yield s2.BlockchainService.current();
        assert.equal(b2s1.number, 2);
        assert.equal(b2s2.number, 2);
        assert.equal(b2s1.hash, b2s2.hash);
        assert.equal(b2s2.joiners.length, 1);
    }));
    it('should see the transactions pending', () => __awaiter(this, void 0, void 0, function* () {
        yield cat.sendMoney(54, toc);
        yield s2.until('transaction', 1);
        yield s2.expect('/tx/history/' + catKeyring.pub, (res) => {
            assert.equal(res.history.sending.length, 1);
        });
        yield s2.expect('/tx/history/' + tocKeyring.pub, (res) => {
            assert.equal(res.history.pending.length, 1);
        });
        yield s2.commit({ time: now });
        yield s1.waitToHaveBlock(3);
        yield s2.waitToHaveBlock(3);
        const b3s1 = yield s1.BlockchainService.current();
        const b3s2 = yield s2.BlockchainService.current();
        assert.equal(b3s1.number, 3);
        assert.equal(b3s2.number, 3);
        assert.equal(b3s1.hash, b3s2.hash);
        assert.equal(b3s2.transactions.length, 1);
    }));
    it('should see the peer documents', () => __awaiter(this, void 0, void 0, function* () {
        yield s1.getPeer();
        yield s2.until('peer', 1);
        yield s2.expect('/network/peers', (res) => {
            assert.equal(res.peers.length, 1);
        });
    }));
});
//# sourceMappingURL=ws2p_doc_sharing.js.map