"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const keyring_1 = require("../../app/lib/common-libs/crypto/keyring");
const toolbox_1 = require("./tools/toolbox");
const WS2PRequester_1 = require("../../app/modules/ws2p/lib/WS2PRequester");
const BlockDTO_1 = require("../../app/lib/dto/BlockDTO");
const constants_1 = require("../../app/modules/ws2p/lib/constants");
const assert = require('assert');
describe('WS2P exchange', () => {
    constants_1.WS2PConstants.CONNEXION_TIMEOUT = 100;
    constants_1.WS2PConstants.REQUEST_TIMEOUT = 100;
    let wss;
    let c1, s1;
    before(() => __awaiter(this, void 0, void 0, function* () {
        const serverPair = new keyring_1.Key('DKpQPUL4ckzXYdnDRvCRKAm1gNvSdmAXnTrJZ7LvM5Qo', '64EYRvdPpTfLGGmaX5nijLXRqWXaVz8r1Z1GtaahXwVSJGQRn7tqkxLb288zwSYzELMEG5ZhXSBYSxsTsz1m9y8F');
        const clientPair = new keyring_1.Key('HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd', '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP');
        const res = yield toolbox_1.newWS2PBidirectionnalConnection("gtest", serverPair, clientPair, new (class TestingHandler {
            handlePushMessage(json) {
                return __awaiter(this, void 0, void 0, function* () {
                });
            }
            answerToRequest(json) {
                return __awaiter(this, void 0, void 0, function* () {
                    return BlockDTO_1.BlockDTO.fromJSONObject({ number: 1, hash: 'A' });
                });
            }
        }));
        s1 = res.p1;
        c1 = res.p2;
        wss = res.wss;
    }));
    after((done) => {
        wss.close(done);
    });
    it('should accept the connection if everything is OK on both side', () => __awaiter(this, void 0, void 0, function* () {
        const requester1 = WS2PRequester_1.WS2PRequester.fromConnection(c1);
        assert.deepEqual(yield requester1.getCurrent(), {
            "actives": [],
            "certifications": [],
            "currency": "",
            "dividend": null,
            "excluded": [],
            "fork": false,
            "hash": "A",
            "identities": [],
            "issuer": "",
            "issuersCount": null,
            "issuersFrame": null,
            "issuersFrameVar": null,
            "joiners": [],
            "leavers": [],
            "medianTime": null,
            "membersCount": null,
            "monetaryMass": 0,
            "nonce": null,
            "number": 1,
            "parameters": "",
            "powMin": null,
            "revoked": [],
            "signature": "",
            "time": null,
            "transactions": [],
            "unitbase": null,
            "version": 10
        });
    }));
});
//# sourceMappingURL=ws2p_exchange.js.map