"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const toolbox_1 = require("./tools/toolbox");
const WS2PCluster_1 = require("../../app/modules/ws2p/lib/WS2PCluster");
const constants_1 = require("../../app/modules/ws2p/lib/constants");
const assert = require('assert');
const should = require('should');
describe("WS2P heads propagation", function () {
    constants_1.WS2PConstants.CONNEXION_TIMEOUT = 100;
    constants_1.WS2PConstants.REQUEST_TIMEOUT = 100;
    const now = 1500000000;
    let s1, s2;
    let cluster1, cluster2;
    let cat, tac;
    const catKeyring = { pub: 'HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd', sec: '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP' };
    const tacKeyring = { pub: '2LvDg21dVXvetTD9GdkPLURavLYEqP3whauvPWX4c2qc', sec: '2HuRLWgKgED1bVio1tdpeXrf7zuUszv1yPHDsDj7kcMC4rVSN9RC58ogjtKNfTbH1eFz7rn38U1PywNs3m6Q7UxE' };
    let b0, b1, b2, b3, b4, portBMA1, portWS1, portWS2;
    before(() => __awaiter(this, void 0, void 0, function* () {
        const t1 = getTestingServer(catKeyring);
        const t2 = getTestingServer(tacKeyring);
        s1 = t1.server;
        s2 = t2.server;
        portWS1 = t1.portWS;
        portWS2 = t2.portWS;
        portBMA1 = t1.portBMA;
        cat = toolbox_1.simpleUser('cat', catKeyring, s1);
        tac = toolbox_1.simpleUser('tac', tacKeyring, s1);
        yield s1.initDalBmaConnections();
        yield s2.initDalBmaConnections();
        yield cat.createIdentity();
        yield tac.createIdentity();
        yield cat.cert(tac);
        yield tac.cert(cat);
        yield cat.join();
        yield tac.join();
        b0 = yield s1.commit({ time: now });
        b1 = yield s1.commit({ time: now });
        b2 = yield s1.commit({ time: now });
        yield s2.writeBlock(b0);
        yield s2.writeBlock(b1);
        yield s2.writeBlock(b2);
        yield s1.waitToHaveBlock(2);
        yield s2.waitToHaveBlock(2);
        cluster1 = WS2PCluster_1.WS2PCluster.plugOn(s1._server);
        cluster2 = WS2PCluster_1.WS2PCluster.plugOn(s2._server);
        yield s1._server.ws2pCluster.listen('127.0.0.1', portWS1);
        yield s2._server.ws2pCluster.listen('127.0.0.1', portWS2);
    }));
    after(() => s1._server.ws2pCluster.close());
    it('should have b#2 on s1, s2 and s3', () => __awaiter(this, void 0, void 0, function* () {
        const currentS1 = yield s1.BlockchainService.current();
        const currentS2 = yield s2.BlockchainService.current();
        assert.equal(currentS1.number, 2);
        assert.equal(currentS2.number, 2);
    }));
    it('should be able to have a connected network on s2 start', () => __awaiter(this, void 0, void 0, function* () {
        const p1 = yield s1.getPeer();
        assert.deepEqual(p1.endpoints, [
            'BASIC_MERKLED_API 127.0.0.1 ' + portBMA1,
            'WS2P 11111111 127.0.0.1 ' + portWS1
        ]);
        yield s2.writePeer(p1);
        yield s1._server.ws2pCluster.startCrawling(true);
        yield s2._server.ws2pCluster.startCrawling(true);
        yield s1.expect('/network/ws2p/info', (res) => {
            assert.equal(res.peers.level1, 0);
            assert.equal(res.peers.level2, 1);
        });
        yield s2.expect('/network/ws2p/info', (res) => {
            assert.equal(res.peers.level1, 1);
            assert.equal(res.peers.level2, 0);
        });
    }));
    it('should be able to receive HEADs', () => __awaiter(this, void 0, void 0, function* () {
        b3 = s1.commit({ time: now });
        yield Promise.all([
            s2.waitToHaveBlock(3),
            s2.waitForHeads(1)
        ]);
        yield s1.expect('/network/ws2p/info', (res) => {
            assert.equal(res.peers.level1, 0);
            assert.equal(res.peers.level2, 1);
        });
        yield s2.expect('/network/ws2p/info', (res) => {
            assert.equal(res.peers.level1, 1);
            assert.equal(res.peers.level2, 0);
        });
        yield s2.expect('/network/ws2p/heads', (res) => {
            res.heads.length.should.be.greaterThan(1);
        });
    }));
    function getTestingServer(keyring) {
        const conf1 = toolbox_1.simpleTestingConf(now, keyring);
        const portBMA = toolbox_1.getNewTestingPort();
        const portWS = toolbox_1.getNewTestingPort();
        conf1.host = '127.0.0.1';
        conf1.port = portBMA;
        // A server
        conf1.ws2p = {
            upnp: false,
            uuid: '11111111',
            host: '127.0.0.1',
            port: portWS,
            remotehost: '127.0.0.1',
            remoteport: portWS,
            privilegedNodes: []
        };
        const server = toolbox_1.simpleTestingServer(conf1);
        server._server.addEndpointsDefinitions(() => __awaiter(this, void 0, void 0, function* () { return 'WS2P 11111111 127.0.0.1 ' + portWS; }));
        return { server, portWS, portBMA };
    }
});
//# sourceMappingURL=ws2p_heads.js.map