"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const toolbox_1 = require("./tools/toolbox");
const WS2PCluster_1 = require("../../app/modules/ws2p/lib/WS2PCluster");
const constants_1 = require("../../app/modules/ws2p/lib/constants");
const assert = require('assert');
describe("WS2P network", function () {
    constants_1.WS2PConstants.CONNEXION_TIMEOUT = 100;
    constants_1.WS2PConstants.REQUEST_TIMEOUT = 100;
    const now = 1500000000;
    let s1, s2;
    let cat, tac, toc;
    const catKeyring = { pub: 'HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd', sec: '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP' };
    const tacKeyring = { pub: '2LvDg21dVXvetTD9GdkPLURavLYEqP3whauvPWX4c2qc', sec: '2HuRLWgKgED1bVio1tdpeXrf7zuUszv1yPHDsDj7kcMC4rVSN9RC58ogjtKNfTbH1eFz7rn38U1PywNs3m6Q7UxE' };
    const tocKeyring = { pub: 'DKpQPUL4ckzXYdnDRvCRKAm1gNvSdmAXnTrJZ7LvM5Qo', sec: '64EYRvdPpTfLGGmaX5nijLXRqWXaVz8r1Z1GtaahXwVSJGQRn7tqkxLb288zwSYzELMEG5ZhXSBYSxsTsz1m9y8F' };
    let b0, b1, b2, portBMA1, portWS1;
    before(() => __awaiter(this, void 0, void 0, function* () {
        const conf1 = toolbox_1.simpleTestingConf(now, catKeyring);
        const conf2 = toolbox_1.simpleTestingConf(now, tacKeyring);
        portBMA1 = toolbox_1.getNewTestingPort();
        portWS1 = toolbox_1.getNewTestingPort();
        conf1.host = '127.0.0.1';
        conf1.port = portBMA1;
        // A server
        conf1.ws2p = {
            upnp: false,
            uuid: '11111111',
            host: '127.0.0.1',
            port: portWS1,
            remotehost: '127.0.0.1',
            remoteport: portWS1
        };
        // A client
        conf2.ws2p = {
            upnp: false,
            uuid: '22222222'
        };
        s1 = toolbox_1.simpleTestingServer(conf1);
        s2 = toolbox_1.simpleTestingServer(conf2);
        s1._server.addEndpointsDefinitions(() => __awaiter(this, void 0, void 0, function* () { return 'WS2P 11111111 127.0.0.1 ' + portWS1; }));
        cat = toolbox_1.simpleUser('cat', catKeyring, s1);
        tac = toolbox_1.simpleUser('tac', tacKeyring, s1);
        toc = toolbox_1.simpleUser('toc', tocKeyring, s1);
        yield s1.initDalBmaConnections();
        yield s2.initDalBmaConnections();
        yield cat.createIdentity();
        yield tac.createIdentity();
        yield cat.cert(tac);
        yield tac.cert(cat);
        yield cat.join();
        yield tac.join();
        b0 = yield s1.commit({ time: now });
        b1 = yield s1.commit({ time: now });
        b2 = yield s1.commit({ time: now });
        yield s2.writeBlock(b0);
        yield s2.writeBlock(b1);
        yield s2.writeBlock(b2);
        yield s2.waitToHaveBlock(2);
        WS2PCluster_1.WS2PCluster.plugOn(s1._server);
        yield s1._server.ws2pCluster.listen('127.0.0.1', portWS1);
    }));
    after(() => s1._server.ws2pCluster.close());
    it('should have b#2 on s1 and s2', () => __awaiter(this, void 0, void 0, function* () {
        const currentS1 = yield s1.BlockchainService.current();
        const currentS2 = yield s2.BlockchainService.current();
        assert.equal(currentS1.number, 2);
        assert.equal(currentS2.number, 2);
    }));
    it('should be able to have a connected network on s2 start', () => __awaiter(this, void 0, void 0, function* () {
        const p1 = yield s1.getPeer();
        assert.deepEqual(p1.endpoints, [
            'BASIC_MERKLED_API 127.0.0.1 ' + portBMA1,
            'WS2P 11111111 127.0.0.1 ' + portWS1
        ]);
        yield s2.writePeer(p1);
        WS2PCluster_1.WS2PCluster.plugOn(s2._server);
        yield s2._server.ws2pCluster.startCrawling(true);
        // const network = await simpleWS2PNetwork(s1, s2)
        // wss = network.wss
        // cluster2 = network.cluster2
        // ProverDependency.duniter.methods.hookServer(s1._server)
        // await cluster2.pullDocpool()
        yield s1.expect('/network/ws2p/info', (res) => {
            assert.equal(res.peers.level1, 0);
            assert.equal(res.peers.level2, 1);
        });
        yield s2.expect('/network/ws2p/info', (res) => {
            assert.equal(res.peers.level1, 1);
            assert.equal(res.peers.level2, 0);
        });
        // const currentS1 = await s1.BlockchainService.current()
        // const currentS2 = await s2.BlockchainService.current()
        // assert.equal(currentS1.number, 2)
        // assert.equal(currentS2.number, 2)
    }));
});
//# sourceMappingURL=ws2p_network.js.map