"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const toolbox_1 = require("./tools/toolbox");
const constants_1 = require("../../app/modules/ws2p/lib/constants");
const assert = require('assert');
describe("WS2P block pulling", function () {
    constants_1.WS2PConstants.CONNEXION_TIMEOUT = 100;
    constants_1.WS2PConstants.REQUEST_TIMEOUT = 100;
    const now = 1500000000;
    let s1, s2, wss;
    let cluster2;
    let cat, tac;
    const catKeyring = { pub: 'HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd', sec: '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP' };
    const tacKeyring = { pub: '2LvDg21dVXvetTD9GdkPLURavLYEqP3whauvPWX4c2qc', sec: '2HuRLWgKgED1bVio1tdpeXrf7zuUszv1yPHDsDj7kcMC4rVSN9RC58ogjtKNfTbH1eFz7rn38U1PywNs3m6Q7UxE' };
    let b0, b1, b2;
    before(() => __awaiter(this, void 0, void 0, function* () {
        const conf1 = toolbox_1.simpleTestingConf(now, catKeyring);
        const conf2 = toolbox_1.simpleTestingConf(now, tacKeyring);
        s1 = toolbox_1.simpleTestingServer(conf1);
        s2 = toolbox_1.simpleTestingServer(conf2);
        cat = toolbox_1.simpleUser('cat', catKeyring, s1);
        tac = toolbox_1.simpleUser('tac', tacKeyring, s1);
        yield s1.initDalBmaConnections();
        yield s2.initDalBmaConnections();
        yield cat.createIdentity();
        yield tac.createIdentity();
        yield cat.cert(tac);
        yield tac.cert(cat);
        yield cat.join();
        yield tac.join();
        b0 = yield s1.commit({ time: now });
        b1 = yield s1.commit({ time: now });
        b2 = yield s1.commit({ time: now });
        yield s1.commit({ time: now });
        yield s1.commit({ time: now });
        yield s1.commit({ time: now });
        yield s1.commit({ time: now }); // b6
        yield s2.writeBlock(b0);
        yield s2.writeBlock(b1);
        yield s2.writeBlock(b2);
        yield s2.waitToHaveBlock(2);
        const network = yield toolbox_1.simpleWS2PNetwork(s1, s2);
        wss = network.wss;
        cluster2 = network.cluster2;
    }));
    after(() => wss.close());
    it('should have b#6 on s1, b#2 on s2', () => __awaiter(this, void 0, void 0, function* () {
        const currentS1 = yield s1.BlockchainService.current();
        const currentS2 = yield s2.BlockchainService.current();
        assert.equal(currentS1.number, 6);
        assert.equal(currentS2.number, 2);
    }));
    it('should be able to pull and have the same current block as a result', () => __awaiter(this, void 0, void 0, function* () {
        yield cluster2.pullBlocks();
        const currentS1 = yield s1.BlockchainService.current();
        const currentS2 = yield s2.BlockchainService.current();
        assert.equal(currentS1.number, 6);
        assert.equal(currentS2.number, 6);
    }));
});
//# sourceMappingURL=ws2p_pulling.js.map